#!/usr/bin/python
# -*- coding: iso-8859-1 -*-

#_______________________________________________________________
# ACTOS
# Asterisk Configuration Tool Open Source
# (c) EOLE - Mai 2004 - Pierre Derrier
#_______________________________________________________________
#    This program is free software; you can redistribute it and/or modify
#    it under the terms of the GNU General Public License as published by
#    the Free Software Foundation; either version 2 of the License, or
#    any later version.
#
#    This program is distributed in the hope that it will be useful,
#    but WITHOUT ANY WARRANTY; without even the implied warranty of
#    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#    GNU General Public License for more details.
#
#   You should have received a copy of the GNU General Public License
#    along with this program; if not, write to the Free Software
#    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
#____________________________________________________________________________
# COM.PY : PROVIDE SIMPLE FTP CLIENT/SERVER SERVICES FOR ACTOS
#___________________________________________________________________________
# Contains :
#___________________________________________________________________________

#_______________________________________________________________
import sys,traceback
from os import listdir,remove
from Global import GLOBAL
import Filesystem
import Actoscrypt as crypt

try:
    import xmlrpclib
except:
    print "Can't import client/server services. Make sure you have Python 2.3 or higher"
    sys.exit(1)
#_______________________________________________________________





#_______________________________________________________________
class ACTOS_Client:
#_______________________________________________________________
    
    #_______________________________________________________________
    def __init__(self,host="localhost",port=8007,passwd="adminpass",remotedir="/etc/asterisk/"):
    #_______________________________________________________________
        self.server=None
        self.server_params=(host,port,passwd,remotedir)
        self.filelist=[]
    #_______________________________________________________________
        
 
    #_______________________________________________________________
    def crypt(self,mot):
        return crypt.crypt(mot,"be")
    #_______________________________________________________________


    #_______________________________________________________________
    def run(self,host=None,port=None,mode="SEND"):
    #_______________________________________________________________

        if host==None:
            host=self.server_params[0]
        if port==None:
            port=self.server_params[1]

        password=self.crypt(self.server_params[2])
        print "Pass :"+password
        print "Path:"+crypt.crypt(self.server_params[3].strip(),"be")
        if password==False:
            return "Invalid password : only alphanumerics characters are valids"
                                
        self.server=xmlrpclib.Server("http://"+host+":"+str(port))

        #test connection
        try:
            result=self.server.ping()
        except:
            print "Connection to http://"+host+":"+str(port)+" failed"
            return "Can't connect to host !"

        if result!="PONG":
            print "Connection to http://"+host+":"+str(port)+" failed"
            return "Bad response from host !"

        print "Connection to http://"+host+":"+str(port)+" OK"
               
        #identify user
        result=self.server.identify(password,self.server_params[3])
        if result!="OK":
            print "Identification failed !"
            return "Authorization refused by server !"

        print "Authorized to tranfer files"
        
        if mode=="SEND":
            print "---- EXPORT TEMPORARY FILES TO "+GLOBAL.PREFIXDIR+" ---"
            saveconfigdir=GLOBAL.GLOBAL_PARAMS["configdir"]
            GLOBAL.GLOBAL_PARAMS["configdir"]=GLOBAL.PREFIXDIR+"tmp_"
            res=Filesystem.save_asterisk_config(False)
            if not res:
                print "!! Can't export config, process stopped"
                GLOBAL.GLOBAL_PARAMS["configdir"]=saveconfigdir
                return
        
            directory=GLOBAL.GLOBAL_PARAMS["configdir"].rstrip("tmp_")
            GLOBAL.GLOBAL_PARAMS["configdir"]=saveconfigdir

            #create file list
            list=listdir(directory)
            self.filelist=[]
            for file in list:
                if file.find(".conf")>-1 and file.find("tmp_")>-1:
                    self.filelist.append(directory+file)

            
            #transfer files
            print "---- BEGINNING OF TRANSFER (SEND) ----"
            for file in self.filelist:
                try:
                    fh=open(file,"r")
                except:
                    print "Can't open file "+file+", skipping..."
                else:
                    lines=fh.readlines()
                    fh.close()
                    realfilename=file.split("/tmp_")[1]
                    content=""
                    print "Sending "+realfilename+"..."
                    for line in lines:
                        #line=unicode(line,"iso-8859-1")
                        #line=line.encode("UTF-8","replace")
                        content+=line
                    result=self.server.getfile(password,self.server_params[3],realfilename,content)
                    if result=="FILE OK":
                        print "Transfer complete !"
                    elif result=="CAN'T OPEN FILE":
                        print "!! The server can't write the file, aborting..."
                    elif result=="BUSY":
                        print "Server is busy, aborting..."
                        return "Server is busy, try again later."
                    else:
                        print "!! Not authorized, aborting file..."
                    try:
                        remove(file)
                    except:
                        pass
                
        
            print "---- END OF TRANSFER ----"
            print "Please wait while remote server is restarting ASTERISK..."
            result=self.server.reload_asterisk(password,self.server_params[3])
            del self.server
            self.server=None

            if result=="OK":
                print "Remote ASTERISK server is now ready with the new configuration"
                return True
            else:
                print "A problem occured while restarting remote ASTERISK"
                return "The remote configuration files has been updated with sucess but the server could not restart remote ASTERISK ! Maybe the server was not launched as ROOT or is down. Check your remote system."


        elif mode=="GET":
            print "Get file list from server..."
            remotelist=self.server.getfilelist(password,self.server_params[3]).split("#")
            del(self.filelist)
            self.filelist=[]
            for f in remotelist:
                if len(f)>0:
                    fname=GLOBAL.PREFIXDIR+"tmp_"+f
                    print "Added "+fname+" to import list"
                    self.filelist.append(fname)
                
            print "---- BEGINNING OF TRANSFER (GET) ----"
            for file in self.filelist:
                realfilename=file.split("/tmp_")[1]
                try:
                    fh=open(file,"w")
                except:
                    print "Can't open file "+file+", skipping..."
                else:
                    result=self.server.sendfile(password,self.server_params[3],realfilename)
                    if result=="CAN'T OPEN FILE":
                        print "!! The server can't find the file, aborting..."
                    elif result=="BUSY":
                        print "Server is busy, aborting..."
                        return "Server is busy, try again later."
                    elif result=="KEY ERROR":
                        print "!! Not authorized, aborting file..."
                    else:
                        #write file to disk
                        fh.write(result)
                        print "File "+file+" imported"
                    fh.close()
                
        
            print "---- END OF TRANSFER ----"

            saveconfigdir=GLOBAL.GLOBAL_PARAMS["configdir"]
            GLOBAL.GLOBAL_PARAMS["configdir"]=GLOBAL.PREFIXDIR+"tmp_"
            result=Filesystem.load_asterisk_config()
            GLOBAL.GLOBAL_PARAMS["configdir"]=saveconfigdir

            for file in self.filelist:
                try:
                    remove(file)
                except:
                    pass
                

            return result

        else :
            print "Unknown mode. Valid modes are GET or SEND"
            return "INTERNAL ERROR"
        
        
    #_______________________________________________________________

#_______________________________________________________________





