#!/usr/bin/python
# -*- coding: iso-8859-1 -*-

#____________________________________________________________________________
# ACTOS
# Asterisk Configuration Tool Open Source
# (c) EOLE - Mai 2004 - Pierre Derrier
#____________________________________________________________________________
#    This program is free software; you can redistribute it and/or modify
#    it under the terms of the GNU General Public License as published by
#    the Free Software Foundation; either version 2 of the License, or
#    any later version.
#
#    This program is distributed in the hope that it will be useful,
#    but WITHOUT ANY WARRANTY; without even the implied warranty of
#    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#    GNU General Public License for more details.
#
#   You should have received a copy of the GNU General Public License
#    along with this program; if not, write to the Free Software
#    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
#____________________________________________________________________________
# CONFFILEDEFS.PY : DEFINITION OF CONF FILES
#____________________________________________________________________________

from Global import GLOBAL,_
import Tools

#-=-=-=-=-=-=-=-=-=-= EXTENSIONS.CONF {

def write_extensions_conf(fh):
    #export DialPlan
    fh.write("\n;***************** General options *****************\n")
    fh.write("[general]\n")
    fh.write("static=yes\n")
    fh.write("writeprotect=yes\n")
    fh.write(";******************* Globals values ******************\n")
    fh.write("[globals]\n")
    for var in GLOBAL.VARIABLES:
        Tools.debug("Found GLOBAL variable '"+var+"' : writing to file")
        fh.write(var+" => "+str(GLOBAL.VARIABLES[var])+"\n")
    fh.write("\n;******************** DIAL PLAN ********************\n")
    for c in GLOBAL.ListeContextes:
        fh.write("\n;------------------------\n")
        fh.write(c.toString())
        fh.write(";-------------------------\n")

#-=-=-=-=-=-=-=-=-=-= } EXTENSIONS.CONF


#-=-=-=-=-=-=-=-=-=-= ASTERISK.CONF {

def write_asterisk_conf(fh):
    fh.write("\n;***************** Directories *****************\n")
    fh.write("[directories]\n")
    for d in GLOBAL.ASTERISK_DIRECTORIES.keys():
        fh.write(d+" => "+GLOBAL.ASTERISK_DIRECTORIES[d]+"\n")
#-=-=-=-=-=-=-=-=-=-= } ASTERISK.CONF


#-=-=-=-=-=-=-=-=-=-= AGEnTS.CONF {

def write_agents_conf(fh):

     fh.write("\n;********************* Agents ***********************\n")
     fh.write("[agents]\n")
     fh.write(";----------- options\n")
     for p in GLOBAL.AGENT_PARAMS.keys():
         fh.write(p+" = "+GLOBAL.AGENT_PARAMS[p]+"\n")

     fh.write("\n;----------- agents definition\n")         
     for u in GLOBAL.ListeUsers:
         if u.protocol=="agent":
             Tools.debug("Found "+u.protocol+" user : writing to file")
             fh.write(u.toString())
                                     
#-=-=-=-=-=-=-=-=-=-=} AGEnTS.CONF


#-=-=-=-=-=-=-=-=-=-= MEETME.CONF {

def write_meetme_conf(fh):

     fh.write("\n;***************** Meetme configuration ******************\n")
     fh.write("[rooms]\n")
     for r in GLOBAL.CONFROOMS.keys():
     	if str(GLOBAL.CONFROOMS[r]).strip()=="":
        	fh.write("conf => "+r+"\n")
        else:
        	fh.write("conf => "+r+","+str(GLOBAL.CONFROOMS[r])+"\n")

#-=-=-=-=-=-=-=-=-=-=} MEETME.CONF



#-=-=-=-=-=-=-=-=-=-= XXX.CONF {

def write_xxx_conf(protocol,fh,promptwarning):
   

    fh.write("\n;************** Protocol definitions ***************\n")
    fh.write("[general]\n;----------- general setup\n")

    #export general keys
    pdic=Tools.copy.deepcopy(GLOBAL.PROTOCOLS[protocol])
    Tools.DeleteKeysIfPossible(pdic,["fichier","bandwidth"])
    Tools.WriteAndDeleteKeysIfPossible(fh,pdic,["port","bindaddr","tos"])

    #export jitter buffer config
    if protocol=="iax2":
        fh.write(";----------- jitter buffer configuration\n")
        Tools.WriteAndDeleteKeysIfPossible(fh,pdic,["jitterbuffer","dropcount","maxjitterbuffer","maxexcessjitterbuffer"])

    #Export codecs config
    fh.write(";----------- codecs setup\n")
    codecs=pdic["codecs"]
    if codecs==None or len(codecs)==len(GLOBAL.CODECS["high"]):
        codecs=["all"]
    else:
        fh.write("disallow = all\n")
        
    if codecs!=None:
        for cod in codecs:
            fh.write("allow = "+cod+"\n")

    del(pdic["codecs"])

    #save other global keys
    fh.write(";----------- other options\n")
    if not ("context" in pdic.keys()):
        if promptwarning:
            Tools.debug("-- Warning : No default context defined for protocol "+protocol+".\n The value 'default' will be written : make sure that this context exists in your dialplan or there will be a security hole in your system.")
        fh.write("context = default\n")

    for k in pdic.keys():
        fh.write(k+" = "+pdic[k]+"\n")

    #save register=>... lines
    fh.write(";----------- register to peers\n")
    if protocol in GLOBAL.TOREGISTER.keys():
        for r in GLOBAL.TOREGISTER[protocol]:
            fh.write("register => "+r+"\n")

    for u in GLOBAL.ListeUsers:
        if u.protocol==protocol and u.registerwith:
            fh.write("register => "+u.registerstring+"\n")

    #export users for this protocol
    fh.write("\n;********************* Users ***********************\n")
    for u in GLOBAL.ListeUsers:
        if u.protocol==protocol:
            Tools.debug("Found "+u.protocol+" user : writing to file")
            fh.write(u.toString())

#-=-=-=-=-=-=-=-=-=-= } XXX.CONF


        

        
