#!/usr/bin/python
# -*- coding: iso-8859-1 -*-

#_______________________________________________________________
# ACTOS
# Asterisk Configuration Tool Open Source
# (c) EOLE - Mai 2004 - Pierre Derrier
#_______________________________________________________________
#    This program is free software; you can redistribute it and/or modify
#    it under the terms of the GNU General Public License as published by
#    the Free Software Foundation; either version 2 of the License, or
#    any later version.
#
#    This program is distributed in the hope that it will be useful,
#    but WITHOUT ANY WARRANTY; without even the implied warranty of
#    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#    GNU General Public License for more details.
#
#   You should have received a copy of the GNU General Public License
#    along with this program; if not, write to the Free Software
#    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
#____________________________________________________________________________
# CONTEXT.PY : CONTAINER FOR CONTEXT DATAS
#_______________________________________________________________
# Contains :
#            CLASS Context
#            CLASS LigneContext
#_______________________________________________________________

#_______________________________________________________________
# EXTERN DEPENDENCIES
#_______________________________________________________________
from Global import GLOBAL,_
import Tools
import copy
import string
#_______________________________________________________________

#_______________________________________________________________
NOTFOUND = NOVALUE = -1
PREFIXE = ["exten","include","register","switch"]
EXTEN=0
INCLUDE=1
#_______________________________________________________________

    

#________________________________________________
def GetFullExtName(self,iname):
#________________________________________________
    #redonne le nom complet si nom abrg
    if iname in ["s","i","h","t","o"]:
        for ext in GLOBAL.DEFAULT_EXTENSIONS:
            if ext[1]==iname.strip():
                iname=ext
                break
    return iname
#________________________________________________


#_______________________________________________________________
class LineContext:
    """Definition d'une ligne de contexte"""
#_______________________________________________________________

    #________________________________________________
    def __init__(self,iextension=None,prio=NOVALUE,iaction="",iprefixe=0):
        """ Constructeur d'une ligne de contexte """
    #________________________________________________
        if iextension==None:
            self.extension=Extension()
        else:
            self.extension=iextension
        self.priority=prio
        self.prefixe=iprefixe
        self.action=iaction
        self.iter=None
    #________________________________________________


    #________________________________________________
    def toString(self):
        """ ligne de contexte vers String"""
    #________________________________________________
        return PREFIXE[self.prefixe]+" => "+self.extension.getName()+","+str(self.priority)+","+self.action
    #________________________________________________


#_______________________________________________________________



#_______________________________________________________________
class Extension:
#_______________________________________________________________
    #________________________________________________
    def __init__(self,iname="_."):
        """ Constructeur d'une extension """
    #________________________________________________
        self.lines =[]
        self.name=iname
    #________________________________________________
    

    #________________________________________________
    def getmaxpriority(self):
    #________________________________________________
       max=0
       indmax=0
       for l in range(len(self.lines)):
           if int(self.lines[l].priority)>max:
               max=int(self.lines[l].priority)
               indmax=l
       return max,indmax
    #________________________________________________

    
    #________________________________________________
    def getindexline(self,priority):
    #________________________________________________
        for l in range(len(self.lines)):
            if int(self.lines[l].priority)==int(priority):
                return l
        return None
    #________________________________________________


    #________________________________________________
    def getnblines(self):
    #________________________________________________
        return len(self.lines)
    #________________________________________________

    
    #________________________________________________
    def addline(self,priority=None,action=""):
    #________________________________________________
        if priority==None:
            priority,ind=self.getmaxpriority()
            priority+=1
        else:
            index=self.getindexline(priority)
            #si la prio existe, il faut dcaler les autres
            if index!=None:
                for j in range(len(self.lines)):
                    if int(self.lines[j].priority)>=int(priority):
                        self.lines[j].priority=int(self.lines[j].priority)+1
                
        line=LineContext(self,priority,action)
        self.lines.append(line)
    #________________________________________________

    
    #________________________________________________
    def changeline(self,priority,action):
    #________________________________________________
        index=self.getindexline(priority)
        if index!=None:
            self.lines[index].action=action
        else:
            self.addline(priority,action)
    #________________________________________________


    #________________________________________________
    def removeline(self,priority=1):
    #________________________________________________
        index=self.getindexline(priority)
        if index!=None:
            del(self.lines[index])
    #________________________________________________


    #________________________________________________
    def sort(self):
        """ Trie les lignes par ordre de priorit croissante """
    #________________________________________________
        if len(self.lines)==0:
            return
        
        listefinale=[]
        while len(self.lines)>0:
            maxprio,indmax=self.getmaxpriority()
            listefinale.insert(0,self.lines[indmax])
            del(self.lines[indmax])

        self.lines=copy.deepcopy(listefinale)
    #________________________________________________


    #________________________________________________
    def getName(self):
    #________________________________________________
        name=self.name
        if name[0]=="(" and name[2]==")":
            name=name[1]
        return name
    #________________________________________________


    #________________________________________________
    def toString(self):
    #________________________________________________
        self.sort()
        chaine=""
        for l in self.lines:
            chaine+=l.toString()+"\n"

        return chaine
    #________________________________________________

#_______________________________________________________________



#_______________________________________________________________
class Context:
    """Definition d'un contexte d'appel"""
#_______________________________________________________________

    #________________________________________________
    def __init__(self,inom="default"):
        """ Constructeur d'un contexte """
    #________________________________________________
        Tools.debug("# New Context instance "+inom)
        self.name=inom
        self.exts={}
        self.includedCtxt=[]
        self.includedFiles=[]
        self.switchLines=[]
        self.ignorepatLines=[]
    #________________________________________________

    #________________________________________________
    def include(self,ctxt):
    #________________________________________________
        self.includedCtxt.append(ctxt)
    #________________________________________________

    #________________________________________________
    def setextension(self,extension,objExt=None):
    #________________________________________________
        if objExt==None:
            Tools.debug("New extension instance : "+extension)
            objExt=Extension(extension)
        
        self.exts[extension]=objExt
    #________________________________________________

    #________________________________________________
    def hasextension(self,extension):
    #________________________________________________
        result=extension in self.exts.keys()
        Tools.debug(extension+" in "+str(self.exts.keys())+" ? = "+str(result))
        return result
    #________________________________________________


    #________________________________________________
    def addline(self,extension,priority=None,action=""):
        """ Ajoute une ligne au contexte """
    #________________________________________________
        #translate extension to full name
        extension=extension.strip()
        if len(extension)==1:
            for e in GLOBAL.DEFAULT_EXTENSIONS:
                if e[1]==extension:
                    extension=e
                    break
    
        #if ext doesn't exists create it
        if extension not in self.exts.keys():
            self.exts[extension]=Extension(extension)

        #add the line
        self.exts[extension].addline(priority,action)
    #________________________________________________


    #________________________________________________
    def removeline(self,extension,priority=None):
        """ Ajoute une ligne au contexte """
    #________________________________________________
        #if ext doesn't exists
        if extension not in self.exts.keys():
            return False
        #remove the line
        self.exts[extension].removeline(priority)
        return True
    #________________________________________________


    #________________________________________________
    def parse(self,lines):
    #________________________________________________

        for l in lines:
          try:
            params=""
            line=l.split("=")
            keyword=string.lower(line[0].strip())

            #remove > char
            try:
                templine=line[1].strip().lstrip(">").strip()
                #bugfix >> if the line contains "=" it has been splitted and we must reconcatenate it
                if len(line)>2:
                    for i in range(len(line)-2):
                        templine+="="+line[i+2]
            except:
                templine=line[0]

            line = templine
	    
            if keyword=="exten":
                #extract params
                indexparam=line.find("(")
                indexfinparam=len(line)
                if indexparam!=-1:
                    #search the last ')'
                    tempindex=line.find(")")
                    if tempindex!=-1:
                        subline=line
                        while ((tempindex!=-1) and (tempindex+1<len(subline))):
                            subline=subline[(tempindex+1):]
                            tempindex+=subline.find(")")+1
                    
                    if tempindex!=-1:
                        indexfinparam=tempindex+1
                else:
                    indexparam=indexfinparam
                params=line[indexparam:indexfinparam]
               
                #extract values
                vals=line.split(",")

                #syntax verification
                try:
                    a=int(vals[1])
                except:
                    Tools.debug("-- Error : Syntax error in line :\n"+l+"\nInvalid priority number")
                else:
                    try:
                        b=vals[2]
                    except:
                        Tools.debug("-- Error : Syntax error in line :\n"+l+"\nInvalid action")
                        
                
                newindexparam=vals[2].find("(")
                if newindexparam==-1:
                    newindexparam=len(vals[2])
                action=vals[2][0:newindexparam]
                self.addline(vals[0].strip(),vals[1].strip(),action+params)

            elif keyword=="include":
                line=line.lstrip("\"")
                line=line.rstrip("\"")
                self.includedCtxt.append(line.strip())
            
            elif keyword=="switch":
            	self.switchLines.append(line.strip())

            elif keyword=="ignorepat":
                self.ignorepatLines.append(line.strip())

            elif keyword[0:8]=="#include":
                fich=keyword.split(" ")
                self.includedFiles.append(fich[1])
                Tools.debug("Added included file "+fich[1])
                
            else:
                Tools.debug("-- Error : Unknown keyword '"+keyword+"' in line :\n"+l+"\nThe line will be ignored")
          except:
              Tools.debug("-- Error : An error occured while parsing line :\n"+l+"\nMaybe the syntax is not correct. Please correct it and try again")

#        Tools.debug("Added "+str(len(self.exts.keys()))+" extensions")
    #________________________________________________

    #________________________________________________
    def put_in_models(self,modelext,modelctxt=None,showonerror=True):
    #________________________________________________
        for ext in self.exts.keys():
            addchildtoiter=None
            iter = modelext.insert_before(None, None)
            modelext.set_value(iter, 0, ext)
            if self.exts[ext].getnblines()>0:
                for l in self.exts[ext].lines:
                    #check syntax
                    try:
                        apriority=int(l.priority)
                    except:
                        Tools.debug("-- Error : Bad priority number in line :\n"+str(l.prefixe)+" "+str(l.priority)+" "+str(l.action)+"\nCan't edit context")
                        return False
                        
                    #check if line is a subline
                    if apriority>101 and showonerror:
                        if addchildtoiter==None:
                            addchildtoiter=modelext.insert_before(iter,None)
                            modelext.set_value(addchildtoiter,0,"--[ "+_("ITEM_ERROR")+" ]--")

                        iter2=modelext.insert_before(addchildtoiter,None)

                    else:
                        iter2 = modelext.insert_before(iter, None)

                    modelext.set_value(iter2, 0, str(apriority)+" :: "+l.action)

                if addchildtoiter==None and showonerror:
                    addchildtoiter=modelext.insert_before(iter,None)
                    modelext.set_value(addchildtoiter,0,"--[ "+_("ITEM_ERROR")+" ]--")
                
                    
        if modelctxt!=None:
            for c in self.includedCtxt:
                iter=modelctxt.insert_before(None,None)
                modelctxt.set_value(iter,0,c)

        return True
    #________________________________________________


    #________________________________________________
    def toString(self):
        """Return the context in string format, ready to be writed in conf file"""
    #________________________________________________
        
        chaine="["+self.name+"]\n"
        templiste=self.exts.keys()
        templiste.sort()
        for i in self.ignorepatLines:
            chaine+="ignorepat => "+i+"\n"

        for s in self.switchLines:
            chaine+="switch => "+s+"\n"
        	
        for c in self.includedCtxt:
            #CORRECTED BUG            chaine+="include => \""+c+"\"\n"
            chaine+="include => "+c+"\n"
            #CORRECTED BUG
        for il in templiste:
            chaine+=self.exts[il].toString()
        return chaine
    #________________________________________________


    #________________________________________________
    def toActosString(self):
        """ Convertit en un format compact pour la sauvegarde"""
    #________________________________________________
        return "CONTEXT#"+self.name+"#"
        #A COMPLETER !!#
    #________________________________________________

#_______________________________________________________________



#_______________________________________________________________
# IF SOMEONE LAUNCH THIS SCRIPT.... 
#_______________________________________________________________
if __name__ == "__main__":
    Tools.Interface.AboutMessage()
    print "Please launch actos.py\n"
#_______________________________________________________________


