#!/usr/bin/python
# -*- coding: iso-8859-1 -*-

#____________________________________________________________________________
# ACTOS
# Asterisk Configuration Tool Open Source
# (c) EOLE - Mai 2004 - Pierre Derrier
#____________________________________________________________________________
#    This program is free software; you can redistribute it and/or modify
#    it under the terms of the GNU General Public License as published by
#    the Free Software Foundation; either version 2 of the License, or
#    any later version.
#
#    This program is distributed in the hope that it will be useful,
#    but WITHOUT ANY WARRANTY; without even the implied warranty of
#    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#    GNU General Public License for more details.
#
#   You should have received a copy of the GNU General Public License
#    along with this program; if not, write to the Free Software
#    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
#____________________________________________________________________________
# FILESYSTEM.PY : TOOLS FOR FILE ACCESS
#____________________________________________________________________________
# Contains :
#            FUNCTION load_actos_config ()
#                   Load config from ACTOS.CONF file
#
#            FUNCTION load_actos_file (filename)
#                   Load a .actos file
#
#            FUNCTION save_actos_file (filename)
#                   Save current datas in a specific .actos format
#
#            FUNCTION parse_actos_string (string)
#                   Extract datas from a string coded in the specific .actos format
#
#            FUNCTION save_conf_file (protocol)
#                   Save protocol configuration and protocol users in the appropriate .conf file
#
#            FUNCTION load_conf_file (protocol)
#                   Load protocol configuration and protocol users from the appropriate .conf file
#
#______________________________________________________________

#______________________________________________________________
# EXTERN DEPENDENCIES
#______________________________________________________________
from User import *
from Context import *
import shutil
import datetime
from ConfFilesDef import *
#import Tools
from Global import GLOBAL,_
#__________________________________________________________________


#__________________________________________________________________
def save_actos_config():
    """ Save actos.conf """
#__________________________________________________________________

    Tools.debug("Writing actos.conf...")
    try:
        fh=open(GLOBAL.PREFIXDIR+"actos.conf","w")
    except:
        Tools.debug("-- Error : "+_("ERROR_FILE",[GLOBAL.PREFIXDIR+"actos.conf"]))
        return

    for p in GLOBAL.GLOBAL_PARAMS.keys():
        fh.write(p+" = "+GLOBAL.GLOBAL_PARAMS[p]+"\n")
    fh.close()


#__________________________________________________________________
def load_actos_config():
    """ Load actos.conf """
#__________________________________________________________________

    #lecture du fichier de configuration actos.conf
    Tools.debug("Reading actos.conf...")
    try:
        fh=open(GLOBAL.PREFIXDIR+"actos.conf","r")
    except:
        #create a new file if error
        Tools.debug("-- Warning : "+_("ERROR_ACTOS_CONF_NOT_FOUND"))
        save_actos_config()
        fh=open(GLOBAL.PREFIXDIR+"actos.conf","r")
        
    file=fh.readlines()
    fh.close()

    #put the content of file in GLOBAL.GLOBAL_PARAMS
    for line in file:
        line=line.split("=")
        GLOBAL.GLOBAL_PARAMS[line[0].strip()]=line[1].strip()

    #init log mode
    Tools.debug_init(GLOBAL.GLOBAL_PARAMS["logtype"])

    #init protocols
    lp= Tools.GetProtocolsSupported()
    GLOBAL.PROTOCOLS={}
    for p in lp.keys():
        Tools.InitDefaultsProtocol(p)
        Tools.SetProtocolKey(p,"fichier",lp[p])

#_______________________________________________________________


#__________________________________________________________________
def load_asterisk_config(ask=True):
#__________________________________________________________________

    #clean config if asked
    if GLOBAL.GLOBAL_PARAMS["cleanbeforeimport"]=="yes":
        if ask:
            #Demande de confirmation
            result=Tools.Interface.DialogBox(_("MSG_CLEAN_DATAS"),type_buttons=Tools.Interface.gtk.BUTTONS_YES_NO,type_dialog=Tools.Interface.gtk.MESSAGE_QUESTION)
            if result==Tools.Interface.gtk.RESPONSE_NO:
                return "Operation cancelled"
            Tools.debug("Cleaning current config")
            del(GLOBAL.ListeUsers)
            del(GLOBAL.ListeContextes)
            del(GLOBAL.TOREGISTER)
            del(GLOBAL.CONFROOMS)
            GLOBAL.TOREGISTER={}
            GLOBAL.ListeUsers=[]
            GLOBAL.ListeContextes=[]
            GLOBAL.CONFROOMS={}

    Tools.debug("############################## Load configuration from"+GLOBAL.GLOBAL_PARAMS["configdir"])

    #for each supported protocol, init protocol and load the users
    listep=Tools.GetProtocolsSupported().keys()
    for p in listep:
        #all the work is done in load_conf_file
        listeu=load_conf_file(p)
        if listeu!=None:
            GLOBAL.ListeUsers.extend(listeu)

    listeu=load_conf_file("agents")
    if listeu!=None:
        GLOBAL.ListeUsers.extend(listeu)

    #check register lines that haven't been associated with a peer
    nb=len(GLOBAL.TOREGISTER)
    if nb>0:
    	Tools.debug("Notice : Found "+str(nb)+" register queries unassociated with a user/peer : "+str(GLOBAL.TOREGISTER))

    #load dialplan in extensions.conf
    listec=load_conf_file("extensions")
    if listec!=None:
        GLOBAL.ListeContextes.extend(listec)

    #load asterisk.conf
    load_conf_file("asterisk")
    if "astetcdir" in GLOBAL.ASTERISK_DIRECTORIES.keys():
        GLOBAL.GLOBAL_PARAMS["configdir"]=GLOBAL.get_asterisk_directory("astetcdir")
        
    load_conf_file("meetme")
    
    Tools.debug("############################# Importation done")

    Tools.Interface.DialogBox(_("MSG_CONF_LOADED",[str(len(GLOBAL.ListeUsers)),str(len(GLOBAL.ListeContextes)),str(len(GLOBAL.CONFROOMS))]))
    return True
#__________________________________________________________________


#__________________________________________________________________
def save_asterisk_config(prompt=True):
#__________________________________________________________________
    Tools.debug("############################# Export configuration in "+GLOBAL.GLOBAL_PARAMS["configdir"])
    #for each supported protocol, save the conf file
    result=True
    listep=Tools.GetProtocolsSupported().keys()
    for p in listep:
        #all the work is done in save_conf_file
        result=save_conf_file(p,prompt) and result

    Tools.debug("## Export agents / queues")
    result=save_conf_file("agents",prompt) and result
    Tools.debug("## Export meetme configuration")
    result=save_conf_file("meetme",prompt) and result
    Tools.debug("## Export Dialplan")
    result=save_conf_file("extensions",prompt) and result
    Tools.debug("## Export asterisk.conf")
    result=save_conf_file("asterisk",prompt) and result

    Tools.debug("############################# Exportation done")
    if prompt and result:
            Tools.Interface.DialogBox(_("MSG_CONF_SAVED"))
    return result
#__________________________________________________________________



#__________________________________________________________________
def save_conf_file(protocol,promptwarning=True):
    """sauvegarde d'un fichier .conf"""
# protocol can be any string that is a key in GLOBAL.PROTOCOLS dictionnary
#__________________________________________________________________

    #choose file to load for protocol
    dic={}
    if protocol not in ["extensions","asterisk","agents","meetme"]:
        dic=GLOBAL.PROTOCOLS[protocol]
    else:
        dic={"fichier": protocol+".conf"}
    filename=GLOBAL.GLOBAL_PARAMS["configdir"]+dic["fichier"]
        
    #save old config if asked
    if GLOBAL.GLOBAL_PARAMS["saveoldconf"]=="yes":
        copyfile=GLOBAL.GLOBAL_PARAMS["configdir"]+protocol+".conf.old"
        try:
            shutil.copy(filename,copyfile)
        except:
            Tools.debug("-- Error : "+_("ERROR_CANT_COPY",[filename]))
            return False

    Tools.debug("## Writing "+filename+" ...")

    #open conf file in write mode
    try:
        fh=open(filename,'w')
    except:
        Tools.debug("-- Error : "+_("ERROR_FILE",[filename]))
        return False

    #get current time
    a=datetime.datetime
    time=a.ctime(a.now())

    #write header
    fh.write(";_____________________________________________________________\n")
    fh.write(";"+dic["fichier"]+"\n")
    fh.write("; Generated by ACTOS (Asterisk Configuration Tool Open Source)\n")
    fh.write("; version 2.25\n")
    fh.write("; See http://www.derrier.com/pierre/code/actos.html\n")
    fh.write("; (c) EOLE - 2004/2005 - Pierre Derrier\n")
    fh.write(";_____________________________________________________________\n")
    fh.write("; File created "+time+"\n")
    fh.write(";_____________________________________________________________\n")

    #for code visibility, writing procedures are in ConfFilesDef.py
    if protocol=="extensions":
        write_extensions_conf(fh)
    elif protocol=="asterisk":
        write_asterisk_conf(fh)
    elif protocol=="agents":
        write_agents_conf(fh)
    elif protocol=="meetme":
        write_meetme_conf(fh)
    else:
        write_xxx_conf(protocol,fh,promptwarning)

    #writing end lines
    fh.write("\n;******************* END OF FILE *******************\n")
    fh.close()
    return True
#______________________________________________________________________________



#__________________________________________________________________________
def load_conf_file(protocol):
    """Chargement d'un fichier .conf"""
# protocol can be any string that is a key in GLOBAL.PROTOCOLS dictionnary
# or the value : "extensions" to load contexts
#___________________________________________________________________________

    NOT_A_PROTOCOL_LIST=["extensions","asterisk","agents","meetme"]
    currentprotocol=protocol

    #choose file name for this protocol
    filename=""
    if protocol in NOT_A_PROTOCOL_LIST:
        filename=GLOBAL.GLOBAL_PARAMS["configdir"]+protocol+".conf"
    else:
        if protocol in GLOBAL.PROTOCOLS.keys():
            filename=GLOBAL.GLOBAL_PARAMS["configdir"]+GLOBAL.PROTOCOLS[protocol]["fichier"]
        else:
            filename=GLOBAL.GLOBAL_PARAMS["configdir"]+protocol
            currentprotocol="extensions"


    Tools.debug("## Reading "+filename+" ...")

    #open file in read mode
    try:
        fh=open(filename,'r')
    except:
        Tools.debug("-- Error : Could not open file "+filename+" in read-mode")
        return None


    #get filename alone, without path... and verify coherence
    #TODO : !!!!!!!!!!!!!! THIS CODE IS VERY UGLY !!!!!!!!!!!!!!!!!!!!

    #nfile=filename.split("/")
    #nfile=nfile[len(nfile)-1].strip()
    #if nfile[:4]=="tmp_":
    #    nfile=nfile.lstrip("tmp_")
    
    #if protocol not in NOT_A_PROTOCOL_LIST:
    #    trouve=False
    #    listep=Tools.GetProtocolsSupported().keys()
    #    for currentprotocol in listep:
    #        if GLOBAL.PROTOCOLS[currentprotocol]["fichier"]==nfile:
    #            trouve=True
    #            break
    #    if not trouve:
    #        Tools.debug("-- Error : "+_("ERROR_FILE_PROTOCOL",[filename]))
    #        return None
    #!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!

 
    #Load file without comments
    lignes=[]
    while 1:
        line=fh.readline()

        if line=='':
            break
        
        line=line.strip()
        if len(line)>0 and line[0]!=';':
            line=line.split(";")
            line=line[0].strip()
            lignes.append(line)
  
    lignes.append("[FIN DE FICHIER]")
    fh.close()

    #Parse loaded file
    listeu=[]
    currentname=""
    currentkeys={}
    currentlines=[]
    count=0
    
    for i in lignes:
        if i[0]=="[":
            newname=i[1:].split("]")[0].strip()
            count+=1

            if count>1: #skip first header...

                if currentprotocol=="extensions":
                    #-=-=-=-=-=-=-=-=-=-= EXTENSIONS.CONF {
                    if currentname=="general":
                        Tools.debug("## TODO : load general section of extensions.conf")
                        Tools.debug("          for the moment : writeprotect=yes and static=yes")

                    elif currentname=="globals":
                        Tools.debug("Loading GLOBALS variables")
                        for line in currentlines:
                            key,val=Tools.ExtractKeyAndValue(line)
                            GLOBAL.VARIABLES[key]=val

                    else:
                        #Dialplan mode : create new context with current name
                        newContext=Context(currentname)
                        newContext.parse(currentlines)
                        listeu.append(newContext)

                        #manage included files
                        for includedfile in newContext.includedFiles:
                            listeotherc=load_conf_file(includedfile)
                            if listeotherc!=None:
                                for cont in listeotherc:
                                    listeu.append(cont)

                    #-=-=-=-=-=-=-=-=-=-= } EXTENSIONS.CONF

                elif currentprotocol=="asterisk":
                    #-=-=-=-=-=-=-=-=-=-= ASTERISK.CONF {
                    if currentname=="directories":
                        Tools.debug("Loading asterisk paths")
                        for line in currentlines:
                            key,val=Tools.ExtractKeyAndValue(line)
                            GLOBAL.ASTERISK_DIRECTORIES[key]=val
                    #-=-=-=-=-=-=-=-=-=-= } ASTERISK.CONF

                elif currentprotocol=="agents":
                    #-=-=-=-=-=-=-=-=-=-= AGEnTS.CONF {
                        if currentname=="agents":
                            for line in currentlines:
                                key,val=Tools.ExtractKeyAndValue(line)
                                if key.strip()=="agent":
                                    val=val.split(",")
                                    ikeys=GLOBAL.DEFAULTS_NEW_USER
                                    ikeys["type"]="agent"
                                    ikeys["secret"]=val[1]
                                    ikeys["login"]=val[0]
                                    newAgent=User(name=val[2],keys=ikeys,protocol="agent")
                                    listeu.append(newAgent)
                                else:
                                    GLOBAL.AGENT_PARAMS[key.strip()]=val
                    #-=-=-=-=-=-=-=-=-=-= } AGents.CONF

                elif currentprotocol=="meetme":
                    #-=-=-=-=-=-=-=-=-=-= MEETME.CONF {
                        if currentname=="rooms":
                            for line in currentlines:
                                key,val=Tools.ExtractKeyAndValue(line)
                                if key.strip()=="conf":
                                    val=val.split(",")
                                    val.append("")
                                    GLOBAL.CONFROOMS[str(val[0])]=str(val[1])
                                    Tools.debug("Found room "+str(val[0]))
                                else:
                                    Tools.debug("Ignoring line : Unknown keyword "+key)
                    #-=-=-=-=-=-=-=-=-=-= } MEETME.CONF

                else:
                    #-=-=-=-=-=-=-=-=-=-= XXX.CONF {
                    if currentname=="general":
                        #Build protocol setup with current values
                        #this function update GLOBAL.TOREGISTER
                        Tools.Update_protocol(currentprotocol,currentkeys)

                    else:
                        #create new user with current name
                        newUser=User(name=currentname,keys=currentkeys,protocol=currentprotocol)
                        listeu.append(newUser)
                    #-=-=-=-=-=-=-=-=-=-= } XXX.CONF
                
                        
                del(currentkeys)
                currentkeys={}
                del(currentlines)
                currentlines=[]
            #fin du if count>1
            
            currentname=Tools.string.lower(newname)

        else:
        #if it's not a header (between [ ]) but a normal line
            if currentprotocol in NOT_A_PROTOCOL_LIST:
                # just stock the line as it
                currentlines.append(i.strip())

            else:
                #-=-=-=-=-=-=-=-=-=-= XXX.CONF {
                #stock the key and the value
                key,val=Tools.ExtractKeyAndValue(i)
                if val!="":
                    if key in currentkeys.keys():
                        val=currentkeys[key]+"##"+val
                    currentkeys[key]=val
                else:
                    Tools.debug("Ignoring empty key "+key)
                #-=-=-=-=-=-=-=-=-=-= } XXX.CONF

    #fin du for
    

    #update users with the GLOBAL.TOREGISTER variables
    if currentprotocol not in NOT_A_PROTOCOL_LIST:
        for u in listeu:
            if u.protocol==currentprotocol and currentprotocol in GLOBAL.TOREGISTER.keys():
                for r in GLOBAL.TOREGISTER[currentprotocol]:
                    login,passw,host=Tools.ParseRegisterString(r)
                    #if the login/pass/host are those used for register, associate the line with the user
                    if login==u.params["username"] and passw==u.params["secret"] and host==u.params["host"]:
                        u.registerwith=True
                        u.registerstring=r
                        Tools.debug("Register with user '"+u.name+"' using string '"+r+"'")
                        #remove line from TOREGISTER table
                        GLOBAL.TOREGISTER[currentprotocol].remove(r)
                        #go to next line
                        break
      
    return listeu
#____________________________________________________________

    

#!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!
#!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!
#!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!
#!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!
#!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!
# BELOW THIS LINE, THAT'S ONLY OLD CODE UNUSED ANYMORE
#!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!
#!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!
#!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!
#!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!
#!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!

    
#__________________________________________________________________
def load_actos_file(filename):
    """ Chargement d'une configuration enregistre"""
#__________________________________________________________________

    Tools.debug("Loading file "+filename+" ...")
    Listeusers=[]
    Listecontextes=[]
    
    try:
        fh=open(filename,'r')
    except:
        Tools.debug("-- Error : can't open file "+filename)
        return False,None,None

    while 1:
        ch = fh.readline()
        if ch=='':
            break

        objet,typeobjet=parse_actos_string(ch)
        if typeobjet=="USER":
            Listeusers.append(objet)
        elif typeobjet=="CONTEXT":
            Listecontextes.append(objet)
        
    Tools.debug("## Done.")
    fh.close()
    return True,Listeusers,Listecontextes
#_______________________________________________________________



#__________________________________________________________________
def save_actos_file(filename):
    """ Sauvegarde de la configuration courante dans un fichier"""
#__________________________________________________________________

    Tools.debug("Sauvegarde effective de la configuration dans "+filename+" ...")

    try:
        fh=open(filename,'w')
    except:
        print "## Erreur : Impossible d'ouvrir le fichier "+filename+" en criture"
        return None
    templist=GLOBAL.ListeUsers[:]
    templist.extend(GLOBAL.ListeContextes)
    for u in templist:
        chaine=u.toActosString()
        try:
            Tools.debug(chaine)
            fh.write(chaine+"\n")
        except:
            print "## Erreur : Problme pendant la sauvegarde du fichier"
            fh.close()
            return None
    
    Tools.debug("## Sauvegarde termine.")
    fh.close()
    return 1
#_______________________________________________________________
    

    
#__________________________________________________________________
def parse_actos_string(chaine):
    """ Rcupration du contenu d'une chaine ACTOS"""
#__________________________________________________________________

    splitted=chaine.split("#")
    result=None
    
    if splitted[0]=="USER":
        Tools.debug("Creating user "+splitted[1])
        newUser=User(splitted[1],splitted[2],splitted[3],splitted[4],splitted[5],splitted[6])
        result=newUser
        
    elif splitted[0]=="CONTEXT":
        Tools.debug("Creating context "+splitted[1])
        newContext=Context(splitted[1])
        result=newContext

    return result,splitted[0]
#_______________________________________________________________



#____________________________________________________________
# IF SOMEONE LAUNCH THIS SCRIPT.... 
#____________________________________________________________
if __name__ == "__main__":
    AboutMessage()
    print "Please launch actos.py\n"
#____________________________________________________________
