#!/usr/bin/python
# -*- coding: iso-8859-1 -*-


#____________________________________________________________________________
# ACTOS
# Asterisk Configuration Tool Open Source
# Copyright (c) EOLE - Mai 2004 - Pierre Derrier
#____________________________________________________________________________
#    This program is free software; you can redistribute it and/or modify
#    it under the terms of the GNU General Public License as published by
#    the Free Software Foundation; either version 2 of the License, or
#    any later version.
#
#    This program is distributed in the hope that it will be useful,
#    but WITHOUT ANY WARRANTY; without even the implied warranty of
#    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#    GNU General Public License for more details.
#
#   You should have received a copy of the GNU General Public License
#    along with this program; if not, write to the Free Software
#    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
#____________________________________________________________________________
# GLOBAL.PY : GLOBAL DATAS
#____________________________________________________________________________
# Notes: 
#
#
#
#
#
#
#
#
#
#
#
#
#____________________________________________________________________________


#________________________________________________
class GlobalClass:
    """ Classe container des variables globales """
#________________________________________________

    def __init__(self):

        self.LANGUAGES={"fr":{"fr":"Francais","en":"Anglais","es":"Espagnol","it":"Italien"},
                        "en":{"fr":"French","en":"English","es":"Spanish","it":"Italian"},
                        "es":{"fr":"Frances","en":"Ingles","es":"Espanol","it":"Italiano"},
                        "it":{"fr":"Francese","en":"Inglesi","es":"Spagnolo","it":"Italiano"},
                       }


        self.PREFIXDIR = "/usr/share/actos/"

        #"fr":"glade/actos_fr.glade",
        self.GLADE_FILE = {"fr":"actos_en.glade",
                           "en":"actos_en.glade",
                           "es":"actos_en.glade",
                           "it":"actos_en.glade"}
        self.LOGFILE=None
        self.ListeUsers=[]
        self.ListeContextes=[]
        
        self.ASTERISK_DIRECTORIES={"astetcdir":"/etc/asterisk",
                                   "astmoddir":"/usr/lib/asterisk/modules",
                                   "astvarlibdir":"/var/lib/asterisk",
                                   "astagidir":"/var/lib/asterisk/agi-bin",
                                   "astspooldir":"/var/spool/asterisk",
                                   "astrundir":"/var/run",
                                   "astlogdir":"/var/log/asterisk"
                                  }

        self.GLOBAL_PARAMS={"configdir" : "/etc/asterisk/",
                            "scriptsdir": self.PREFIXDIR+"scripts/",
                            "logtype":"file",
                            "language":"fr",
                            "saveoldconf":"yes",
                            "autoimport" : "yes",
                            "autoexport" : "yes",
                            "autosavemostusedactions":"yes",
                            "cleanbeforeimport" :"yes",
                            "natsupport" : "yes",
                            "externip" : "194.167.18.244",
                            "localnet" : "192.168.0.0",
                            "localmask" : "255.255.255.0",
                            "protocols" : "sip/sip.conf##iax2/iax.conf##h323/h323.conf",
                            "lastremotehost":"localhost",
                            "lastremoteport":"8007"}
        

        self.AGENT_PARAMS={"autologoff":"15",
                           "ackcall":"yes",
                           "wrapuptime":"5000",
                           "recordagentcalls":"no",
                           "recordformat":"wav",
                           "createlink":"yes"}
                           
        self.PROTOCOLS={"":{}}
        self.TEMP_PROTOCOLS={}
        self.TOREGISTER={}

       
        self.USER_TYPES=["friend","user","peer","agent"]
        self.AUTH_TYPES=["plaintext","md5","rsa"]
        self.TOS_TYPES=["none","lowdelay","throughput","reliability","mincost"]
        self.DMTF_MODES=["default","inband","rfc2833","info"]
        self.COLUMN_OF_TABLE_USERS=["name","protocol","type","host","context","login","secret"]
        self.OPTIONS_USER=["auth","nat","trunk","canreinvite","qualify"]

        self.DEFAULT_QUALIFY_MS = 2000
        self.DEFAULT_CONTEXTS=["default","local-extensions","incoming-calls","outgoing-calls"]
        self.DEFAULTS_NEW_USER={"nat":"",
                                "trunk":"",
                                "canreinvite":"",
                                "auth":"",
                                "qualify":"",
                                "dmtfmode":"",
                                "host":"dynamic",
                                "name":"newUser",
                                "login":"",
                                "secret":"",
                                "context":self.DEFAULT_CONTEXTS[0],
                                "type":self.USER_TYPES[0],
                                "protocol":self.PROTOCOLS.keys()[0]}

        self.DEFAULT_EXTENSIONS=["(s) : Start",
                                 "(i) : Invalid",
                                 "(h) : Hangup",
                                 "(t) : Time Out in context",
                                 "(T) : Absolute Time Out",
                                 "(o) : Operator"]


        self.CODECS={"low":["g723.1","gsm","lpc10"],
                     "medium":["g723.1","gsm","lpc10","g726","adpcm","g729","speex","ilbc","h261","h263"], 
                     "high":["g723.1","gsm","lpc10","ulaw","alaw","g726","slinear","adpcm","g729","speex","ilbc","h261","h263"]
                    }

        

        #now defined in file actions.def
        #imported by Tools.import_defs()
        self.ACTIONS = {}
        
        self.ACTIONS_AVAILABLE_WIZARD=["dial","goto","random","playback","background","adsiprog",
                                       "authenticate","voicemail","meetme"]
        self.ACTIONS_WITHOUT_PARAMS=[]
        self.ACTIONS_MOSTUSED={}

        self.VARIABLES = {}
        self.CONFROOMS = {}

        #now defined in file channelvars.def
        #imported by Tools.import_defs()
        self.CHANNEL_VARS = {}
        
        self.WORDS={"1_ACTION":["The action","l'action","la accion"],
                    "1_EXTENSION":["The extension and all actions in it","l'extension ainsi que toutes les actions associees","la extension asi como todas las acciones asociadas"]
                    }

    def get_asterisk_directory(self,direc):
        res=self.ASTERISK_DIRECTORIES[direc].strip()
        if res[len(res)-1]!='/':
            return res+'/'
        else:
            return res
                                      
                    
#__________________________________
# CONSTRUCTOR OF GLOBAL INSTANCE 
GLOBAL = GlobalClass()
#__________________________________

DICO={"MSG_CONFIRM_QUIT" : {"en":"Are you sure to want to quit ACTOS ? All changes not exported will be losed !",
                            "fr":"Etes-vous sur de vouloir quitter ACTOS ? Tous les changements non exports seront perdus !",
                            "es":"Esta seguro de querer salir de ACTOS ? Todos los cambios cuales no han sido exportados seran perdidos!"
                            },

      "MSG_CONFIRM_IMPORT":{"en":"Do you wish to import current ASTERISK config from :\n%0% ?",
                            "fr":"Voulez-vous importer la configuration ASTERISK depuis :\n%0% ?",
                            "es":"Quieren importar la configuracion ASTERISK despues :\n%0% ?"
                            },
      "MSG_CONFIRM_DELETE_X":{"en":"%0% will be deleted! Are you sure ?",
                              "fr":"Voulez-vous vraiment supprimer %1% ?",
                              "es":"Quiere realmente suprimir %2% ?"
                                 },
      "MSG_CONFIRM_DELETE_USER":{"en":"The user will be deleted! Are you sure?",
                                 "fr":"Voulez-vous vraiment supprimer l'utilisateur ?",
                                 "es":"Quiere realmente suprimir el usuario ?"
                                 },
      "MSG_CONFIRM_DELETE_VAR":{"en":"The variable %0% will be deleted! Are you sure?",
                                "fr":"Voulez-vous vraiment supprimer la variable %0% ?",
                                "es":"Quiere realmente suprimir la variabile %0% ?"
                                 },
      "MSG_CONFIRM_DELETE_CONTEXT":{"en":"The context will be deleted! Are you sure?",
                                    "fr":"Voulez-vous vraiment supprimer le contexte ?",
                                    "es":"Quiere realmente suprimir el contesto ?"
                                 },
      "MSG_CONFIRM_DELETE_PROTOCOL":{"en":"Disable management of %0% ?",
                                     "fr":"Voulez-vous vraiment supprimer le protocole %0% ?",
                                     "es":"Quiere realmente suprimir el protocollo %0% ?"
                                 },
      "MSG_NB_FOUND_ACTIONS":{"en":"Found %0% actions to load in most-used list",
                              "fr":"Chargement de %0% actions",
                              "es":"%0% accion leidas"
                              },
      "MSG_QUIT":{"en":" \nThank you for using ACTOS",
                  "fr":" \nMerci d'utiliser ACTOS",
                  "es":" \nGracias por utilizar ACTOS"
                  },
      "MSG_CREATE_WINDOW":{"en":"Create window %0%",
                           "fr":"Cration de la fentre %0%",
                           "es":"Creacion de la ventana %0%"
                           },
      "MSG_HIDE_WINDOW":{"en":"Hiding window %0%",
                         "fr":"Masquage de la fentre %0%",
                         "es":"Ocultacion de la ventana %0%"
                         },
      "MSG_DESTROY_WINDOW":{"en":"Destroying window %0%",
                            "fr":"Destruction de la fentre %0%",
                            "es":"Destruccion de la ventana %0%"
                            },
      "MSG_CLEAN_DATAS":{"en":"You are going to import a new ASTERISK configuration, this will erase current data, are you sure ? (if not, change the import parameters)",
                         "fr":"Vous allez importer une nouvelle configuration ASTERISK et perdre les donnes courantes, tes-vous sr ? (si non, changez les paramtres d'importation)",
                         "es":"Los datos corrientes van a estar suprimidos, esta seguro ? (si no, cambias las opciones de importacion)"
                         },
      "MSG_CONF_LOADED":{"en":"Configuration loaded\n Found %0% user(s)/peer(s)/agent(s) and %1% context(s)\n Found %2% conference room(s)",
                         "fr":"""Importation termine\n %0% utilisateur(s)/poste(s)/agent(s) et %1% contexte(s) d'appel chargs\n + %2% salle(s) de confrence"""
                         },
      "MSG_CONF_SAVED":{"en":"Configuration saved",
                         "fr":"Exportation termine"
                         },
      "ERROR_SAVE_ACTIONS":{"en":"Could not save most used actions",
                            "fr":"Impossible de sauvegarder les actions"
                            },
      "ERROR_CANT_COPY":{"en":"Could not make a copy of file %0%, verify that you have permissions",
                         "fr":"Impossible de copier le fichier %0%, vrifiez les permissions"
                         },
      "ERROR_MUST_DEFINE_REGISTER":{"en":"You must define %0% for register with this peer",
                                    "fr":"Vous devez entrer %1% pour vous enregistrer avec ce poste"
                                    },
      "ERROR_AUTH_SECRET":{"en":"You must enter a password for this type of authentification",
                           "fr":"Vous devez entrer un mot de passe avec ce type d'authentification"
                           },
      "ERROR_LOAD_WINDOW":{"en":"Window %0% not found in GLADE file",
                           "fr":"Fenetre %0% non trouvee dans le fichier GLADE"
                           },
      "ERROR_MISSING_EVENT":{"en":"Missing event %0% in glade file",
                             "fr":"Evenement %0% non trouve dans le fichier GLADE"
                             },
      "ERROR_NO_PARENT":{"en":"Window has no parent",
                         "fr":"La fentre n'a pas de parent"
                        },
      "ERROR_NO_SELECTION":{"en":"You must first select an item of the list",
                            "fr":"Slectionnez d'abord un lement de la liste"
                            },
      "ERROR_INVALID_VALUE":{"en":"%0% value is invalid !",
                             "fr":"La valeur de %0% est invalide !"
                             },
      "ERROR_NO_WIZARD":{"en":"Sorry, no wizard available for this function at the moment... You have to enter the parameters manually or click OK if you don't want any parameters",
                         "fr":"Pas encore d'assistant pour cette fonction... Vous devez entrer manuellement les paramtres ou cliquez sur VALIDER pour laisser tel quel"
                         },
      "ERROR_EXT_ALREADY_EXISTS":{"en":"This extension already exists in the context. Please enter another extension, or edit the existing one",
                                  "fr":"Cette extension existe deja dans le contexte d'appel, entrez une autre extension ou editez celle qui existe"
                                  },
      "ERROR_CTXT_ALREADY_EXISTS":{"en":"The context already exists. Choose another name",
                                   "fr":"Le contexte existe deja, choisissez un autre nom"
                                   },
      "ERROR_VAR_ALREADY_EXISTS":{"en":"The variable already exists. Choose another name, or edit the existing one",
                                   "fr":"La variable existe deja, choisissez un autre nom ou editez celle qui existe"
                                   },
      "ERROR_MODIFY_PROTOCOL":{"en":"Failed to modify protocol [%0%][%1]",
                               "fr":"Impossible de modifier le protocole [%0%][%1%]"
                               },
      "ERROR_ACTOS_CONF_NOT_FOUND":{"en":"actos.conf not found ! Creating new file...",
                                    "fr":"actos.conf non trouve ! Creation d'un nouveau fichier..."
                                    },
      "ERROR_FILE":{"en":"Could not open file %0%. If the file exists, check permissions.",
                    "fr":"Impossible d'ouvrir le fichier %0%. Si le fichier existe, vrifiez les permissions."
                    },
      "ERROR_FILE_PROTOCOL":{"en":"The file %0% didn't match with any known protocol. Please contact development crew at neoh@bonbon.net",
                    "fr":"Le fichier %0% ne correspond  aucun protocole connu. Merci de contacter l'quipe de dveloppement  neoh@bonbon.net"
                    },
      "ITEM_ERROR":{"en":"ON ERROR",
                   "fr":"CAS D'ERREUR"
                   },
      "ERROR_SLASH":{"en":"The password field can't contain character '/'",
                   "fr":"Le caractre '/' est invalide dans le champ mot de passe"
                   },
      "ERROR_EMPTY_FIELD":{"en":"The field can't be empty !",
                           "fr":"Le champ ne peut pas tre vide !"
                   }
      }

def _(msgid,args=[]):
	#translation function

    lang=GLOBAL.GLOBAL_PARAMS["language"]
    if not msgid in DICO.keys():
        return "% Unknown message %"
    
    if not lang in DICO[msgid]:
        lang="en"

    msg=DICO[msgid][lang]
    cpt=0
    for a in args:
        pattern="%"+str(cpt)+"%"
        msg=msg.replace(pattern,args[cpt])
        cpt+=1
        
    return msg




#config samples for protocols
#    "sip" : {"fichier":"sip.conf",
#                         "port":"5060",
#                         "bindaddr":"0.0.0.0",
#                         "tos":"none",
#                         "codecs" : None
#                        },
#                "iax2" : {"fichier":"iax.conf",
#                         "port":"5036",
#                         "bindaddr":"0.0.0.0",
#                         "tos":"none",
#                         "jitterbuffer":"yes",
#                         "dropcount":"3",
#                         "maxjitterbuffer":"500",
#                         "maxexcessjitterbuffer":"100",
#                         "codecs" : None
#                        },
#                "h323": {"fichier":"h323.conf",
#                         "port":"1720",
#                         "bindaddr":"0.0.0.0",
#                         "tos":"none",
#                         "codecs" : None
#                        }
#               }

