#!/usr/bin/python
# -*- coding: iso-8859-1 -*-

#_______________________________________________________________
# ACTOS
# Asterisk Configuration Tool Open Source
# (c) EOLE - Mai 2004 - Pierre Derrier
#_______________________________________________________________
#    This program is free software; you can redistribute it and/or modify
#    it under the terms of the GNU General Public License as published by
#    the Free Software Foundation; either version 2 of the License, or
#    any later version.
#
#    This program is distributed in the hope that it will be useful,
#    but WITHOUT ANY WARRANTY; without even the implied warranty of
#    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#    GNU General Public License for more details.
#
#   You should have received a copy of the GNU General Public License
#    along with this program; if not, write to the Free Software
#    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
#____________________________________________________________________________
# TOOLS.PY : SOME OTHER TOOLS
#_______________________________________________________________
# Contains :
#            FUNCTION Update_protocol (protocol, keys)
#                   Put keys values in GLOBAL.PROTOCOLS
#
#            FUNCTION Build_liste(widget_liste,colonnes[,resize]):
#                   Create columns in a TreeView widget
#
#            CLASS GladeComboContainer
#                   Container object for GTK combos + entry fields associated
#
#            CLASS GladeGroupContainer
#                   Container object for associate one or more glade objects
#
#            ..... and others functions
#_______________________________________________________________

#_______________________________________________________________
# EXTERN DEPENDENCIES
#_______________________________________________________________
import Interface
import sys
from Global import GLOBAL,_
import copy
import string
import os

try:
    import datetime
except:
    import sys
    print "Module 'datetime' can't be loaded : check your Python version (2.3 or higher required)"
    sys.exit(1)

#_______________________________________________________________


#________________________________________________
def debug_init(val):
#________________________________________________
    if val.strip()=="":
        val="none"
    debug("Log mode : "+val)
    GLOBAL.GLOBAL_PARAMS["logtype"]=val
    if val.find("file")>-1:
        try:
            GLOBAL.LOGFILE=open(GLOBAL.PREFIXDIR+"actos.log","a")
        except:
            print "-- Error : can't open log file; switching to screen mode"
            GLOBAL.GLOBAL_PARAMS["logtype"]="(screen)"
            return
    
        a=datetime.datetime
        time=a.ctime(a.now())

        GLOBAL.LOGFILE.write("------------[ "+time+" ]---------\n")
        GLOBAL.LOGFILE.write("Log mode : "+val+"\n")
        
    else:
        if GLOBAL.LOGFILE!=None:
            GLOBAL.LOGFILE.close()
#________________________________________________
        
        

#________________________________________________
def debug_close():
#________________________________________________
    g=GLOBAL.GLOBAL_PARAMS["logtype"]

    if GLOBAL.LOGFILE!=None:
        GLOBAL.LOGFILE.close()
#________________________________________________


#________________________________________________
def debug(objet):
#________________________________________________
    texte=str(objet)
    g=GLOBAL.GLOBAL_PARAMS["logtype"]
    if g.find("screen")>-1:
        print texte
    if g.find("file")>-1 and GLOBAL.LOGFILE!=None:
        GLOBAL.LOGFILE.write(texte+"\n")
    # TIPS : for showing a dialog box : begin message with '-- Error :'
    if texte[0:10]=="-- Error :":
        Interface.DialogBox(texte=texte[10:],type_dialog=Interface.gtk.MESSAGE_ERROR)
    elif texte[0:12]=="-- Warning :":
        Interface.DialogBox(texte=texte[12:],type_dialog=Interface.gtk.MESSAGE_WARNING)
    elif texte=="-- TODO --":
        Interface.DialogBox("Sorry, this function is not available for the moment.\nIf you are interested in the development of ACTOS, see :\nhttp://www.voip-info.org/wiki-ACTOS")
#________________________________________________


#_______________________________________________________________
def HelpWindow(glade,windowname):
    """ show help window """
#_______________________________________________________________

    def hidehelp(wid,fen,*args):
        fen.hide()

    fen=glade.get_widget("Fen_help")
    wid=glade.get_widget("text_help")
    try:
        fh=open(GLOBAL.PREFIXDIR+GLOBAL.GLOBAL_PARAMS["language"]+".help","r")
    except:
        return

    lines=fh.readlines()
    fh.close()

    stri=""
    findcurrent=False
    for l in lines:
        l=l.strip()
        if len(l)>0:
            if not findcurrent:
                if l[0]=="[":
                    l=l.rstrip("]").lstrip("[")
                    if l==windowname:
                        findcurrent=True
            else:
                if l[0]=="[":
                    break
                else:
                    stri+=l+"\n"
        else:
            stri+="\n"

    if not findcurrent:
        stri=" NO HELP DEFINED FOR THIS WINDOW.\n PLEASE EDIT THE FILES .HELP IN ACTOS DIRECTORY"
    stri=unicode(stri,"iso-8859-1")
    stri=stri.encode("UTF-8")
    wid.set_text(stri)
    glade.signal_autoconnect({"on_close_help":(hidehelp,fen)})
    fen.present()
#_______________________________________________________________
        

#_______________________________________________________________
def GetScriptsDic():
    """ Get all scripts from scripts/ directory """
#_______________________________________________________________
    list=os.listdir(GLOBAL.GLOBAL_PARAMS["scriptsdir"])
    namedic={}
    for file in list:
        if file.strip().rfind(".actos")==len(file.strip())-6:
            filename=GLOBAL.GLOBAL_PARAMS["scriptsdir"]+file
            fh=open(filename,"r")
            lines=fh.readlines()
            fh.close()
            for line in lines:
                line=line.strip()
                if len(line)>0:
                    if line[0]=="[" and line[len(line)-1]=="]":
                        line=line.lstrip("[")
                        line=line.rstrip("]")
                        namedic[line]=filename
    
    return namedic
#_______________________________________________________________


#_______________________________________________________________
def GetScript(file,script):
    """ get script from an actos file"""
#_______________________________________________________________
    script="["+script+"]"
    
    fh=open(file,"r")
    lines=fh.readlines()
    fh.close()

    extract=False
    liste=[]
    for line in lines:
        line=line.strip()
        if len(line)>0:
            if line==script:
                extract=True
            else:
                if extract:
                    if line[0]=="[" and line[len(line)-1]=="]":
                        extract=False
                        break
                    else:
                        liste.append(line)
                
    return liste
#_______________________________________________________________



#_______________________________________________________________
def SwapRows(model,iter1,iter2):
    #exchange two rows in a treemodel
#_______________________________________________________________
    row1 = model[iter1]
    row2 = model[iter2]

    for p in range(len(row1)):
        val=row1[p]
        row1[p]=row2[p]
        row2[p]=val
#_______________________________________________________________


#_______________________________________________________________
def ExtractPriority(model,iter):
    #extract priority from a treemodel line wich has the format:
    #"priority :: xxxxxxxxxxxxxx"
    #return -1 if the line begins by "--[" (error field)
#_______________________________________________________________
    currow=model[iter][0]
    if currow[0:2]=="--[":
        return -1
    currow=currow.split("::")
    return int(currow[0].strip())
#_______________________________________________________________


#_______________________________________________________________
def SetPriority(model,iter,prio):
    #change priority in a treemodel line with the format:
    #"priority :: xxxxxxxxxxxxxx"
#_______________________________________________________________
    currow=model[iter][0].split("::")
    model.set_value(iter,0,str(prio)+" :: "+currow[1].strip())
#_______________________________________________________________


#_______________________________________________________________
def GetCurrentLanguageName():
    #get the real name of the current language
#_______________________________________________________________
    curlang=GLOBAL.GLOBAL_PARAMS["language"]
    dic=GLOBAL.LANGUAGES[curlang]
    if curlang not in dic.keys():
        return "Unknown language"
    else:
        return dic[curlang]
#_______________________________________________________________



#_______________________________________________________________
def GetLanguageAbrevFromName(name):
    #get the abreviation of the language from the full name
#_______________________________________________________________
    curdic=GLOBAL.LANGUAGES[GLOBAL.GLOBAL_PARAMS["language"]]
    for l in curdic.keys():
        if curdic[l]==name:
            return l
#_______________________________________________________________



#_______________________________________________________________
def GetLanguagesList(curlang):
    #return the list of supported languages
#_______________________________________________________________
    return GLOBAL.LANGUAGES[curlang].values()
#_______________________________________________________________
    


#_______________________________________________________________
def SetProtocolKey(protocol,key,value):
    """ update keys for protocol"""
#_______________________________________________________________
    if not (protocol in GLOBAL.PROTOCOLS.keys()):
        GLOBAL.PROTOCOLS[protocol]={}
    GLOBAL.PROTOCOLS[protocol][key]=value
#_______________________________________________________________



#_______________________________________________________________
def GetProtocolsSupported():
    """ Return list of supported protocols"""
#_______________________________________________________________
    if "protocols" not in GLOBAL.GLOBAL_PARAMS.keys():
        GLOBAL.GLOBAL_PARAMS["protocols"]=""
        return []
    l=GLOBAL.GLOBAL_PARAMS["protocols"].split("##")
    dic={}
    for p in l:
        str=p.split("/")
        dic[str[0]]=str[1]
    return dic
#_______________________________________________________________



#_______________________________________________________________
def InitDefaultsProtocol(p):
    """ Create skeleton of protocol"""
#_______________________________________________________________
    GLOBAL.PROTOCOLS[p]={}
    GLOBAL.PROTOCOLS[p]["fichier"]=p+".conf"
    GLOBAL.PROTOCOLS[p]["port"]=""
    GLOBAL.PROTOCOLS[p]["bindaddr"]="0.0.0.0"
    GLOBAL.PROTOCOLS[p]["tos"]="none"
    GLOBAL.PROTOCOLS[p]["codecs"]=None
    if p=="iax2":
        GLOBAL.PROTOCOLS[p]["jitterbuffer"]="yes"
        GLOBAL.PROTOCOLS[p]["dropcount"]="3"
        GLOBAL.PROTOCOLS[p]["maxjitterbuffer"]="500"
        GLOBAL.PROTOCOLS[p]["maxexcessjitterbuffer"]="100"
#_______________________________________________________________

    

#_______________________________________________________________
def IsProtocolSupported(p):
    """ return True if protocol is supported, else False"""
#_______________________________________________________________
    i=GLOBAL.GLOBAL_PARAMS["protocols"].find(p)
    if i==-1:
        return False
    else:
        return True
#_______________________________________________________________



#_______________________________________________________________
def ParseRegisterString(string):
    """ return login, password,host from register string"""
#_______________________________________________________________
    string=string.split(":")
    login=string[0].strip()
    string=string[1].split("@")
    passw=string[0].strip()
    string=string[1].split("/")
    host=string[0].strip()
    return login,passw,host
#_______________________________________________________________

    

#_______________________________________________________________
def Update_protocol(currentprotocol,keys):
    """ update protocol with keys list"""
#_______________________________________________________________
    debug("# Building protocol "+currentprotocol)

    if "register" in keys.keys():
        GLOBAL.TOREGISTER[currentprotocol]=[]
        peers=keys["register"].split("##")
        for p in peers:
            GLOBAL.TOREGISTER[currentprotocol].append(p)
        del(keys["register"])

    listeCodecs=[]
    #if key "bandwidth" is present, get the list of codec for the bandwidth selected
    if "bandwidth" in keys.keys():
        listeCodecs=copy.deepcopy(GLOBAL.CODECS[keys["bandwidth"].strip()])
    if "disallow" in keys.keys() and keys["disallow"]=="all":
        listeCodecs=[]
        del(keys["disallow"])
        
    #iterate the keys
    for k in keys.keys():
        #creation of the codec list
        if k=="allow":
            kkeys=keys[k].split("##")
            for kk in kkeys:
                if kk=="all":
                    listeCodecs=copy.deepcopy(GLOBAL.CODECS["high"])
                else:
                    listeCodecs.append(kk)
        elif k=="disallow":
            kkeys=keys[k].split("##")
            for kk in kkeys:
                if kk in listeCodecs:
                    listeCodecs.remove(kk)
        else:
            #else save key
            kkeys=keys[k].split("##")
            if len(kkeys)>1:
                debug("-- Warning : While parsing conf file of "+currentprotocol+" protocol, found "+str(len(kkeys))+" values for '"+str(k)+"' :\n"+str(kkeys)+"\n\nThe value used will be "+str(kkeys[0]))
            GLOBAL.PROTOCOLS[currentprotocol][k]=kkeys[0]

    #update de la liste des codecs
    if len(listeCodecs)==0:
        listeCodecs=None
    GLOBAL.PROTOCOLS[currentprotocol]["codecs"]=copy.deepcopy(listeCodecs)

    debug(GLOBAL.PROTOCOLS[currentprotocol])
#_______________________________________________________________


#_______________________________________________________________
def ExtractKeyAndValue(i):
    """ return key and value from a string 'key = value' or 'key => value'"""
#_______________________________________________________________
    val=""
    line = i.split("=")
    key= line[0].strip()
    if len(line)>1:
        val=line[1].strip()
        val=val.lstrip(">").strip()
    return key,val
#_______________________________________________________________


#_______________________________________________________________
def DeleteKeysIfPossible(dic,keys):
    """ delete a key from a dic if possible"""
#_______________________________________________________________
    for k in keys:
        try:
            del(dic[k])
        except:
            debug("-- Notice for debugging : failed to delete key '"+k+"'")
#_______________________________________________________________

#_______________________________________________________________
def WriteAndDeleteKeysIfPossible(fh,dic,keys):
    """ write key in file and delete it in a dic if possible"""
#_______________________________________________________________
    for k in keys:
        try:
            if dic[k]!="":
                fh.write(k+" = "+dic[k]+"\n")
            del(dic[k])
        except:
            debug("-- Notice for debuggging : failed to write and/or delete key '"+k+"'")
#_______________________________________________________________

#________________________________________________
def cell_edited_cb(cell_renderer, path, text, model):
#________________________________________________
    _iter = model.get_iter_from_string(path)
    model.set(_iter, 0, text)
#________________________________________________

#________________________________________________
def Build_liste(widget_liste,colonnes,resize=False,ceditable=None,model=None):
    """ Build a list with the specified columns"""
#________________________________________________
    debug("Creating list...")
    #Cration des colonnes de la liste Contextes
    renderer = Interface.gtk.CellRendererText()
    if ceditable!=None:
        renderer.set_property("editable",Interface.gtk.TRUE)
        renderer.connect("edited", cell_edited_cb, model)
    j=0
    for c in colonnes:
        if ceditable!=None:
            column = Interface.gtk.TreeViewColumn(c, renderer, text=j,editable=ceditable)
        else:
            column = Interface.gtk.TreeViewColumn(c, renderer, text=j)
        column.set_resizable(resize)
        widget_liste.append_column(column)
        j+=1
    widget_liste.show()
#_______________________________________________________________


#_______________________________________________________________
def ItemExistsInList(ext,model,maxlevel=None,exception=None):
    """ return True if item is in model, can define en exception"""
#_______________________________________________________________
    def verify(model, path, iter, liste):
        #function called for each row
        #verify that the value isn't in list
        if maxlevel!=None and len(path)>maxlevel:
            return False
        #prends en compte l'exception
        if iter!=None and exception!=None and model.get_path(iter)==model.get_path(exception):
            return False
        val=str(model.get_value(iter,0)).strip()
        if val in liste:
            liste.append("__#ERROR#__")
            return True
        else:
            liste.append([val])
            return False

    l=[ext.strip()]
    model.foreach(verify,l)
    if "__#ERROR#__" in l:
        return True
    else:
        return False
    
#_______________________________________________________________

#_______________________________________________________________
def GetListFromModel(model,col=0):
    """ get a list from a model"""
#_______________________________________________________________
    def addinlist(model, path, iter, liste):
        #function called for each row
        val=model.get_value(iter,col).strip()
        liste.append(str(val))
        return False

    l=[]
    model.foreach(addinlist,l)
    return l
#_______________________________________________________________


#_______________________________________________________________
def SortContexts(liste,reverse=False):
    """ sort in alphabetical  order"""
#_______________________________________________________________
    debug("Sorting contexts in alphabetical order")
    listeres=[]
    for context in liste:
        i=0
        while i<len(listeres) and (context.name>listeres[i].name):
            i+=1
        if i<len(listeres):
            listeres.insert(i,context)
        else:
            listeres.append(context)

    if reverse:
        listeres.reverse()
            
    return listeres
#_______________________________________________________________



#_______________________________________________________________
def SeekParent(obj,pname):
    """ return first parent that have the desired name"""
#_______________________________________________________________
    try:
        while obj.parent!=None and obj.parent.name!=pname:
            obj=obj.parent
        return obj.parent
    except:
        debug("-- Error : can't find parent '"+pname+"' of object")
        return None
#_______________________________________________________________



#_______________________________________________________________
def import_defs():
#_______________________________________________________________
    try:
        fh=open(GLOBAL.PREFIXDIR+"actions.def","r")
    except:
        debug("-- Error : Can't open "+GLOBAL.PREFIXDIR+"actions.def !")
        return False

    lines=fh.readlines()
    fh.close()

    stri=""
    curcatname=""
    nbcat=0
    nbact=0
    GLOBAL.ACTIONS={}
    GLOBAL.ACTIONS_WITHOUT_PARAMS=[]
    for l in lines:
        l=l.strip()
        if len(l)>0:
            if l[0]=="[":
                l=l.rstrip("]").lstrip("[")
                curcatname=l
                GLOBAL.ACTIONS[curcatname]={}
                nbcat+=1
            else:
                temp=l.split("##")
                nomcom=temp[0].strip()
                params=temp[1].strip()
                desc=temp[2].strip()
                GLOBAL.ACTIONS[curcatname][nomcom]="Syntax : "+nomcom+"("+params+")\nDescription : "+desc
                nbact+=1
                if len(params)==0:
                    GLOBAL.ACTIONS_WITHOUT_PARAMS.append(string.lower(nomcom))

    debug("Imported "+str(nbcat)+" categories and "+str(nbact)+" actions from actions.def")


    try:
        fh=open(GLOBAL.PREFIXDIR+"channelvars.def","r")
    except:
        debug("-- Error : Can't open "+GLOBAL.PREFIXDIR+"channelvars.def !")
        return False

    lines=fh.readlines()
    fh.close()

    stri=""
    nbact=0
    GLOBAL.CHANNEL_VARS={}
    for l in lines:
        l=l.strip()
        if len(l)>0:
            temp=l.split("##")
            nomcom=temp[0].strip()
            desc=temp[1].strip()
            GLOBAL.CHANNEL_VARS[nomcom]=desc
            nbact+=1

    debug("Imported "+str(nbact)+" variables from channelvars.def")

    return True
#_______________________________________________________________






#_______________________________________________________________
class GladeComboContainer:
#_______________________________________________________________
    def __init__(self,glade,name):
        self.combo=glade.get_widget("combo_"+name)
        self.entry=glade.get_widget("entry_"+name)
        self.items=[]

    def get_text(self):
        return self.entry.get_text().strip()

    def set_text(self,text):
        text=unicode(text,"iso-8859-1")
        text=text.encode("UTF-8")
        self.entry.set_text(text)

    def fill(self,with=None):
        if with!=None:
            self.items=with
            
        if len(self.items)>0:
            #encodage->UTF8
            templ=[]
            for i in self.items:
                i=unicode(i,"iso-8859-1")
                i=i.encode("UTF-8")
                templ.append(i)
            self.combo.set_popdown_strings(templ)
        else:
            self.combo.set_popdown_strings([""])


    def getNbItems(self):
        return len(self.items)


    def clear(self):
        del(self.items)
        self.items=[]
        self.combo.children=[]
        


    def set_sensitive(self,state):
        self.combo.set_sensitive(state)
        self.entry.set_sensitive(state)

    def get_sensitive(self):
        return self.combo.get_sensitive()

    def activate(self):
        self.set_sensitive(True)

    def desactivate(self):
        self.set_sensitive(False)

    def add_item(self,val,updateCombo=True):
        self.items.append(val)
        if updateCombo:
            self.fill(None)


    def show(self):
        self.combo.show()
        self.entry.show()

    def hide(self):
        self.combo.hide()
        self.entry.hide()
#_______________________________________________________________



#_______________________________________________________________
class GladeGroupContainer:
#_______________________________________________________________

     #_______________________________________________________________
     def __init__(self,listewidgets=[]):
     #_______________________________________________________________
         self._wid=listewidgets

     #_______________________________________________________________
     def add(self,widget):
     #_______________________________________________________________
         self._wid.append(widget)
     #_______________________________________________________________

     #_______________________________________________________________
     def set_sensitive(self,state):
     #_______________________________________________________________
         for w in self._wid:
             w.set_sensitive(state)
     #_______________________________________________________________


     #_______________________________________________________________
     def activate(self):
     #_______________________________________________________________
         self.set_sensitive(True)
     #_______________________________________________________________

     #_______________________________________________________________
     def desactivate(self):
     #_______________________________________________________________
         self.set_sensitive(False)
     #_______________________________________________________________
     #_______________________________________________________________
     def hide(self):
     #_______________________________________________________________
         for w in self._wid:
             w.hide()
     #_______________________________________________________________
     #_______________________________________________________________
     def show(self):
     #_______________________________________________________________
         for w in self._wid:
             w.show()
     #_______________________________________________________________

#_______________________________________________________________
         



#_______________________________________________________________
# IF SOMEONE LAUNCH THIS SCRIPT.... 
#_______________________________________________________________
if __name__ == "__main__":
    Interface.AboutMessage()
    print "Please launch actos.py\n"
#_______________________________________________________________
