#!/usr/bin/python
# -*- coding: iso-8859-1 -*-

#_______________________________________________________________
# ACTOS
# Asterisk Configuration Tool Open Source
# Copyright (c) EOLE - Mai 2004 - Pierre Derrier
#_______________________________________________________________
#    This program is free software; you can redistribute it and/or modify
#    it under the terms of the GNU General Public License as published by
#    the Free Software Foundation; either version 2 of the License, or
#    any later version.
#
#    This program is distributed in the hope that it will be useful,
#    but WITHOUT ANY WARRANTY; without even the implied warranty of
#    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#    GNU General Public License for more details.
#
#   You should have received a copy of the GNU General Public License
#    along with this program; if not, write to the Free Software
#    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
#____________________________________________________________________________
# WINDOWS.PY : GRAPHIC USER INTERFACE
#_______________________________________________________________
# Contains :
#            CLASS WindowCtrl
#                   Generic controller for a GTK window
#
#            CLASS WXXXCtrl (herited for WindowCtrl)
#                   Controller for XXX window
#
#_______________________________________________________________

#_______________________________________________________________
# EXTERN DEPENDENCIES
#_______________________________________________________________
from Global import GLOBAL,_
import Tools
import Filesystem
import User
import Context
import Com

#_______________________________________________________________


#_______________________________________________________________
class WindowCtrl:
# ABSTRACT CLASS FOR WINDOWS
# Define the behavior of a window if the methods are not redefined
#_______________________________________________________________

    #________________________________________________
    def __init__(self,iname,parent=None):
    #________________________________________________
        self.name=iname
        self.parent=parent
        self.children={}
        self.glade=parent.glade
        self.mode="NORMAL"
        self.ASSOCIATED_MAIN_BT=None

        try:
            Tools.debug(_("MSG_CREATE_WINDOW",[iname])+" (parent = "+self.parent.name+")")
        except:
            Tools.debug(_("MSG_CREATE_WINDOW",[iname])+" (Warning : no parent)")

        try:
            self.widget=self.glade.get_widget(iname)
            if self.widget==None:
                Tools.debug("-- Error : "+_("ERROR_LOAD_WINDOW",[self.name]))
                self.widget=self.parent.widget
                return
        except:
            Tools.debug("-- Error : "+_("ERROR_LOAD_WINDOW",[self.name]))
            return


        self.widget.set_transient_for(parent.widget)
        self.widget.set_position(Tools.Interface.gtk.WIN_POS_CENTER_ON_PARENT)

        #Connect appropriate delete event
        #the event must be called "on_[Name of the glade window]_delete_event"
        try:
            if iname=="Fen_main":
                #connect to close function
                self.glade.signal_autoconnect({"on_help_Fen_main":self.showHelp,"on_Fen_main_delete_event" : self.close})
            else:
                #connect to clic_on_x function and hide
                self.glade.signal_autoconnect({"on_help_"+iname:self.showHelp,"on_"+iname+"_delete_event" : self.clic_on_x})
                self.hide()
        
        except:
            Tools.debug("-- Error : "+_("ERROR_MISSING_EVENT",["on_"+iname+"_delete_event"]))
    #________________________________________________


    #________________________________________________
    def hide(self,*args):
    #________________________________________________
        #Tools.debug("Hiding window "+self.name)
        self.widget.hide()
    #________________________________________________

    #________________________________________________
    def show(self,*args):
    #________________________________________________
        #Tools.debug("Showing window "+self.name)
        self.widget.present()
    #________________________________________________


    #________________________________________________
    def update(self,*args):
    #________________________________________________
        if self.ASSOCIATED_MAIN_BT!=None:
            self.ASSOCIATED_MAIN_BT.set_sensitive(False)

        self.show()
    #________________________________________________

    #________________________________________________
    def clic_on_x(self,widget,*args):
    #________________________________________________
        if widget!=None:
            Tools.debug(_("MSG_HIDE_WINDOW",[widget.name])+" (delete_event) ")
            widget.hide()
            self.update_parent()
            #!!! IMPORTANT !!!!! 
            return Tools.Interface.gtk.TRUE
    #________________________________________________


    #________________________________________________
    def update_parent(self,*args):
    #________________________________________________
        if self.ASSOCIATED_MAIN_BT!=None:
            self.ASSOCIATED_MAIN_BT.set_sensitive(True)
            
        if self.parent!=None:
            self.parent.update()
        else:
            Tools.debug("-- Warning : "+_("ERROR_NO_PARENT"))
    #________________________________________________

        
    #________________________________________________
    def close(self,widget=None,event=None,kill=False,updatep=True):
    #________________________________________________
        if kill==True:
            for win in self.children:
                self.children[win].close(kill=True)
            Tools.debug(_("MSG_DESTROY_WINDOW",[self.name]))
            self.widget.destroy()
        else:
            for win in self.children:
                self.children[win].close(kill=False,updatep=False)
            self.hide()
            if updatep:
                self.update_parent()
    #________________________________________________


    #________________________________________________
    def valide(self,*args):
    #________________________________________________
        #Si la mthode n'est pas dfinie pour la classe fille, on affiche
       #le message TODO
        Tools.debug("-- TODO --")
    #________________________________________________

    #________________________________________________
    def showHelp(self,*args):
    #________________________________________________
        Tools.debug("Help for "+self.name)
        Tools.HelpWindow(self.glade,self.name)
        return Tools.Interface.gtk.TRUE
    #________________________________________________

#_______________________________________________________________




#----------------------------------------------------------------------------------------
#------------------------------------ FENETRE WIZARD DIAL -------------------------------
#----------------------------------------------------------------------------------------


#_______________________________________________________________
class WizardDial(WindowCtrl):
#_______________________________________________________________

    #________________________________________________
    def __init__(self,parent=None):
    #________________________________________________
        WindowCtrl.__init__(self,"Fen_wizard_dial",parent)

        self.handlers = {"on_btvalider_wizdial_clicked":self.valide,
                         "on_btannuler_wizdial_clicked":self.close,
                         "on_radio_predefpeer_wizdial_toggled":self.toggle_predefpeer,
                         "on_radio_newpeer_wizdial_toggled":self.toggle_newpeer,
                         "on_radio_defaultport_wizdial_toggled":self.toggle_defaultport,
                         "on_radio_newport_wizdial_toggled":self.toggle_newport,
                         "on_check_connectext_wizdial_toggled":self.toggle_extension,
                         "on_list_protocol_wizdial_cursor_changed":self.select_protocol,
                         "on_btajouter_wizdial_clicked":self.addtolist,
                         "on_btenlever_wizdial_clicked":self.removefromlist
                         }
        self.glade.signal_autoconnect(self.handlers)


        self.list={}
        self.radio={}
        self.entry={}
        self.check={}
        self.model={}
        
        #create lists
        for i in ["protocol","dial"]:
            self.list[i]=self.glade.get_widget("list_"+i+"_wizdial")
            Tools.Build_liste(self.list[i],[i])
            self.model[i]=Tools.Interface.gtk.ListStore(Tools.Interface.gobject.TYPE_STRING)
            self.list[i].set_model(self.model[i])
        

        #get fields
        self.radio["predefpeer"]=self.glade.get_widget("radio_predefpeer_wizdial")
        self.radio["newpeer"]=self.glade.get_widget("radio_newpeer_wizdial")
        self.radio["defaultport"]=self.glade.get_widget("radio_defaultport_wizdial")
        self.radio["newport"]=self.glade.get_widget("radio_newport_wizdial")

        self.check["extension"]=self.glade.get_widget("check_connectext_wizdial")
        for opt in ["t","T","r","m","H","C","g"]:
            self.check["option_"+opt]=self.glade.get_widget("check_option_wizdial_"+opt)

        self.entry["newpeer_host"]=self.glade.get_widget("entry_host_wizdial")
        self.entry["newpeer_username"]=self.glade.get_widget("entry_username_wizdial")
        self.entry["newpeer_password"]=self.glade.get_widget("entry_password_wizdial")
        self.entry["newport"]=self.glade.get_widget("entry_newport_wizdial")
        self.entry["timeout"]=self.glade.get_widget("entry_timeout_wizdial")
        self.entry["extension"]=self.glade.get_widget("entry_connectext_wizdial")
        self.entry["context"]=self.glade.get_widget("entry_connectctxt_wizdial")
        
        self.groupNewpeer=Tools.GladeGroupContainer([self.glade.get_widget("vbox_newpeer_wizdial")])
        self.comboPeer=Tools.GladeComboContainer(self.glade,"predefpeer_wizdial")

        self.groupNewport=Tools.GladeGroupContainer([self.entry["newport"]])
        self.groupExtension=Tools.GladeGroupContainer([self.entry["extension"],
                                                       self.entry["context"],
                                                       self.glade.get_widget("label_connectctxt_wizdial"),
                                                       self.glade.get_widget("label_connectext_wizdial")])
      
        self.btValider=self.glade.get_widget("btValider_wizdial")
        self.EditedProtocol=""
    #________________________________________________


    #________________________________________________
    def create_paramstring(self):
        """ create param string from fields and options"""
    #________________________________________________
        chaine=""
        if self.radio["predefpeer"].get_active():
            chaine=self.comboPeer.get_text()
        else:
            chaine=self.entry["newpeer_host"].get_text()

        if self.radio["newport"].get_active():
            chaine=chaine+":"+self.entry["newport"].get_text().strip()

        username=self.entry["newpeer_username"].get_text().strip()
        
        if self.EditedProtocol in ["iax","iax2"]:
            #iax format
            if self.radio["newpeer"].get_active():
                if username!="":
                    passw=self.entry["newpeer_password"].get_text().strip()
                    if passw!="":
                        username=username+":"+passw
                    chaine=username+"@"+chaine

            if self.check["extension"].get_active():
                chaine=chaine+"/"+self.entry["extension"].get_text()
                cont=self.entry["context"].get_text().strip()
                if cont!="":
                    chaine=chaine+"@"+cont

            chaine="IAX/"+chaine
        
        elif self.EditedProtocol=="sip":
            #sip format
            if self.check["extension"].get_active():
                chaine=self.entry["extension"].get_text()+"@"+chaine
            else:
                if self.radio["newpeer"].get_active() and username!="":
                    chaine=username+"@"+chaine

            chaine="SIP/"+chaine

        else:
            #other format
            Tools.debug("Wizard not available for this protocol")
            Tools.debug("-- TODO --")
            chaine=Tools.string.upper(self.EditedProtocol)+"/"+chaine
            
        return chaine
    #________________________________________________


    #________________________________________________
    def select_protocol(self,*args):
        """ handler when click on protocol list"""
    #________________________________________________
        #Rcupration de la slection
        selected,iter=self.list["protocol"].get_selection().get_selected()
        if iter==None:
            Tools.debug("-- Error : "+_("ERROR_NO_SELECTION"))
            return

        self.EditedProtocol=self.model["protocol"].get_value(iter,0)

        listeUsers=[]
        for u in GLOBAL.ListeUsers:
            if u.protocol==self.EditedProtocol:
                listeUsers.append(u.name)
        self.comboPeer.fill(listeUsers)

        if len(listeUsers)==0:
            self.radio["newpeer"].set_active(True)
            self.radio["predefpeer"].set_sensitive(False)
            self.comboPeer.set_sensitive(False)
        else:
            self.radio["predefpeer"].set_sensitive(True)
            self.radio["predefpeer"].set_active(True)
    #________________________________________________

    #________________________________________________
    def removefromlist(self,*args):
        """ remove item from call list"""
    #________________________________________________
        model,iter = self.list["dial"].get_selection().get_selected()
        if iter is not None:
            self.model["dial"].remove(iter)
        else:
            Tools.debug("-- Error : "+_("ERROR_NO_SELECTION"))
            return
        self.list["dial"].set_model(self.model["dial"])
        self.list["dial"].show()
        if len(self.model["dial"])==0:
            self.btValider.set_sensitive(False)
    #________________________________________________

    #________________________________________________
    def addtolist(self,*args):
        """ add item to call list"""
    #________________________________________________
        str=self.create_paramstring()
        iter=self.model["dial"].insert(0,row=[str])
        self.list["dial"].set_model(self.model["dial"])
        self.list["dial"].show()
        self.btValider.set_sensitive(True)
    #________________________________________________

    #________________________________________________
    def toggle_extension(self,*args):
    #________________________________________________
        active=self.check["extension"].get_active()
        self.groupExtension.set_sensitive(active)
        if active:
            self.entry["extension"].grab_focus()
    #________________________________________________

    #________________________________________________
    def toggle_newport(self,*args):
    #________________________________________________
        self.groupNewport.activate()
        self.entry["newport"].grab_focus()
    #________________________________________________
    #________________________________________________
    def toggle_defaultport(self,*args):
    #________________________________________________
        self.groupNewport.desactivate()
    #________________________________________________

    #________________________________________________
    def toggle_predefpeer(self,*args):
    #________________________________________________
        self.comboPeer.activate()
        self.groupNewpeer.desactivate()
    #________________________________________________
    #________________________________________________
    def toggle_newpeer(self,*args):
    #________________________________________________
        self.comboPeer.desactivate()
        self.groupNewpeer.activate()
        self.entry["newpeer_host"].grab_focus()
    #________________________________________________

    #________________________________________________
    def update_parent(self,*args):
    #________________________________________________
        self.parent.show()
    #________________________________________________

    #________________________________________________
    def update(self,*args):
    #________________________________________________
        self.radio["predefpeer"].set_active(True)
        self.toggle_predefpeer()
        self.radio["defaultport"].set_active(True)
        self.toggle_defaultport()

        self.entry["timeout"].set_text("10")

        for c in self.check.keys():
            self.check[c].set_active(False)
        self.toggle_extension()

        #init protocol list
        self.model["protocol"] = Tools.Interface.gtk.ListStore(Tools.Interface.gobject.TYPE_STRING)
        dicp=Tools.GetProtocolsSupported()
        for i in dicp.keys():
            iter=self.model["protocol"].insert(0,row=[i])
        self.list["protocol"].set_model(self.model["protocol"])
        self.list["protocol"].set_sensitive(True)
        self.list["protocol"].show()

        self.model["dial"]=Tools.Interface.gtk.ListStore(Tools.Interface.gobject.TYPE_STRING)
        self.btValider.set_sensitive(False)

        self.show()
    #________________________________________________


    #________________________________________________
    def valide(self,*args):
    #________________________________________________
        #get call types&identifiers
        stri=""
        for line in self.model["dial"]:
            stri+=line[0]+"&"
        stri=stri.rstrip("&")

        #get timeout value
        try:
            timeout=int(self.entry["timeout"].get_text().strip())
        except:
            Tools.debug(_("ERROR_INVALID_VALUE",["time out"]))
            self.entry["timeout"].grab_focus()
            return

        #build option string
        options=""
        for opt in ["t","T","r","m","H","C","g"]:
            if self.check["option_"+opt].get_active():
                options+=opt
        if options!="":
            options=","+options
                
        stri+=","+str(timeout)+options

        #update parent field
        try:
            self.parent.champ["params"].set_text(stri)
            self.parent.champ["params"].grab_focus()
        except:
            raise "INTERNAL ERROR ! Problem with parent ! 'params' field not found"

        self.close()
    #________________________________________________

#_______________________________________________________________



#----------------------------------------------------------------------------------------
#------------------------------------ FENETRE WIZARD MEETME -------------------------------
#----------------------------------------------------------------------------------------


#_______________________________________________________________
class WizardMeetMe(WindowCtrl):
#_______________________________________________________________

    #________________________________________________
    def __init__(self,parent=None):
    #________________________________________________
        WindowCtrl.__init__(self,"Fen_wizard_meetme",parent)

        self.handlers = {"on_btvalider_wizmeetme_clicked":self.valide,
                         "on_btannuler_wizmeetme_clicked":self.close,
                         "on_check_wizmeetme_passwd_toggled":self.togglepasswd,
                         "on_entry_wizmeetme_conf_changed":self.changecombo
                         }
        self.glade.signal_autoconnect(self.handlers)


        #get fields
        self.check={}
        self.entry={}
        self.label={}
        self.entry["passwd"]=self.glade.get_widget("entry_wizmeetme_passwd")
        self.label["passwd"]=self.glade.get_widget("label_wizmeetme_passwd")
        self.check["passwd"]=self.glade.get_widget("check_wizmeetme_passwd")
        self.groupPasswd = Tools.GladeGroupContainer([self.entry["passwd"],self.label["passwd"]])
        
        for opt in ["m","t","p","X","v","q","d","D","M","e","E","b","a","A","s","x","w"]:
            self.check["option_"+opt]=self.glade.get_widget("check_wizmeetme_opt_"+opt)

        
        self.comboConf=Tools.GladeComboContainer(self.glade,"wizmeetme_conf")

    #________________________________________________
    def togglepasswd(self,*args):
    #________________________________________________
        isactive = self.check["passwd"].get_active()
        self.groupPasswd.set_sensitive(isactive)
        self.changecombo()
    #________________________________________________
        
    #________________________________________________
    def changecombo(self,*args):
    #________________________________________________
        isactive = self.check["passwd"].get_active()
        if isactive:
            self.entry["passwd"].set_text(GLOBAL.CONFROOMS.get(self.comboConf.get_text().strip(),""))
    #________________________________________________

    #________________________________________________
    def update_parent(self,*args):
    #________________________________________________
        self.parent.show()
    #________________________________________________
 
    #________________________________________________
    def update(self,*args):
    #________________________________________________
        self.comboConf.fill(GLOBAL.CONFROOMS.keys())
        self.check["passwd"].set_active(False)
        self.togglepasswd()
        self.show()
    #________________________________________________


    #________________________________________________
    def valide(self,*args):
    #________________________________________________
 
        
        #build option string
        options=""
        for opt in ["m","t","p","X","v","q","d","D","M","e","E","b","a","A","s","x","w"]:
            if self.check["option_"+opt].get_active():
                options+=opt

        stri=self.comboConf.get_text().strip()
        stri+="|"+options
        
        if self.check["passwd"].get_active():
            stri+="|"+self.entry["passwd"].get_text().strip()

        #update parent field
        try:
            self.parent.champ["params"].set_text(stri)
            self.parent.champ["params"].grab_focus()
        except:
            raise "INTERNAL ERROR ! Problem with parent ! 'params' field not found"

        self.close()
    #________________________________________________

#_______________________________________________________________



#----------------------------------------------------------------------------------------
#------------------------------------ FENETRE WIZARD GOTO -------------------------------
#----------------------------------------------------------------------------------------


#_______________________________________________________________
class WizardGoto(WindowCtrl):
#_______________________________________________________________

    #________________________________________________
    def __init__(self,parent=None):
    #________________________________________________
        WindowCtrl.__init__(self,"Fen_wizard_goto",parent)

        self.handlers = {"on_btvalider_wizgoto_clicked":self.valide,
                         "on_btannuler_wizgoto_clicked":self.close,
                         "on_treeview_ctxt_wizgoto_cursor_changed" : self.clicCtxt,
                         "on_treeview_ext_wizgoto_cursor_changed":self.clicExt,
                         "on_treeview_ext_wizgoto_row_activated":self.valide,
                         }
        self.glade.signal_autoconnect(self.handlers)

        self.selectedExt=None
        self.selectedPrio=None
        self.selectedCtxt=None
        self.random=False

        self.contextes=[]
        
        self.list={}
        self.model={}

        self.btValider=self.glade.get_widget("btValider_wizgoto")
        self.spinRandom=self.glade.get_widget("spin_random_wizgoto")
        self.groupRandom=self.glade.get_widget("hbox_random_wizgoto")
        
        #build listes
        self.model["ctxt"]=Tools.Interface.gtk.TreeStore(Tools.Interface.gobject.TYPE_STRING,Tools.Interface.gobject.TYPE_STRING)
        self.list["ctxt"]=self.glade.get_widget("treeview_ctxt_wizgoto")
        Tools.Build_liste(self.list["ctxt"],["Nom du contexte"])
        self.list["ctxt"].set_model(self.model["ctxt"])

        self.model["ext"]=Tools.Interface.gtk.TreeStore(Tools.Interface.gobject.TYPE_STRING)
        self.list["ext"]=self.glade.get_widget("treeview_ext_wizgoto")
        Tools.Build_liste(self.list["ext"],["Extension/Action"])
        self.list["ext"].set_model(self.model["ext"])

    #________________________________________________


    #________________________________________________
    def clicCtxt(self,*args):
    #________________________________________________
        selected,iter=self.list["ctxt"].get_selection().get_selected()
        self.selectedCtxt=int(self.model["ctxt"].get_value(iter,1))
        self.model["ext"].clear()
        if not self.contextes[self.selectedCtxt].put_in_models(self.model["ext"],showonerror=False):
            return
        self.list["ext"].expand_all()
        self.list["ext"].show()
    #________________________________________________

    #________________________________________________
    def clicExt(self,*args):
    #________________________________________________
        selected,iter=self.list["ext"].get_selection().get_selected()
        path=self.model["ext"].get_path(iter)
        if len(path)==1:
            iter_ext=iter
            iter_act=self.model["ext"].iter_children(iter)
        elif len(path)>1:
            iter_act=iter
            iter_ext=self.model["ext"].iter_parent(iter)

        try:
            self.selectedPrio=int(self.model["ext"].get_value(iter_act,0).split("::")[0].strip())
        except:
            self.selectedPrio=1

        self.selectedExt=self.model["ext"].get_value(iter_ext,0)
            
        if self.selectedExt[0]=="(" and self.selectedExt[2]==")":
            self.selectedExt=self.selectedExt[1]
    #________________________________________________


    #________________________________________________
    def update_parent(self,*args):
    #________________________________________________
        self.parent.show()
    #________________________________________________

    #________________________________________________
    def update(self,random=False):
    #________________________________________________
        self.random=random
        
        #Remplissage de la liste des contextes
        self.contextes=Tools.SortContexts(GLOBAL.ListeContextes,False)
        j=0
        for c in self.contextes:
            if len(c.exts.keys())>0:
                iter=self.model["ctxt"].insert_before(None,None)
                self.model["ctxt"].set_value(iter,0,c.name)
                self.model["ctxt"].set_value(iter,1,j)
            j+=1

        self.list["ctxt"].show()

        if random:
            self.groupRandom.show()
        else:
            self.groupRandom.hide()
            
        
        self.show()
    #________________________________________________


    #________________________________________________
    def valide(self,*args):
    #________________________________________________
        if self.selectedExt==None or self.selectedPrio==None or self.selectedCtxt==None:
            return

        ctxt=self.contextes[self.selectedCtxt].name.strip()
        result=ctxt+","+self.selectedExt+","+str(self.selectedPrio)

        if self.random:
            result=str(int(self.spinRandom.get_value()))+":"+result

        #update parent field
        try:
            self.parent.champ["params"].set_text(result)
            self.parent.champ["params"].grab_focus()
        except:
            raise "INTERNAL ERROR ! Problem with parent ! 'params' field not found."
        
        self.close()
        self.parent.valide()
    #________________________________________________

#_______________________________________________________________



#----------------------------------------------------------------------------------------
#------------------------------------ FENETRE WIZARD AUTHENTICATE -----------------------
#----------------------------------------------------------------------------------------


#_______________________________________________________________
class WizardAuth(WindowCtrl):
#_______________________________________________________________

    #________________________________________________
    def __init__(self,parent=None):
    #________________________________________________
        WindowCtrl.__init__(self,"Fen_wizard_authenticate",parent)

        self.handlers = {"on_btvalider_wizauth_clicked":self.valide,
                         "on_btannuler_wizauth_clicked":self.close,
                         "on_radio_filepass_wizauth_toggled":self.toggle_filepass,
                         "on_check_option_wizauth_d_toggled":self.toggle_option_d,
                         "on_radio_onepass_wizauth_toggled":self.toggle_onepass,
                         "on_bt_filepass_wizauth_clicked" : self.select_filepass
                         }
        self.glade.signal_autoconnect(self.handlers)

        self.btValider=self.glade.get_widget("btValider_wizauth")
        self.entryOnepass=self.glade.get_widget("entry_onepass_wizauth")
        self.entryFilepass=self.glade.get_widget("entry_filepass_wizauth")
        self.groupFilepass=Tools.GladeGroupContainer([self.entryFilepass,
                                                      self.glade.get_widget("bt_filepass_wizauth")])
        self.radioOnepass=self.glade.get_widget("radio_onepass_wizauth")

        self.options={}
        for opt in ["a","d","r"]:
            self.options[opt]=self.glade.get_widget("check_option_wizauth_"+opt)
    #________________________________________________

    #________________________________________________
    def toggle_option_d(self,*args):
    #________________________________________________
        self.options["r"].set_sensitive(self.options["d"].get_active())
    #________________________________________________

    #________________________________________________
    def toggle_filepass(self,*args):
    #________________________________________________
        self.groupFilepass.set_sensitive(True)
        self.entryOnepass.set_sensitive(False)
    #________________________________________________

    #________________________________________________
    def select_filepass(self,*args):
    #________________________________________________
        self.parent.children["selectFile"].parent=self
        self.parent.children["selectFile"].update("SELECT_FILE",self.entryFilepass)
    #________________________________________________

    #________________________________________________
    def toggle_onepass(self,*args):
    #________________________________________________
        self.groupFilepass.set_sensitive(False)
        self.entryOnepass.set_sensitive(True)
    #________________________________________________

    #________________________________________________
    def update_parent(self,*args):
    #________________________________________________
        self.parent.show()
    #________________________________________________

    #________________________________________________
    def update(self,*args):
    #________________________________________________
        self.parent.hide()
        self.entryOnepass.set_text("")
        self.entryFilepass.set_text("")
        self.radioOnepass.set_active(True)
        self.toggle_onepass()
        self.toggle_option_d()
 
        self.show()
    #________________________________________________


    #________________________________________________
    def valide(self,*args):
    #________________________________________________

        param=""
        act=self.radioOnepass.get_active()

        if act:
            param=self.entryOnepass.get_text().strip()
            if ("/" in param):
                Tools.debug("-- Error : "+_("ERROR_SLASH"))
                return
            if len(param)==0:
                Tools.debug("-- Error : "+_("ERROR_EMPTY_FIELD"))
                self.entryOnepass.grab_focus()
                return
        else:
            param=self.entryFilepass.get_text().strip()
            if len(param)==0:
                Tools.debug("-- Error : "+_("ERROR_EMPTY_FIELD"))
                self.entryFilepass.grab_focus()
                return
            #add / at beginning if needed
            if param[0]!="/":
                param="/"+param

        options=""
        for opt in ["a","d"]:
            if self.options[opt].get_active():
                options+=opt

        if self.options["d"].get_active() and self.options["r"].get_active():
            options+="r"

        if len(options)>0:
            param+=","+options

        #update parent field
        try:
            self.parent.champ["params"].set_text(param)
            self.parent.champ["params"].grab_focus()
        except:
            raise "INTERNAL ERROR ! Problem with parent ! 'params' field not found"
        
        self.close()
        self.parent.valide()
    #________________________________________________

#_______________________________________________________________



#----------------------------------------------------------------------------------------
#------------------------------------ FENETRE WIZARD VOICEMAIL --------------------------
#----------------------------------------------------------------------------------------


#_______________________________________________________________
class WizardVoicemail(WindowCtrl):
#_______________________________________________________________

    #________________________________________________
    def __init__(self,parent=None):
    #________________________________________________
        WindowCtrl.__init__(self,"Fen_wizard_voicemail",parent)

        self.handlers = {"on_btvalider_wizvmail_clicked":self.valide,
                         "on_btannuler_wizvmail_clicked":self.close,
                         "on_btajouter_wizvmail_clicked":self.addtolist,
                         "on_btenlever_wizvmail_clicked":self.removefromlist
                         }
        self.glade.signal_autoconnect(self.handlers)

        self.btValider=self.glade.get_widget("btValider_wizvmail")
        self.entryBox=self.glade.get_widget("entry_box_wizvmail")

        self.option={}
        self.option["s"]=self.glade.get_widget("check_skip_wizvmail")
        self.option["u"]=self.glade.get_widget("radio_unavailable_wizvmail")
        self.option["b"]=self.glade.get_widget("radio_busy_wizvmail")
        #build list
        self.listeBoxes=self.glade.get_widget("tree_boxes_wizvmail")
        Tools.Build_liste(self.listeBoxes,["Boxes"])
        self.modelBoxes= Tools.Interface.gtk.ListStore(Tools.Interface.gobject.TYPE_STRING)
    #________________________________________________

    #________________________________________________
    def toggle_btValider(self,*args):
    #________________________________________________
        self.btValider.set_sensitive( (len(self.modelBoxes)==0))
    #________________________________________________


    #________________________________________________
    def addtolist(self,*args):
    #________________________________________________
        box=self.entryBox.get_text().strip()
        if box=="":
            return

        flags=""
        for opt in self.option.keys():
            if self.option[opt].get_active():
                flags+=opt
        self.modelBoxes.insert(0,row=[flags+box])
        self.entryBox.grab_focus()
        self.btValider.set_sensitive(True)
    #________________________________________________

    #________________________________________________
    def removefromlist(self,*args):
    #________________________________________________
        model,iter = self.listeBoxes.get_selection().get_selected()
        if iter is not None:
            self.modelBoxes.remove(iter)
        else:
            Tools.debug("-- Error : "+_("ERROR_NO_SELECTION"))
            return
        self.listeBoxes.set_model(self.modelBoxes)
        self.listeBoxes.show()
        if len(self.modelBoxes)==0:
            self.btValider.set_sensitive(False)
    #________________________________________________
   

    #________________________________________________
    def update_parent(self,*args):
    #________________________________________________
        self.parent.show()
    #________________________________________________

    #________________________________________________
    def update(self,*args):
    #________________________________________________
        self.modelBoxes= Tools.Interface.gtk.ListStore(Tools.Interface.gobject.TYPE_STRING)
        self.listeBoxes.set_model(self.modelBoxes)
        self.entryBox.set_text("")
        self.btValider.set_sensitive(False)


        self.entryBox.grab_focus()
        self.show()
    #________________________________________________

    #________________________________________________
    def valide(self,*args):
    #________________________________________________

        param=""
        #get boxes list
        for line in self.modelBoxes:
            param+=line[0]+"&"
        param=param.rstrip("&")

        try:
            self.parent.champ["params"].set_text(param)
            self.parent.champ["params"].grab_focus()
        except:
            raise "INTERNAL ERROR ! Problem with parent ! 'params' field not found"
        
        self.close()
    #________________________________________________

#_______________________________________________________________





#----------------------------------------------------------------------------------------
#------------------------------------ FENETRE USERS -------------------------------------
#----------------------------------------------------------------------------------------


#________________________________________________
class WUsersCtrl(WindowCtrl):
#________________________________________________

    #________________________________________________
    def __init__(self,parent):
    #________________________________________________
        WindowCtrl.__init__(self,"Fen_users",parent)

        self.handlers = {"on_btferme_users_clicked" : self.close,
                         "on_btnouveau_users_clicked" : self.newUser,
                         "on_btediter_users_clicked" : self.editUser,
                         "on_btsuppr_users_clicked" : self.supprUser,
                         "on_liste_users_row_activated" : self.editUser,
                         }
        self.glade.signal_autoconnect(self.handlers)
        self.ASSOCIATED_MAIN_BT=GLOBAL.FEN_MAIN.btUsers

        self.children["editUser"]=WEditUserCtrl(self)

        
        #Init
        self.listeView = self.glade.get_widget('ListeUsers')
        self.modelView= Tools.Interface.gtk.ListStore(Tools.Interface.gobject.TYPE_STRING,Tools.Interface.gobject.TYPE_STRING,Tools.Interface.gobject.TYPE_STRING,Tools.Interface.gobject.TYPE_STRING,Tools.Interface.gobject.TYPE_STRING,Tools.Interface.gobject.TYPE_STRING,Tools.Interface.gobject.TYPE_STRING,Tools.Interface.gobject.TYPE_INT)

        self.EditedUser=None
        Tools.Build_liste(self.listeView,GLOBAL.COLUMN_OF_TABLE_USERS,True)
    #________________________________________________


    #________________________________________________
    def update(self,widget=0):
        """ Mise  jour de la fentre Users"""
    #________________________________________________
        self.ASSOCIATED_MAIN_BT.set_sensitive(False)
        
        #Initialisation du modele
        self.modelView= Tools.Interface.gtk.ListStore(Tools.Interface.gobject.TYPE_STRING,Tools.Interface.gobject.TYPE_STRING,Tools.Interface.gobject.TYPE_STRING,Tools.Interface.gobject.TYPE_STRING,Tools.Interface.gobject.TYPE_STRING,Tools.Interface.gobject.TYPE_STRING,Tools.Interface.gobject.TYPE_STRING,Tools.Interface.gobject.TYPE_INT)
        #Remplissage de la liste
        j=0
        for i in GLOBAL.ListeUsers:
            irow=[i.name,i.protocol]
            for nk in GLOBAL.COLUMN_OF_TABLE_USERS[2:]:
                irow.append(i.params[nk])
            irow.append(j)
            iter=self.modelView.insert(0,row=irow)
            j+=1

        #Affichage de la liste
        self.listeView.set_model(self.modelView)
        self.listeView.set_sensitive(True)
        self.listeView.show()
        
        #Retour en mode NORMAL et affichage de la fentre
        self.mode = "NORMAL"
        self.show()
    #________________________________________________


    #________________________________________________
    def supprUser(self,*args):
        """Clic sur bouton 'Supprimer' """
    #________________________________________________
        Tools.debug("## Delete user")

        #Rcupration de la slection
        selected,iter=self.listeView.get_selection().get_selected()
        if iter==None:
            Tools.debug("-- Error : "+_("ERROR_NO_SELECTION"))
            return

        #Demande de confirmation
        result=Tools.Interface.DialogBox(_("MSG_CONFIRM_DELETE_USER"),type_buttons=Tools.Interface.gtk.BUTTONS_YES_NO,type_dialog=Tools.Interface.gtk.MESSAGE_QUESTION)
        if result==Tools.Interface.gtk.RESPONSE_NO:
            return


        #model=self.listeView.get_model()

        #La 7me colonne contient l'indice dans ListeUsers
        self.EditedUser=self.modelView.get_value(iter,len(GLOBAL.COLUMN_OF_TABLE_USERS))
        #Suppression de la valeur dans la liste
        GLOBAL.ListeUsers.pop(self.EditedUser)

        #Mise  jour de la fentre
        self.update()
    #________________________________________________


    #________________________________________________
    def editUser(self,*args):
        """Clic sur bouton 'Editer' """
    #________________________________________________
        Tools.debug("## Modify user")

        #Rcupration de la slection
        selected,iter=self.listeView.get_selection().get_selected()
        if iter==None:
            Tools.debug("-- Error : "+_("ERROR_NO_SELECTION"))
            return

        #Rend la liste grise
        self.listeView.set_sensitive(False)

        #Affiche la fentre d'dition en mode EDIT
        self.mode = "EDIT"
        #model=self.listeView.get_model()
        self.children["editUser"].update(self.modelView,iter)
    #________________________________________________
 
      
    #________________________________________________
    def newUser(self,*args):
        """Clic sur bouton 'Nouvel utilisateur' """
    #________________________________________________
        Tools.debug("## Create new user")

        #Rend la liste grise
        self.listeView.set_sensitive(False)

        #Affiche la fentre d'dition en mode CREATE
        self.mode = "CREATE"
        self.children["editUser"].update(None,None)
    #________________________________________________




#----------------------------------------------------------------------------------------
#------------------------------------ FENETRE EDIT USER ---------------------------------
#----------------------------------------------------------------------------------------

#________________________________________________
class WEditUserCtrl(WindowCtrl):
#________________________________________________

    #________________________________________________
    def __init__(self,parent):
    #________________________________________________
        WindowCtrl.__init__(self,"Fen_editUser",parent)
        self.handlers = {"on_btannuler_edituser_clicked" : self.close,
                         "on_btvalider_edituser_clicked" : self.valide,
                         "on_check_auth_edituser_toggled" : self.toggleauth,
                         "on_check_nat_edituser_toggled" : self.togglenat,
                         "on_check_qualify_edituser_toggled" : self.togglequalify,
                         "on_entry_edituser_changed" : self.changeprotocol,
                         "on_entry_edituser_type_changed" : self.changetype,
                         "on_entry_edituser_auth_changed" : self.changeauth,
                         "on_check_register_edituser_toggled" : self.toggleregister,
                         "on_btpermit_edituser_clicked" : self.authorizations
                         }

        self.glade.signal_autoconnect(self.handlers)

        #Init
        self.EditedUser=None
        self.isiaxactive=False
        self.champ={}

        for i in ["name","host","login","secret"]:
            self.champ[i]=self.glade.get_widget("entry_edituser_"+i)
        

        self.champ["check_register"]=self.glade.get_widget("check_register_edituser")
        self.champ["entry_registerext"]=self.glade.get_widget("entry_registerext_edituser")
        self.champ["check_auth"]=self.glade.get_widget("check_auth_edituser")
        self.champ["check_canreinvite"]=self.glade.get_widget("check_canreinvite_edituser")
        self.champ["check_nat"]=self.glade.get_widget("check_nat_edituser")
        self.champ["check_qualify"]=self.glade.get_widget("check_qualify_edituser")
        self.champ["qualify"]=self.glade.get_widget("entry_qualify_edituser")
        self.champ["check_trunk"]=self.glade.get_widget("check_trunk_edituser")

        self.champ["type"]=Tools.GladeComboContainer(self.glade,"edituser_type")
        self.champ["context"]=Tools.GladeComboContainer(self.glade,"edituser_context")
        self.champ["protocol"]=Tools.GladeComboContainer(self.glade,"edituser_protocol")
        self.champ["auth"]=Tools.GladeComboContainer(self.glade,"edituser_auth")
        self.champ["dmtf"]=Tools.GladeComboContainer(self.glade,"edituser_dmtf")
        self.groupRegister=Tools.GladeGroupContainer([self.champ["entry_registerext"],self.glade.get_widget("label_register_edituser")])

        self.groupAgent=Tools.GladeGroupContainer([self.glade.get_widget("editbox6"),
                                                   self.glade.get_widget("editbox3"),
                                                   self.glade.get_widget("editbox5"),
                                                   self.glade.get_widget("hbox_dmtf_edituser"),
                                                   self.glade.get_widget("hbox54"),
                                                   self.glade.get_widget("hbox34"),
                                                   self.glade.get_widget("hbox_auth_edituser"),
                                                   self.glade.get_widget("btPermit_edituser")])
    #________________________________________________


    #________________________________________________
    def authorizations(self,*args):
        """Authorizations"""
    #________________________________________________
        Tools.debug("-- TODO --")
    #________________________________________________


    #________________________________________________
    def checkCanRegister(self):
    #________________________________________________
        if self.champ["login"].get_text()=="":
            Tools.Interface.DialogBox(_("ERROR_MUST_DEFINE_REGISTER",["a login","un login"]))
            self.champ["login"].grab_focus()
            return False
        if self.champ["secret"].get_text()=="":
            Tools.Interface.DialogBox(_("ERROR_MUST_DEFINE_REGISTER",["a password","un mot de passe"]))
            self.champ["secret"].grab_focus()
            return False
        if self.champ["host"].get_text()=="" or self.champ["host"].get_text()=="dynamic":
            Tools.Interface.DialogBox(_("ERROR_MUST_DEFINE_REGISTER",["a static host","un hte statique"]))
            self.champ["host"].grab_focus()
            return False
        return True
    #________________________________________________

            
    #________________________________________________
    def toggleregister(self,*args):
        """Modification du slecteur qualify"""
    #________________________________________________
        active=self.champ["check_register"].get_active()
        self.groupRegister.set_sensitive(active)
        if active:
            if not self.checkCanRegister():
                self.champ["check_register"].set_active(False)
                return
            self.champ["entry_registerext"].grab_focus()
    #________________________________________________


    #________________________________________________
    def togglequalify(self,*args):
        """Modification du slecteur qualify"""
    #________________________________________________
        active=self.champ["check_qualify"].get_active()
        self.glade.get_widget("hbox_qualify_edituser").set_sensitive(active)
        if active and self.champ["qualify"].get_text().strip()=="":
            self.champ["qualify"].set_text(str(GLOBAL.DEFAULT_QUALIFY_MS))
        self.champ["qualify"].grab_focus()
    #________________________________________________

    #________________________________________________
    def togglenat(self,*args):
        """Modification du slecteur NAT"""
    #________________________________________________
        active=self.champ["check_nat"].get_active()
        self.champ["check_canreinvite"].set_active(active)
        self.champ["check_canreinvite"].set_sensitive(not active)
        if active:
            self.champ["check_qualify"].set_active(True)
            #(removed) self.togglequalify()
    #________________________________________________

    #________________________________________________
    def toggleauth(self,*args):
        """Modification du slecteur authentification"""
    #________________________________________________

        isauthactive=self.isiaxactive and self.champ["check_auth"].get_active()
#        isauthactive=self.champ["check_auth"].get_active()
        self.champ["check_auth"].set_sensitive(True)
        self.champ["auth"].set_sensitive(isauthactive)
        method=self.champ["auth"].get_text()
        
        #Active le bouton "RSA keys" seulement si on a choisi "rsa"
        if method=="rsa":
            self.glade.get_widget("btRSAkeys_edituser").set_sensitive(isauthactive)
        else:
            self.glade.get_widget("btRSAkeys_edituser").set_sensitive(False)

        #Active le choix de l'authentification seulement si on est en mode "iax2"
        self.glade.get_widget("hbox_auth_edituser").set_sensitive(self.isiaxactive)
    #________________________________________________


    #________________________________________________
    def changeauth(self,*args):
        """Modification de combo auth"""
    #________________________________________________
        if self.champ["auth"].get_text()=="rsa":
            Tools.debug("-- TODO --")
    #________________________________________________


    #________________________________________________
    def changetype(self,*args):
        """Modification de combo type"""
    #________________________________________________
        #Disable checks unused
        act=(self.champ["type"].get_text().strip()=="agent")
        label=self.glade.get_widget("label_edituser_login")

        self.groupAgent.set_sensitive(not act)
        if act:
            label.set_text("Agent ID")
        else:
            label.set_text("Login")
    #________________________________________________


    #________________________________________________
    def changeprotocol(self,*args):
        """Modification de protocole"""
    #________________________________________________
        #Disable checks unused
        l=[]
        if self.champ["protocol"].get_text().strip()=="iax2":
            self.isiaxactive=True
            l1=["nat","canreinvite","qualify"]
            l2=["trunk"]
            #(removed) ,"auth"]
            self.glade.get_widget("hbox_dmtf_edituser").set_sensitive(False)
        else:
            self.isiaxactive=False
            l1=["trunk"]
            #(removed) ,"auth"]
            l2=["canreinvite","qualify"]
            #NAT SUPPORT depends of global parameters
            if GLOBAL.GLOBAL_PARAMS["natsupport"]=="yes":
                l2.extend(["nat"])
            else:
                l1.extend(["nat"])

            self.glade.get_widget("hbox_dmtf_edituser").set_sensitive(True)

        for ch in l1:
            self.champ["check_"+ch].set_active(False)
            self.champ["check_"+ch].set_sensitive(False)
        for ch in l2:
            self.champ["check_"+ch].set_sensitive(True)

        #Remplissage de la combo d'authentification selon le mode
        templ=Tools.copy.deepcopy(GLOBAL.AUTH_TYPES)
        if not self.isiaxactive:
            templ.remove("rsa")
        self.champ["auth"].fill(templ)

        self.toggleauth()
    #________________________________________________


    #________________________________________________
    def valide(self,*args):
        #Prise en compte de la valeur des champs d'dition
    #________________________________________________

        Tools.debug("## Validation")

        #Rcupration des valeurs dans les champs et les combos
        entries={}
        for i in GLOBAL.COLUMN_OF_TABLE_USERS:
            entries[i]=self.champ[i].get_text()

        if entries["type"]=="agent":
            entries["protocol"]="agent"
            rwith=False
            rext=""
        else:                              
            #Sauvegarde du type d'authentification
            if self.champ["check_auth"].get_active():
                entries["auth"]=self.champ["auth"].get_text()
                if entries["auth"] in ["plaintext","md5"] and self.champ["secret"].get_text().strip()=="":
                    Tools.debug("-- Error : "+_("ERROR_AUTH_SECRET"))
                    return
            else:
                entries["auth"]=""

            #Sauvegarde des options annexes
            emptyyes={False : "",True : "yes"}
            noempty={False : "no",True : ""}
                
            prot=self.champ["protocol"].get_text()
            if prot=="sip":
                entries["nat"]=emptyyes[self.champ["check_nat"].get_active()]
                entries["canreinvite"]=noempty[not self.champ["check_canreinvite"].get_active()]
                if self.champ["check_qualify"].get_active()==True:
                    entries["qualify"]=self.champ["qualify"].get_text().strip()
                else:
                    entries["qualify"]=""

                entries["trunk"]=""
                entries["dmtfmode"]=self.champ["dmtf"].get_text()
                if entries["dmtfmode"]=="default":
                    entries["dmtfmode"]=""

            elif prot=="iax2":
                for z in ["nat","canreinvite","qualify","dmtfmode"]:
                    entries[z]=""
                entries["trunk"]=emptyyes[self.champ["check_trunk"].get_active()]

            else:
                #XXX TODO (others protocols)
                entries["nat"]=""
                entries["trunk"]=""
                entries["dmtfmode"]=""
            
        
            rwith=self.champ["check_register"].get_active()
            if rwith and not self.checkCanRegister():
                return
            rext=self.champ["entry_registerext"].get_text()


        #Cration d'un utilisateur temporaire avec ces valeurs
        entries2=Tools.copy.deepcopy(entries)
        entries2.pop("name")
        entries2.pop("protocol")
        TempUser=User.User(name=entries["name"],protocol=entries["protocol"],keys=entries2,register=rwith,registerext=rext)

        #Selon le mode, ajout ou modification de l'utilisateur courant
        if self.mode=="CREATE":
            GLOBAL.ListeUsers.append(TempUser)
        elif self.mode=="EDIT":
            del(GLOBAL.ListeUsers[self.EditedUser])
            GLOBAL.ListeUsers.insert(self.EditedUser,TempUser)

        #Masquage de la fentre
        self.close()
        self.parent.update()
    #________________________________________________
        

    #________________________________________________
    def update(self,model,iter=None):
        """Prpare les champs d'dition"""
        #si Model==None : MODE CREATE
        #si Model!=None et iter!=None : MODE EDIT
    #________________________________________________


        #Remplissage de la combo des contextes
        listeitem=[]
        for i in GLOBAL.ListeContextes:
            listeitem.append(i.name)
        self.champ["context"].fill(listeitem)

        #Remplissage de la combo des types
        self.champ["type"].fill(GLOBAL.USER_TYPES)
        #Remplissage de la combo des protocoles
        self.champ["protocol"].fill(GLOBAL.PROTOCOLS.keys())
        self.changeprotocol()
        #Remplissage de la combo d'authentification
        #self.champ["auth"].fill(GLOBAL.AUTH_TYPES)
        #Remplissage de la combo DMTF
        self.champ["dmtf"].fill(GLOBAL.DMTF_MODES)
        #Rcupration du champ d'information pour afficher le mode
        lab=self.glade.get_widget("label_info_edituser")



        if model==None:
            #PREPARATION DU MODE CREATION
            
            #Affectation des valeurs par dfaut aux champs
            tempwidget=None
            for i in GLOBAL.COLUMN_OF_TABLE_USERS:
                if i in GLOBAL.DEFAULTS_NEW_USER.keys():
                    self.champ[i].set_text(GLOBAL.DEFAULTS_NEW_USER[i])
                self.champ[i].show()
            self.champ["entry_registerext"].set_text("")

            #Disable check buttons
            for field in ["auth","nat","canreinvite","qualify","trunk","register"]:
                self.champ["check_"+field].set_active(False)

            self.toggleregister()
                       
            #old stuffs, maybe not necessary any more?
            self.glade.get_widget("btwwValider").show()
            self.glade.get_widget("btwwAnnuler").show()
            
            #Passage en mode CREATION
            self.mode = "CREATE"
            lab.set_text("CREATE NEW USER")

        elif iter!=None:
            #PREPARATION DU MODE EDITION

            #Affectation des valeurs de l'utilisateur courant aux champs
            for i in GLOBAL.COLUMN_OF_TABLE_USERS:
                self.champ[i].set_text(model.get_value(iter,GLOBAL.COLUMN_OF_TABLE_USERS.index(i)))

            #La colonne 7 est l'indice de l'utilisateur dans la liste
            self.EditedUser=model.get_value(iter,len(GLOBAL.COLUMN_OF_TABLE_USERS))
            user=GLOBAL.ListeUsers[self.EditedUser]
            
            if user.protocol!="agent":
                #initialisation du DMTF Mode
                dmtf=user.params["dmtfmode"].strip()
                if dmtf=="":
                    dmtf="default"
                self.champ["dmtf"].set_text(dmtf)

                #initialisation de l'authentification
                auth=user.params["auth"].strip()
                if auth=="":
                    self.champ["check_auth"].set_active(False)
                else:
                    self.champ["check_auth"].set_active(True)
                    self.champ["auth"].set_text(auth)


                #intialisation des options annexes
                for i in GLOBAL.OPTIONS_USER:
                    act=(user.params[i].strip() not in ["no",""])
                    self.champ["check_"+i].set_active(act)

                #Warning : CANREINVITE is special (must invert it)
                self.champ["check_canreinvite"].set_active(user.params["canreinvite"].strip()=="no")

                #init register button
                self.champ["check_register"].set_active(user.registerwith)
                self.champ["entry_registerext"].set_text(user.getRegisterExt())
                self.toggleregister()
            #fin if not agent
            
            #Passage en mode EDITION
            self.mode = "EDIT"
            lab.set_text("EDIT USER")

        else:
            #Test de scurit, on n'est pas cens venir ici
            #--------> Insulter le programmeur
            raise "INTERNAL ERROR ! Invalid iterator"
            return


        #Disable qualify time field
        self.togglequalify()

        
        #Define focus
        self.champ["name"].grab_focus()
        #Show window
        self.show()
    #________________________________________________




#----------------------------------------------------------------------------------------
#------------------------------------- FENETRE EDITVAR ---------------------------------
#----------------------------------------------------------------------------------------


#________________________________________________
class WEditVarCtrl(WindowCtrl):
#________________________________________________

    #________________________________________________
    def __init__(self,parent):
    #________________________________________________
        WindowCtrl.__init__(self,"Fen_editVar",parent)
        self.handlers = {"on_btannuler_editvar_clicked" : self.close,
                         "on_btvalider_editvar_clicked" : self.valide,
                        }
        self.glade.signal_autoconnect(self.handlers)

        #Init
        self.entryVar=self.glade.get_widget("entry_var_editvar")
        self.entryValue=self.glade.get_widget("entry_value_editvar")
        
        self.mode="CREATE"
        
    #___________________________________________________________


    #________________________________________________
    def update_parent(self,*args):
    #________________________________________________
        self.parent.update_list()
        self.parent.show()
    #________________________________________________

    #________________________________________________
    def update(self,var=None,value=None):
    #________________________________________________
        #update fields
        self.old_var=var
        if var==None:
            self.mode="CREATE"
            self.entryVar.set_text("")
            self.entryValue.set_text("")
        else:
            self.mode="EDIT"
            self.entryVar.set_text(var)
            self.entryValue.set_text(value)

        self.entryVar.grab_focus()
        self.show()
    #________________________________________________

    #________________________________________________
    def valide(self,*args):
    #________________________________________________
        #get values
        var=Tools.string.upper(self.entryVar.get_text().strip())
        value=self.entryValue.get_text().strip()
        
        if self.mode=="CREATE":
            if var in GLOBAL.VARIABLES.keys():
                Tools.debug("-- Error :"+_("ERROR_VAR_ALREADY_EXISTS"))
                return
        elif self.mode=="EDIT":
            #delete old key
            if self.old_var in GLOBAL.VARIABLES.keys():
                del(GLOBAL.VARIABLES[self.old_var])

        #create new key
        GLOBAL.VARIABLES[var]=value
        
        self.close()
    #________________________________________________

#________________________________________________



#----------------------------------------------------------------------------------------
#------------------------------------- FENETRE VARIABLES --------------------------------
#----------------------------------------------------------------------------------------



#________________________________________________
class WVariablesCtrl(WindowCtrl):
#________________________________________________

    #________________________________________________
    def __init__(self,parent):
    #________________________________________________
        WindowCtrl.__init__(self,"Fen_variables",parent)
        self.handlers = {"on_btannuler_var_clicked" : self.close,
                         "on_btvalider_var_clicked" : self.valide,
                         "on_btnouveau_var_clicked" : self.newVariable,
                         "on_btsuppr_var_clicked" : self.supprVariable,
                         "on_btediter_var_clicked" : self.editVariable,
                         "on_radio_channel_var_toggled" : self.toggleChannelVars,
                         "on_radio_usermade_var_toggled" : self.toggleUserVars,
                         "on_entry_channel_var_changed" : self.showHelp,
                         "on_listevars_row_activated" : self.editVariable
                        }
        self.glade.signal_autoconnect(self.handlers)
        self.ASSOCIATED_MAIN_BT=GLOBAL.FEN_MAIN.btVariables

        self.children["editVar"]=WEditVarCtrl(self)

        #Init
        self.listeVar=self.glade.get_widget('listeVars')
        self.comboChannelVars=Tools.GladeComboContainer(self.glade,"channel_var")
        self.radioChannelVars=self.glade.get_widget("radio_channel_var")
        self.labelHelp=self.glade.get_widget("label_help_var")
        self.groupSelect=Tools.GladeGroupContainer([self.glade.get_widget("hbox_var_select"),
                                                    self.glade.get_widget("radio_usermade_var"),
                                                    self.radioChannelVars,                                                   
                                                    self.comboChannelVars,
                                                    self.labelHelp])

        self.groupUserVars=Tools.GladeGroupContainer([self.glade.get_widget("scroll_listeVars"),
                                                      self.glade.get_widget("btNew_var"),
                                                      self.glade.get_widget("btEdit_var"),
                                                      self.glade.get_widget("btDelete_var")])
                
        self.btFermer=self.glade.get_widget("btFermer_var")

        #self.modelVar=Tools.Interface.gtk.ListStore(Tools.Interface.gobject.TYPE_STRING,Tools.Interface.gobject.TYPE_STRING)
        Tools.Build_liste(self.listeVar,["variable name","value"])
        self.mode="SELECT"
        
    #___________________________________________________________


    #________________________________________________
    def showHelp(self,*args):
    #________________________________________________
        key=self.comboChannelVars.get_text()
        if key=="":
            return
        self.labelHelp.set_text(GLOBAL.CHANNEL_VARS[key])
    #________________________________________________
    
    #________________________________________________
    def toggleChannelVars(self,*args):
    #________________________________________________
        if self.mode!="SELECT":
            return
        self.comboChannelVars.activate()
        self.groupUserVars.desactivate()
    #________________________________________________

    #________________________________________________
    def toggleUserVars(self,*args):
    #________________________________________________
        if self.mode!="SELECT":
            return
        self.comboChannelVars.desactivate()
        self.labelHelp.set_text("")
        self.groupUserVars.activate()
    #________________________________________________

    #________________________________________________
    def newVariable(self,*args):
    #________________________________________________
        self.children["editVar"].update(None)
    #________________________________________________

    #________________________________________________
    def supprVariable(self,*args):
    #________________________________________________
        #Get selected row
        selected,iter=self.listeVar.get_selection().get_selected()
        if iter==None:
            Tools.debug("-- Error : "+_("ERROR_NO_SELECTION"))
            return

        var=str(self.modelVar.get_value(iter,0))
        #Demande de confirmation
        result=Tools.Interface.DialogBox(_("MSG_CONFIRM_DELETE_VAR",[var]),type_buttons=Tools.Interface.gtk.BUTTONS_YES_NO,type_dialog=Tools.Interface.gtk.MESSAGE_QUESTION)
        if result==Tools.Interface.gtk.RESPONSE_NO:
            return

        try:
            del(GLOBAL.VARIABLES[var])
            self.modelVar.remove(iter)
        except:
            pass
            
        self.listeVar.set_model(self.modelVar)
        self.listeVar.show()
    #________________________________________________


    #________________________________________________
    def editVariable(self,*args):
    #________________________________________________
        #Get selected row
        selected,iter=self.listeVar.get_selection().get_selected()
        if iter==None:
            Tools.debug("-- Error : "+_("ERROR_NO_SELECTION"))
            return
        self.children["editVar"].update(str(self.modelVar.get_value(iter,0)),str(self.modelVar.get_value(iter,1)))
    #________________________________________________


    #________________________________________________
    def update_list(self,*args):
    #________________________________________________
        #fill list
        self.modelVar=Tools.Interface.gtk.ListStore(Tools.Interface.gobject.TYPE_STRING,Tools.Interface.gobject.TYPE_STRING)
        for v in GLOBAL.VARIABLES.keys():
            irow=[v,GLOBAL.VARIABLES[v]]
            iter=self.modelVar.insert(0,row=irow)
        self.listeVar.set_model(self.modelVar)
    #________________________________________________
    

    #________________________________________________
    def update(self,mode="SELECT",parent=None):
    #________________________________________________
        self.ASSOCIATED_MAIN_BT.set_sensitive(False)
        self.mode=mode

        #change parent if needed
        if parent==None:
            self.parent=GLOBAL.FEN_MAIN
        else:
            self.parent=parent
        
        #hide unused buttons / mode
        if mode=="SELECT":
            self.groupSelect.show()
            self.btFermer.hide()

            l=Tools.copy.deepcopy(GLOBAL.CHANNEL_VARS.keys())
            l.sort()
            self.comboChannelVars.fill(l)
            
            self.radioChannelVars.set_active(True)
            self.toggleChannelVars()
        else:
            self.groupSelect.hide()
            self.groupUserVars.activate()
            self.btFermer.show()

        self.update_list()

        self.show()
    #________________________________________________


    #________________________________________________
    def update_parent(self,*args):
    #________________________________________________
         GLOBAL.FEN_MAIN.btVariables.set_sensitive(True)
         self.parent.show()
         return
     
         #------------------------------------------------
         #TOOLS.DEBUGGING PART
         #------------------------------------------------
         print "\n## BUG CHECK [WVariablesCtrl.update_parent] (mode/parent)=("+str(self.mode)+"/"+str(self.parent)+")"
         if self.mode=="SELECT":
             if "WEditActionCtrl" in str(self.parent):
                 print "  Seems OK this time..."
             else:
                 print "  Bad parent !.... What have u done exactly ??"
         elif self.mode=="LIST":
             if "WMainCtrl" in str(self.parent):
                 print "  Seems OK this time..."
             else:
                 print "  Bad parent !.... What have u done exactly ??"
         else:
             print "  Unknown mode !... you really sucks !"
         #------------------------------------------------
   
    #________________________________________________


    #________________________________________________
    def valide(self,*args):
    #________________________________________________
         if self.parent.name=="Fen_editAction":
             pcp = self.parent.champ["params"].get_property("cursor-position")
             act=self.radioChannelVars.get_active()
             var="${"
             if act:
                 var+=self.comboChannelVars.get_text()+"}"
             else:
                 #Get selected row
                 selected,iter=self.listeVar.get_selection().get_selected()
                 if iter==None:
                     Tools.debug("-- Error : "+_("ERROR_NO_SELECTION"))
                     return
                 var+=self.modelVar.get_value(iter,0)+"}"
                                  
             self.parent.champ["params"].insert_text(var,pcp)

         else:
             raise "INTERNAL ERROR ! This button shouldn't be visible because parent didn't ask to select a variable"
             
            
         self.close()
    #________________________________________________

#________________________________________________


#----------------------------------------------------------------------------------------
#------------------------------------- FENETRE DES CONTEXTES ----------------------------
#----------------------------------------------------------------------------------------


#________________________________________________
class WContextsCtrl(WindowCtrl):
#________________________________________________

    #________________________________________________
    def __init__(self,parent):
    #________________________________________________
        WindowCtrl.__init__(self,"Fen_contextes",parent)
        self.handlers = {"on_btfermer_ctxt_clicked" : self.close,
                         "on_btnouveau_ctxt_clicked" : self.newContext,
                         "on_btediter_ctxt_clicked" : self.editContext,
                         "on_btsuppr_ctxt_clicked" : self.supprContext,
                         "on_listecontextes_row_activated" : self.editContext
                         }
        self.glade.signal_autoconnect(self.handlers)
        self.ASSOCIATED_MAIN_BT=GLOBAL.FEN_MAIN.btContextes
        
        self.children["editContext"]=WEditContextCtrl(self)

        #Init
        self.EditedContext=None
        self.listeView=self.glade.get_widget('ListeContextes')
        Tools.Build_liste(self.listeView,["Nom du contexte"])
    #___________________________________________________________


    #___________________________________________________________
    def supprContext(self,*args):
        """Clic sur bouton 'Supprimer' """
    #___________________________________________________________
        Tools.debug("## Delete context")

        #Rcupration de la slection
        selected,iter=self.listeView.get_selection().get_selected()
        if iter==None:
            Tools.debug("-- Error : "+_("ERROR_NO_SELECTION"))
            return

        #Demande de confirmation
        result=Tools.Interface.DialogBox(_("MSG_CONFIRM_DELETE_CONTEXT"),type_buttons=Tools.Interface.gtk.BUTTONS_YES_NO,type_dialog=Tools.Interface.gtk.MESSAGE_QUESTION)
        if result==Tools.Interface.gtk.RESPONSE_NO:
            return

        #Suppression de la valeur dans la liste
        model=self.listeView.get_model()
        self.EditedContext=model.get_value(iter,1)
        GLOBAL.ListeContextes.pop(self.EditedContext)

        #Mise  jour de la fentre
        self.update()
    #________________________________________________


    #________________________________________________
    def editContext(self,*args):
        """Clic sur bouton 'Editer' """
    #________________________________________________
        Tools.debug("## Modify context")

        #Rcupration de la slection
        selected,iter=self.listeView.get_selection().get_selected()
        if iter==None:
            Tools.debug("-- Error : "+_("ERROR_NO_SELECTION"))
            return

        self.EditedContext=self.listeView.get_model().get_value(iter,1)

        #Grisage de la liste
        self.listeView.set_sensitive(False)

        #Affichage de la fentre d'dition en mode EDITION
        model=self.listeView.get_model()
        self.children["editContext"].update(model,iter)
    #________________________________________________
        

    #________________________________________________
    def newContext(self,*args):
        """Clic sur bouton 'Nouveau' """
    #________________________________________________
        Tools.debug("## New context")

        #Affichage de la fentre d'dition en mode CREATION
        self.children["editContext"].update(None,None)
    #________________________________________________



    #________________________________________________
    def update(self,widget=0):
        """ Mise  jour de la fentre Contextes"""
    #________________________________________________
        self.resultiter=None

        def getitermatchselected(model, path, iter, rien):
            #function called for each row :
            #get the selected iter from the number in EditedContext
            val=model.get_value(iter,1)
            if val==self.EditedContext:
                self.resultiter=iter
                return True
            return False


        self.ASSOCIATED_MAIN_BT.set_sensitive(False)
        
        #if no contexts defined, create some default stuffs
        if len(GLOBAL.ListeContextes)==0:
            for c in GLOBAL.DEFAULT_CONTEXTS:
                GLOBAL.ListeContextes.append(Context.Context(c))

        #Initialisation du modele
        model = Tools.Interface.gtk.ListStore(Tools.Interface.gobject.TYPE_STRING,Tools.Interface.gobject.TYPE_INT)
        #Remplissage de la liste
        GLOBAL.ListeContextes=Tools.SortContexts(GLOBAL.ListeContextes,True)
        j=0
        for i in GLOBAL.ListeContextes:
            irow=[i.name,j]
            iter=model.insert(0,row=irow)
            j+=1

        #Affichage de la liste
        self.listeView.set_model(model)
        self.listeView.set_sensitive(True)
        self.listeView.show()

        #select the right row
        model.foreach(getitermatchselected,None)
        if self.resultiter!=None:
            selection=self.listeView.get_selection()
            selection.select_iter(self.resultiter)

        self.mode = "NORMAL"
        
        self.show()
    #________________________________________________



#----------------------------------------------------------------------------------------
#------------------------------------- FENETRE EDIT ACTION -------------------------------
#----------------------------------------------------------------------------------------


#________________________________________________
class WEditActionCtrl(WindowCtrl):
#________________________________________________

    #________________________________________________
    def __init__(self,parent):
    #________________________________________________
        WindowCtrl.__init__(self,"Fen_editAction",parent)

        self.handlers = {"on_btannuler_editaction_clicked" : self.close,
                         "on_btvalider_editaction_clicked" : self.valide,
                         "on_entry_cat_editaction_changed" : self.update_actions,
                         "on_list_actions_cursor_changed" : self.show_action_help,
                         "on_list_actions_row_activated" : self.launchWizard,
                         "on_btvariables_clicked" : self.insert_variable,
                         "on_radio_usermade_act_toggled":self.toggleUserMade,
                         "on_radio_predefined_act_toggled":self.togglePredefined,
                         "on_btwizard_editaction_clicked" : self.launchWizard,
                         "on_radio_sortmost_editaction_toggled" : self.toggleSortMostUsed,
                         "on_radio_sortalpha_editaction_toggled" : self.toggleSortAlpha
                         }

        self.glade.signal_autoconnect(self.handlers)
        #self.widget.set_modal(True)

        self.SortMode="MOSTUSED"

        #create children
        if not "variables" in GLOBAL.FEN_MAIN.children:
            self.children["variables"]=WVariablesCtrl(self)
            GLOBAL.FEN_MAIN.children["variables"]=self.children["variables"]
        else:
            self.children["variables"]=GLOBAL.FEN_MAIN.children["variables"]

        if not "selectFile" in GLOBAL.FEN_MAIN.children:
            self.children["selectFile"]=WSelectFileCtrl(self)
            GLOBAL.FEN_MAIN.children["selectFile"]=self.children["selectFile"]
        else:
            self.children["selectFile"]=GLOBAL.FEN_MAIN.children["selectFile"]

        self.children["wizard_dial"]=WizardDial(self)
        self.children["wizard_goto"]=WizardGoto(self)
        self.children["wizard_authenticate"]=WizardAuth(self)
        self.children["wizard_voicemail"]=WizardVoicemail(self)
        self.children["wizard_meetme"]=WizardMeetMe(self)
        
        self.champ={}
        self.champ["help"]=self.glade.get_widget("label_help_editaction")
        self.champ["params"]=self.glade.get_widget("entry_params_editaction")
        self.champ["label"]=self.glade.get_widget("label_action_editaction")
        self.champ["label2"]=self.glade.get_widget("label_action2_editaction")
        self.champ["actions"]=self.glade.get_widget("list_actions")
        
        self.champ["category"]=Tools.GladeComboContainer(self.glade,"cat_editaction")
        self.champ["scripts"]=Tools.GladeComboContainer(self.glade,"predefined_act")

        listecat=GLOBAL.ACTIONS.keys()
        listecat.sort()
        listecat.insert(0,"[ALL]")
        self.champ["category"].fill(listecat)

        self.btVariables=self.glade.get_widget("btVariables_editaction")
        self.btWizard=self.glade.get_widget("btWizard_editaction")

        self.groupUserMade=Tools.GladeGroupContainer([self.champ["help"],
                                                       self.champ["params"],
                                                       self.champ["label"],
                                                       self.champ["label2"],
                                                       self.champ["category"],
                                                       self.glade.get_widget("scroll_list_actions"),
                                                       self.glade.get_widget("label_cat_editaction"),
                                                       self.btVariables,
                                                       self.btWizard,
                                                       self.glade.get_widget("hbox_sort_editaction")])

        self.groupPredefined=Tools.GladeGroupContainer([self.champ["scripts"],self.glade.get_widget("btNewScript_editaction")])

        self.modelActions=None
        Tools.Build_liste(self.champ["actions"],["Action"])

        #Init
        self.EditedExtension=""
        self.EditedAction=None
        self.SaveParams=None
    #________________________________________________


    #________________________________________________
    def launchWizard(self,*args):
    #________________________________________________
        action=Tools.string.lower(self.EditedAction)
        wizard_available=(action in GLOBAL.ACTIONS_AVAILABLE_WIZARD)
        if not wizard_available:
            if action in GLOBAL.ACTIONS_WITHOUT_PARAMS:
                self.valide()
            else:
                Tools.debug("-- Error : "+_("ERROR_NO_WIZARD"))
                self.champ["params"].grab_focus()
            return

        if action=="goto":
            self.hide()
        elif action=="random":
            self.hide()
            self.children["wizard_goto"].update(random=True)
            return
        elif action in ["playback","background","adsiprog"]:
            self.children["selectFile"].parent=self
            self.children["selectFile"].update("SELECT_FILE",self.champ["params"])
            return
            
        self.children["wizard_"+action].update()
    #________________________________________________


    #________________________________________________
    def toggleSortMostUsed(self,*args):
    #________________________________________________
        self.SortMode="MOSTUSED"
        self.update_actions()
    #________________________________________________

    #________________________________________________
    def toggleSortAlpha(self,*args):
    #________________________________________________
        self.SortMode="ALPHA"
        self.update_actions()
    #________________________________________________

    #________________________________________________
    def togglePredefined(self,*args):
    #________________________________________________
        self.groupUserMade.set_sensitive(False)
        self.groupPredefined.set_sensitive(True)
    #________________________________________________

    #________________________________________________
    def toggleUserMade(self,*args):
    #________________________________________________
        self.groupUserMade.set_sensitive(True)
        self.groupPredefined.set_sensitive(False)
    #________________________________________________

    #________________________________________________
    def insert_variable(self,*args):
    #________________________________________________
        self.children["variables"].update(mode="SELECT",parent=self)
    #________________________________________________

    #________________________________________________
    def create_squelette(self,*args):
    #________________________________________________
        self.champ["label"].set_text(self.EditedAction+"(")
        self.champ["label2"].set_text(")")    

        act=Tools.string.lower(self.EditedAction)
        if act in GLOBAL.ACTIONS_WITHOUT_PARAMS:
            self.SaveParams=self.champ["params"].get_text()
            self.champ["params"].set_text("")
            self.champ["params"].hide()
            self.btVariables.set_sensitive(False)
        else:
            if self.SaveParams!=None:
                self.champ["params"].set_text(self.SaveParams)
            self.champ["params"].show()
            self.champ["params"].grab_focus()
            self.btVariables.set_sensitive(True)
            
    #________________________________________________

    #________________________________________________
    def toggle_wizard_button(self,*args):
    #________________________________________________
        wizard_available=(Tools.string.lower(self.EditedAction) in GLOBAL.ACTIONS_AVAILABLE_WIZARD)
        if wizard_available:
            self.btWizard.set_sensitive(True)
        else:
            self.btWizard.set_sensitive(False)
    #________________________________________________
    

    #________________________________________________
    def show_action_help(self,*args):
    #________________________________________________
        #Rcupration de la slection
        selected,iter=self.champ["actions"].get_selection().get_selected()
        if iter==None:
            Tools.debug("-- Error : "+_("ERROR_NO_SELECTION"))
            return
        self.EditedAction=self.modelActions.get_value(iter,0)

        category=self.EditedCategory
        if category=="[ALL]":
            for cat in GLOBAL.ACTIONS.keys():
                if self.EditedAction in GLOBAL.ACTIONS[cat].keys():
                    category=cat
                    break
        self.champ["help"].set_text(GLOBAL.ACTIONS[category][self.EditedAction])
        self.toggle_wizard_button()
        self.create_squelette()
    #________________________________________________

    #________________________________________________
    def update_actions(self,widget=None,category=None,selected=None):
    #________________________________________________
        if category==None:
            category=self.champ["category"].get_text()

        if category=="":
            return
        self.EditedCategory=category

        self.modelActions = Tools.Interface.gtk.ListStore(Tools.Interface.gobject.TYPE_STRING)

        listeactions=[]
        if category=="[ALL]":
            for cat in GLOBAL.ACTIONS.keys():
                for act in GLOBAL.ACTIONS[cat].keys():
                    listeactions.append(act)
        else:
            listeactions=GLOBAL.ACTIONS[category].keys()
        listeactions.sort()

        if self.SortMode=="MOSTUSED":
            #place les actions plus utilises en dbut de liste
            tempactlist=GLOBAL.ACTIONS_MOSTUSED.keys()
            while len(tempactlist)>0:
                min=None
                min=None
                #trouve le minimum
                for a in tempactlist:
                    if min==None or GLOBAL.ACTIONS_MOSTUSED[a]<min:
                        min=GLOBAL.ACTIONS_MOSTUSED[a]
                        minact=a

                if minact!=None:
                    #si minimum trouv, place l'action en dbut de liste
                    if minact in listeactions:
                        listeactions.remove(minact)
                        listeactions.insert(0,minact)
                    tempactlist.remove(minact)
                    
        #fill the model (reverse + insert at beginning)
        listeactions.reverse()
        for action in listeactions:
            iter=self.modelActions.insert(0,row=[action])

        self.champ["actions"].set_model(self.modelActions)

        if selected!=None:
            iter=self.modelActions.get_iter_first()
            trouve=None
            while (iter!=None and trouve==None):
                if self.modelActions.get_value(iter,0)==selected:
                    trouve=iter
                iter=self.modelActions.iter_next(iter)
            if trouve!=None:
                self.champ["actions"].set_cursor(self.modelActions.get_path(trouve))
                #selection=self.champ["actions"].get_selection()
                #selection.select_iter(trouve)

        self.champ["label"].set_text("")
        self.champ["params"].grab_focus()
    #________________________________________________

   
    #________________________________________________
    def update(self,iter=None,model=None,mode="CREATE"):
    #________________________________________________
        self.SaveParams=None
        self.parentmodel=model
        self.parentiter=iter
        self.EditedAction=""

        if iter==None:
            return
        
        self.mode=mode
        self.glade.get_widget("radio_usermade_act").set_active(True)
        self.toggleUserMade()
        self.glade.get_widget("radio_sortmost_editaction").set_active(True)
        self.toggle_wizard_button()

        #fill combo of scripts
        self.scripts=Tools.GetScriptsDic()
        list=self.scripts.keys()
        list.sort()
        self.champ["scripts"].fill(list)

        if self.mode=="EDIT":
            chaine=self.parentmodel.get_value(self.parentiter,0)
            chaine=chaine.split("(")
            action=chaine[0].split("::")[1].strip()
            params=""
            if len(chaine)>1:
                params=chaine[1].split(")")[0]

            self.update_actions(category=None,selected=action)

         
                
            self.champ["params"].set_text(params)
            self.EditedAction=action
            self.create_squelette()

        else:
            self.update_actions(category=None,selected=None)


        self.show()
    #________________________________________________


    #________________________________________________
    def update_parent(self,*args):
    #________________________________________________
        self.parent.clicOnListe()
        self.parent.listeExt.expand_all()
        self.parent.show()
    #________________________________________________



    #________________________________________________
    def valide(self,*args):
    #________________________________________________
       nb_actions=1
       actions=[]

       predef=self.glade.get_widget("radio_predefined_act").get_active()
       if predef:
           script=self.champ["scripts"].get_text().strip()
           actions=Tools.GetScript(self.scripts[script],script)
           nb_actions=len(actions)

       #update parent window
       iter=None
       prio_courante=1
           
       if self.mode=="CREATE":
           #get path of extension
           path=self.parentmodel.get_path(self.parentiter)

           #!! ATTENTION : c'est parti pour du gros code bourin !!
           #!! WARNING : complex code !! #
           #il faut grer les cas o on est plac dans l'arborescence ou bien en tte d'arborescence

           #Onerroritem est  True si on est sur la ligne ## CAS D'ERREUR ##
           Onerroritem=(self.parentmodel.get_value(self.parentiter,0).strip()[0:3]=="--[")
               
           if (len(path)==2 and not Onerroritem) or len(path)==3:
               #si on est plac au niveau 2 ou 3, on insere simplement  la suite
               realparentiter=self.parentmodel.iter_parent(self.parentiter)
               iter = self.parentmodel.insert_after(realparentiter,self.parentiter)
               prio_courante=int(self.parentmodel.get_value(self.parentiter,0).split("::")[0])+1

               #il faut aussi dcaler le reste, sinon a serait trop simple
               tempiter=iter
               while tempiter!=None:
                   tempiter=self.parentmodel.iter_next(tempiter)
                   if tempiter!=None:
                       #get current priority
                       cur=self.parentmodel.get_value(tempiter,0)
                       #check if that's not -- [ ON ERROR ] --
                       if not self.parent.onError(tempiter):
                           #set new priority
                           prio=Tools.ExtractPriority(self.parentmodel,tempiter)
                           if prio>=prio_courante:
                               Tools.SetPriority(self.parentmodel,tempiter,prio+nb_actions)
                   
           elif (len(path)==2 and Onerroritem) or len(path)==1:
               #si on est plac sur une extension, il faut ajouter l'action  la fin des enfants
               #on doit donc rcuprer la prio de la dernire action
               curiter=self.parentmodel.iter_children(self.parentiter)
               if curiter==None:
                   #pas encore d'enfants
                   prio_courante=1
                   #ajoute l'lment
                   iter= self.parentmodel.insert_before(self.parentiter,None)
                   if len(path)==1:
                       #ajoute le "##  ON ERROR ##"
                       iter_on=self.parentmodel.insert_before(self.parentiter,None)
                       self.parentmodel.set_value(iter_on,0,"--[ "+_("ITEM_ERROR")+" ]--")
                   else:
                       prio_courante=102
               else:
                   #cherche la dernire action avant ## ON ERROR ##
                   tempiter=1
                   onerrorreached=self.parent.onError(curiter)
                   #vrifie que le premier enfant n'est pas ## ON ERROR ##
                   if not onerrorreached:
                       while tempiter!=None and onerrorreached==False:
                           tempiter=self.parentmodel.iter_next(curiter)
                           if tempiter!=None:
                               onerrorreached=self.parent.onError(tempiter)
                               if not onerrorreached:
                                   curiter=tempiter
                       #rcupre la priorit +1
                       prio_courante=Tools.ExtractPriority(self.parentmodel,curiter)+1
                       #ajoute l'lment
                       iter= self.parentmodel.insert_after(self.parentiter,curiter)
                   else:
                       #sinon on insre juste avant
                       iter= self.parentmodel.insert_before(self.parentiter,curiter)
                       prio_courante=1

           else:
               #si on arrive l, c'est qu'on est dans un autre niveau d'arborescence
               #peut-tre qu'il suffit alors de se jeter par la fentre... ;)
               raise "INTERNAL ERROR ! The tree shouldn't have more than 2 levels..."
               return

       elif self.mode=="EDIT":
           #n'oublions pas le mode EDITION : plus simple
           iter=self.parentiter
           prio_courante=Tools.ExtractPriority(self.parentmodel,self.parentiter)
           
       #hop! update value
       if predef:
           #add all actions from selected script
           j=1
           for a in actions:
               self.parentmodel.set_value(iter, 0,str(prio_courante)+" :: "+a)
               prio_courante+=1
               j+=1
               if j<len(actions):
                   iter= self.parentmodel.insert_after(None,iter)
       else:
           #add just one action
           self.parentmodel.set_value(iter, 0,str(prio_courante)+" :: "+ self.champ["label"].get_text()+self.champ["params"].get_text()+")")
           
       #select the row
       treeselection = self.parent.listeExt.get_selection()
       treeselection.select_iter(iter)

       #manage most used actions
       if self.EditedAction!=None:
           self.EditedAction=self.EditedAction.strip()
           if self.EditedAction in GLOBAL.ACTIONS_MOSTUSED.keys():
               GLOBAL.ACTIONS_MOSTUSED[self.EditedAction]+=1
           else:
               GLOBAL.ACTIONS_MOSTUSED[self.EditedAction]=1

       #close window
       self.close()
    #________________________________________________



#_______________________________________________________________________



#----------------------------------------------------------------------------------------
#------------------------------------- FENETRE EDIT EXTENSION ---------------------------
#----------------------------------------------------------------------------------------


#________________________________________________
class WEditExtensionCtrl(WindowCtrl):
#________________________________________________

    #________________________________________________
    def __init__(self,parent):
    #________________________________________________
        WindowCtrl.__init__(self,"Fen_editExtension",parent)


        self.handlers = {"on_btannuler_editext_clicked" : self.close,
                         "on_btvalider_editext_clicked" : self.valide,
                         "on_entry_ext_editext_changed" : self.gereEntryMask,
                         "on_radio_usermade_ext_toggled" : self.toggleUserMade,
                         "on_radio_predefined_ext_toggled" : self.togglePredefined
                         }

        self.glade.signal_autoconnect(self.handlers)
        self.champ={}

        self.listeCtxt=self.glade.get_widget("Listectxt_editctxt")
        self.entryExt=self.glade.get_widget("entry_ext_editext")
        self.btValider=self.glade.get_widget("btValider_editext")

        self.comboPredefinedExt=Tools.GladeComboContainer(self.glade,"predefined_ext")
        self.champ["radio_predefined"]= self.glade.get_widget("radio_predefined_ext")
        self.champ["radio_usermade"]= self.glade.get_widget("radio_usermade_ext")

        self.champ["radio_predefined"].set_active(True)

        self.saveExt=""
        self.groupUserMade=Tools.GladeGroupContainer([self.glade.get_widget("hbox_usermade_editext"),self.glade.get_widget("label_usermade_editext")])
        self.groupUserMade.desactivate()

        #Init
     
        self.EditedExtension=""
        self.MASK={"NORMAL":"XZN#1234567890.[","BRACKET":"123456789-]"}
    #________________________________________________


    #________________________________________________
    def toggleUserMade(self,*args):
    #________________________________________________
        self.groupUserMade.activate()
        self.comboPredefinedExt.desactivate()
        self.btValider.set_sensitive((len(self.entryExt.get_text().strip())>0))

        self.entryExt.grab_focus()
    #________________________________________________

    #________________________________________________
    def togglePredefined(self,*args):
    #________________________________________________
        self.groupUserMade.desactivate()
        self.comboPredefinedExt.activate()
        self.btValider.set_sensitive(True)

    #________________________________________________


    #________________________________________________
    def gereEntryMask(self,*args):
        """ Gestion du masque de saisie """
    #________________________________________________
        texte=Tools.string.upper(self.entryExt.get_text())
        #Evite le rebouclage
        if texte==self.saveExt:
            return

        #check entry
        error=False
        maskMode="NORMAL"
        for ch in texte:
            if ch not in self.MASK[maskMode]:
                error=True

            if ch=="[":
                #entering mode BRACKET
                if maskMode=="BRACKET":
                    error=True
                else:    
                    maskMode="BRACKET"
            elif ch=="]":
                #quitting mode BRACKET
                if maskMode=="NORMAL":
                    error=True
                else:
                    maskMode="NORMAL"
                
            if error:
                Tools.debug("Invalid character "+ch)
                break

        if not error:
            self.saveExt=texte

        #update entry and buttons
        self.entryExt.set_text(self.saveExt)
        self.btValider.set_sensitive((len(self.saveExt)>0 or self.champ["radio_predefined"].get_active()))
    #________________________________________________


    #________________________________________________
    def update(self,iter=None,model=Tools.Interface.gtk.TreeStore(Tools.Interface.gobject.TYPE_STRING)):
    #________________________________________________
        if model==None:
            raise "INTERNAL ERROR ! No model to update"
            return

        self.model=model
        ext=""
        list_predef_ext=Tools.copy.deepcopy(GLOBAL.DEFAULT_EXTENSIONS)

        self.champ["radio_predefined"].set_sensitive(True)
        
        if iter!=None:
            #mode EDIT
            self.comboPredefinedExt.fill(list_predef_ext)

            self.EditedExt=iter
            ext=model.get_value(iter,0)
           
            if ext in GLOBAL.DEFAULT_EXTENSIONS:
                self.champ["radio_predefined"].set_active(True)
                self.comboPredefinedExt.set_text(ext)
            else:
                self.champ["radio_usermade"].set_active(True)
                if ext[0]=="_":
                    ext=ext[1:]
                self.entryExt.set_text(ext)
                
            self.mode="EDIT"
                        
        else:
            #mode CREATE

            self.entryExt.set_text("")
            self.mode="CREATE"

            #put in combo only predef extensions that are not in context
            self.comboPredefinedExt.clear()
            list_modelext=Tools.GetListFromModel(self.model)
            for it in list_predef_ext:
                if it not in list_modelext:
                    self.comboPredefinedExt.add_item(it,False)
            self.comboPredefinedExt.fill()
            if self.comboPredefinedExt.getNbItems()==0:
                self.champ["radio_predefined"].set_sensitive(False)
                self.champ["radio_predefined"].set_active(False)
                self.champ["radio_usermade"].set_active(True)
                
                    
        self.btValider.set_sensitive((len(ext)>0 or self.champ["radio_predefined"].get_active()))
        self.entryExt.grab_focus()
        self.show()
    #________________________________________________
    

    #________________________________________________
    def valide(self,*args):
    #________________________________________________

       if self.champ["radio_predefined"].get_active():
           ext=self.comboPredefinedExt.get_text()
       else:
           #complete extension if needed
           ext=Tools.string.upper(self.entryExt.get_text())
           for ch in "XZN.[]":
               if ch in ext:
                   ext="_"+ext
                   break
           Tools.debug("Extension : "+ext)

       
       #update parent window
       iter=None
       if self.mode=="CREATE":
           #verify if the extension can be added to the list
           if Tools.ItemExistsInList(ext,self.model):
               Tools.debug("-- Error : "+_("ERROR_EXT_ALREADY_EXISTS"))
               return

           iter = self.model.insert_before(None, None)
           
       elif self.mode=="EDIT":

           #verify if the extension can be added to the list
           if Tools.ItemExistsInList(ext,self.model,exception=self.EditedExt):
               Tools.debug("-- Error : "+_("ERROR_EXT_ALREADY_EXISTS"))
               return

           iter = self.EditedExt

       self.model.set_value(iter, 0, ext)
       treeselection = self.parent.listeExt.get_selection()
       treeselection.select_iter(iter)
       self.parent.currentExtIter=iter

       self.hide()
       self.parent.listeExt.expand_all()
       self.parent.listeExt.show()
       self.parent.show()
    #________________________________________________
   

    #________________________________________________
    def update_parent(self,*args):
    #________________________________________________
        self.parent.clicOnListe()
        self.parent.show()
    #________________________________________________


#----------------------------------------------------------------------------------------
#------------------------------------- FENETRE EDIT CONTEXT ------------------------------
#----------------------------------------------------------------------------------------


#________________________________________________
class WEditContextCtrl(WindowCtrl):
#________________________________________________

    #________________________________________________
    def __init__(self,parent):
    #________________________________________________
        WindowCtrl.__init__(self,"Fen_editContext",parent)

        self.children["selectContext"]=WSelectContextCtrl(self)
        self.children["editExtension"]=WEditExtensionCtrl(self)
        self.children["editAction"]=WEditActionCtrl(self)
        self.children["editSwitch"]=WSwitchCtrl(self)

        self.handlers = {"on_btannuler_editctxt_clicked" : self.close,
                         "on_btvalider_editctxt_clicked" : self.valide,
                         "on_btinclude_editctxt_clicked" : self.includeContext,
                         "on_btexclude_editctxt_clicked" : self.excludeContext,
                         "on_btaddext_editctxt_clicked" : self.addExtension,
                         "on_btaddaction_editctxt_clicked" : self.addAction,
                         "on_bteditext_editext_clicked" : self.editExtension,
                         "on_btsuppr_editctxt_clicked" : self.supprExtension,
                         "on_listeext_editctxt_row_activated" : self.editExtension,
                         "on_listeext_editctxt_move_cursor" : self.clicOnListe,
                         "on_bt_priodown_editctxt_clicked" : self.priority_down,
                         "on_bt_prioup_editctxt_clicked" : self.priority_up,
                         "on_btswitch_editctxt_clicked" : self.editswitchlines
                         }

        self.glade.signal_autoconnect(self.handlers)

        self.champ={}
        self.champ["name"]=self.glade.get_widget("entry_editctxt_name")
        self.groupPriority=self.glade.get_widget("vbox_priority_editctxt")
        self.btEdit=self.glade.get_widget("btEditExt_editctxt")
        self.btDelete=self.glade.get_widget("btSuppr_editctxt")
        self.groupOnError=Tools.GladeGroupContainer([self.btEdit,self.btDelete])
        
        self.btAddAction=self.glade.get_widget("btAddAction_editctxt")
        
        self.listeCtxt=self.glade.get_widget("Listectxt_editctxt")
        self.listeExt=self.glade.get_widget("Listeext_editctxt")


        #Init
        self.EditedContext=""
        self.currentExtIter=None
        Tools.Build_liste(self.listeCtxt,["Included contexts"])
        Tools.Build_liste(self.listeExt,["Extension"])
        self.listeExt.set_reorderable(False)

        self.modelExt = Tools.Interface.gtk.TreeStore(Tools.Interface.gobject.TYPE_STRING)
        self.modelCtxt = Tools.Interface.gtk.TreeStore(Tools.Interface.gobject.TYPE_STRING)
        self.listeExt.set_model(self.modelExt)
        self.switchLines=[]
    #________________________________________________

    #________________________________________________
    def editswitchlines(self,*args):
    #________________________________________________
    	self.children["editSwitch"].update()
    #________________________________________________

    #________________________________________________
    def priority_up(self,*args):
    #________________________________________________
        #decrease priority (up in model)
        
        iter=iter_up=self.currentExtIter
        iterpath = self.modelExt.get_path(iter)
        lenpath=len(iterpath)

        oldprio=Tools.ExtractPriority(self.modelExt,iter)
        if oldprio>1 and not (lenpath==3 and oldprio<=102):
            Tools.SetPriority(self.modelExt,iter,oldprio-1)
        else:
            return

        if iterpath[lenpath-1]-1>=0:
            if lenpath==2:
                iter_up=self.modelExt.get_iter((iterpath[0],iterpath[1]-1))
            elif lenpath==3:
                iter_up=self.modelExt.get_iter((iterpath[0],iterpath[1],iterpath[2]-1))

            prioup=Tools.ExtractPriority(self.modelExt,iter_up)
            if prioup>=oldprio-1:
                #Tools.SetPriority(self.modelExt,iter,oldprio-2)                
                Tools.SwapRows(self.modelExt,iter,iter_up)
                self.listeExt.set_cursor(self.modelExt.get_path(iter_up))
                if prioup==oldprio-1:
                    Tools.SetPriority(self.modelExt,iter,oldprio)                

    #________________________________________________

    #________________________________________________
    def priority_down(self,*args):
    #________________________________________________
        #increase priority (down in model)
        
        iter=iter_down=self.currentExtIter
        if iter==None:
            return
        iterpath = self.modelExt.get_path(iter)
        lenpath=len(iterpath)

        oldprio=Tools.ExtractPriority(self.modelExt,iter)
        if not (lenpath==2 and oldprio>=101) and oldprio!=-1:
            Tools.SetPriority(self.modelExt,iter,oldprio+1)
        else:
            return

        try:
            if lenpath==2:
                iter_down=self.modelExt.get_iter((iterpath[0],iterpath[1]+1))
            elif lenpath==3:
                iter_down=self.modelExt.get_iter((iterpath[0],iterpath[1],iterpath[2]+1))
                
            priodown=Tools.ExtractPriority(self.modelExt,iter_down)
            if priodown<=oldprio+1:
                Tools.SwapRows(self.modelExt,iter,iter_down)
                self.listeExt.set_cursor(self.modelExt.get_path(iter_down))
                if priodown==oldprio+1:
                    Tools.SetPriority(self.modelExt,iter,oldprio)                
        except:
            pass
    #________________________________________________

    #________________________________________________
    def onError(self,iter):
    #________________________________________________
        return (self.modelExt.get_value(iter,0).strip()[0:3]=="--[")
    #________________________________________________

    #________________________________________________
    def clicOnListe(self,*args):
    #________________________________________________
        #handler when clic on list of extensions
        
        selected,self.currentExtIter=self.listeExt.get_selection().get_selected()
        if self.currentExtIter==None:
            return
        path=self.modelExt.get_path(self.currentExtIter)
        if len(path)==1:
            self.groupPriority.set_sensitive(False)
            self.groupOnError.set_sensitive(True)
        else:
            onerror=self.onError(self.currentExtIter)
            self.groupPriority.set_sensitive(not onerror)
            self.groupOnError.set_sensitive(not onerror)
    #________________________________________________

    #________________________________________________
    def addAction(self,*args):
        """Clic on 'add action'"""
    #________________________________________________
        #Rcupration de la slection
        selected,self.currentExtIter=self.listeExt.get_selection().get_selected()
        if self.currentExtIter==None:
            Tools.debug("-- Error : "+_("ERROR_NO_SELECTION"))
            return

        self.children["editAction"].update(self.currentExtIter,self.modelExt)

    #________________________________________________


    #________________________________________________
    def includeContext(self,*args):
        """Delete include of context from current extension"""
    #________________________________________________
        self.children["selectContext"].update(self.modelCtxt)
    #________________________________________________


    #________________________________________________
    def excludeContext(self,*args):
        """Delete include of context from current extension"""
    #________________________________________________
        #Rcupration de la slection
        selected,iter=self.listeCtxt.get_selection().get_selected()
        if iter==None:
            Tools.debug("-- Error : "+_("ERROR_NO_SELECTION"))
            return

        self.modelCtxt.remove(iter)
        self.listeCtxt.set_model(self.modelCtxt)
        self.listeCtxt.show()
    #________________________________________________


    #________________________________________________
    def addExtension(self,*args):
        """Add an extension to current context"""
    #________________________________________________
        self.btAddAction.set_sensitive(True)
        self.children["editExtension"].update(None,self.modelExt)
    #________________________________________________


    #________________________________________________
    def supprExtension(self,*args):
        """Delete an line from current context"""
    #________________________________________________

        #Get iterator on selected item
        selected,iter=self.listeExt.get_selection().get_selected()
        if iter==None:
            Tools.debug("-- Error : "+_("ERROR_NO_SELECTION"))
            return

        #Ask for confirmation
        l=""
        path=self.modelExt.get_path(iter)
        if len(path)>1:
            l="1_ACTION"
        else:
            l="1_EXTENSION"

        result=Tools.Interface.DialogBox(_("MSG_CONFIRM_DELETE_X",GLOBAL.WORDS[l]),type_buttons=Tools.Interface.gtk.BUTTONS_YES_NO,type_dialog=Tools.Interface.gtk.MESSAGE_QUESTION)
        if result==Tools.Interface.gtk.RESPONSE_NO:
            return

        
        #Select row
        newsel=self.modelExt.iter_next(iter)
        self.modelExt.remove(iter)

        if newsel==None:
            newsel=self.modelExt.iter_parent(iter)
            if newsel==None:
                newsel=self.modelExt.get_iter_root()
        else:
            #Decrease priority of the others
            savecur=self.currentExtIter
            self.currentExtIter=newsel
            while self.currentExtIter!=None and not self.onError(self.currentExtIter):
                self.priority_up()
                self.currentExtIter=self.modelExt.iter_next(self.currentExtIter)
            self.currentExtIter=savecur
                
        

        if newsel!=None:
            treeselection = self.listeExt.get_selection()
            treeselection.select_iter(newsel)

        self.clicOnListe()
        self.listeExt.show()
    #__________________________________________________________

    #________________________________________________
    def editExtension(self,*args):
        """Edit an line in current context"""
    #________________________________________________
    
        if self.currentExtIter==None:
            Tools.debug("-- Error : "+_("ERROR_NO_SELECTION"))
            return

        path=self.modelExt.get_path(self.currentExtIter)
        if len(path)>1:
            #edit action
            if not self.onError(self.currentExtIter):
                self.children["editAction"].update(self.currentExtIter,self.modelExt,"EDIT")
            else:
                if not self.modelExt.iter_has_child(self.currentExtIter):
                    self.children["editAction"].update(self.currentExtIter,self.modelExt)
                else:
                    if self.listeExt.row_expanded(path):
                        self.listeExt.collapse_row(path)
                    else:
                        self.listeExt.expand_row(path,True)
                
        else:
            #edit extension
            self.children["editExtension"].update(self.currentExtIter,self.modelExt)
    #__________________________________________________________


    #________________________________________________
    def update(self,model,iter=None):
        """Prpare les champs d'dition"""
        #si Model==None : MODE CREATE
        #si Model!=None et iter!=None : MODE EDIT
    #________________________________________________

        self.currentExtIter=None
        self.btAddAction.set_sensitive(True)

        #Rcupration du champ d'information pour afficher le mode
        lab=self.glade.get_widget("label_info_editctxt")

        self.modelExt = Tools.Interface.gtk.TreeStore(Tools.Interface.gobject.TYPE_STRING)
        self.modelCtxt= Tools.Interface.gtk.TreeStore(Tools.Interface.gobject.TYPE_STRING)
        
        if model==None:
            #PREPARATION DU MODE CREATION
            
            #Affectation des valeurs par dfaut aux champs
            self.EditedContext=None
            
            self.listeExt.set_model(self.modelExt)
            self.listeCtxt.set_model(self.modelCtxt)
            self.champ["name"].set_text("")
            del(self.switchLines)
            self.switchLines=[]
            
            #Passage en mode CREATION
            self.mode = "CREATE"
            lab.set_text("CREATE NEW CONTEXT")

        elif iter!=None:
            #PREPARATION DU MODE EDITION

            #Affectation des valeurs de l'utilisateur courant aux champs
            self.EditedContext=model.get_value(iter,1)
            Tools.debug("Edited context ="+str(self.EditedContext))
            self.champ["name"].set_text(GLOBAL.ListeContextes[self.EditedContext].name)

            if not GLOBAL.ListeContextes[self.EditedContext].put_in_models(self.modelExt,self.modelCtxt):
                self.close()
                return
            
            self.listeExt.set_model(self.modelExt)
            self.listeCtxt.set_model(self.modelCtxt)
            del(self.switchLines)
            self.switchLines=GLOBAL.ListeContextes[self.EditedContext].switchLines

            
            #Passage en mode EDITION
            self.mode = "EDIT"
            lab.set_text("EDIT CONTEXT")

        else:
            #Test de scurit, on n'est pas cens venir ici
            raise "INTERNAL ERROR ! Null iterator on non-null model !"
            self.mode = "NORMAL"
            return

        #Afficher la fentre
        if len(self.modelExt)==0:
            self.btAddAction.set_sensitive(False)
        self.champ["name"].grab_focus()
        self.listeExt.expand_all()
        self.listeExt.show()
        self.listeCtxt.show()
        self.show()
    #________________________________________________


    #________________________________________________
    def valide(self,*args):
    #________________________________________________
             
        self.currentext=None
        
        #fonction de cration du contexte
        #add extension content in the context
        def addextincontext(model, path, iter, context):
            curval=model.get_value(iter,0)
            if len(path)==1:
                self.currentext=curval
                context.setextension(self.currentext)
            elif (len(path)==2 or len(path)==3) and self.currentext!=None:
                if not self.onError(iter):
                    action=curval.split("::")
                    prior=action[0].strip()
                    action=action[1].strip()
                    context.addline(self.currentext,prior,action)
                

        def includectxt(model, path, iter, context):
            context.include(model.get_value(iter,0))

        #get name of context
        nom=self.champ["name"].get_text().strip()

        #create temp object
        Tools.debug("Building context...")
        tempContext=Context.Context(nom)
        self.modelExt.foreach(addextincontext,tempContext)
        self.modelCtxt.foreach(includectxt,tempContext)
        del(tempContext.switchLines)
        tempContext.switchLines=self.switchLines
        
        Tools.debug("*******************************************************")
        Tools.debug(str(tempContext.toString()))
        Tools.debug("*******************************************************")
        
        #add it or replace it in GLOBAL list
        if self.mode=="CREATE":
            if Tools.ItemExistsInList(nom,self.modelExt):
                Tools.debug("-- Error : "+_("ERROR_CTXT_ALREADY_EXISTS"))
                return
            GLOBAL.ListeContextes.append(tempContext)
            
        elif self.mode=="EDIT":
            del(GLOBAL.ListeContextes[self.EditedContext])
            GLOBAL.ListeContextes.insert(self.EditedContext,tempContext)
        else:
            #---> insulter le programmeur si on arrive ici
            raise "INTERNAL ERROR ! Window Mode Unknown !"
            return

        self.parent.EditedContext=self.EditedContext
        self.close()
        #update parent automatically (see WindowCtrl class)
    #________________________________________________


    

#________________________________________________________________


#----------------------------------------------------------------------------------------
#------------------------------------- FENETRE SWITCH ----------------------------------
#----------------------------------------------------------------------------------------

#________________________________________________
class WSwitchCtrl(WindowCtrl):
#________________________________________________

    #________________________________________________
    def __init__(self,parent):
    #________________________________________________
        WindowCtrl.__init__(self,"Fen_switch",parent)

        #definition des handlers d'vnements
        self.handlers = {"on_btvalider_switch_clicked" : self.valide,
                         "on_btannuler_switch_clicked" : self.close,
                         "on_btnew_switch_clicked":self.newline,
                         "on_btremove_switch_clicked":self.removeline,
                         "on_btedit_switch_clicked":self.editline,
                         }
        

        self.glade.signal_autoconnect(self.handlers)

        self.listeLines=self.glade.get_widget("liste_switch")
        self.modelLines = Tools.Interface.gtk.TreeStore(Tools.Interface.gobject.TYPE_STRING,Tools.Interface.gobject.TYPE_BOOLEAN)
        self.listeLines.set_model(self.modelLines)
        #Init
        self.EditedContext=""
        Tools.Build_liste(self.listeLines,["Switch lines"],ceditable=1,model=self.modelLines)
    #________________________________________________


    #________________________________________________
    def newline(self,*args):
    #________________________________________________
        iter=self.modelLines.insert_before(None,None)
        self.modelLines.set_value(iter,0,"")
        self.modelLines.set_value(iter,1,Tools.Interface.gtk.TRUE)
        column=self.listeLines.get_column(0)
        column.set_flags(Tools.Interface.gtk.CAN_FOCUS)
        
        self.listeLines.set_cursor(self.modelLines.get_path(iter),column,start_editing=Tools.Interface.gtk.TRUE)
    #________________________________________________
	
    #________________________________________________
    def editline(self,*args):
    #________________________________________________
	    #Get iterator on selected item
        selected,iter=self.listeLines.get_selection().get_selected()
        if iter==None:
            Tools.debug("-- Error : "+_("ERROR_NO_SELECTION"))
            return
        path,column=self.listeLines.get_cursor()
        column.set_flags(Tools.Interface.gtk.CAN_FOCUS)
        self.listeLines.set_cursor(path, column, start_editing=Tools.Interface.gtk.TRUE)
	#________________________________________________
	
    #________________________________________________
    def removeline(self,*args):
    #________________________________________________
	    #Get iterator on selected item
        selected,iter=self.listeLines.get_selection().get_selected()
        if iter==None:
            Tools.debug("-- Error : "+_("ERROR_NO_SELECTION"))
            return

        #Select row
        newsel=self.modelLines.iter_next(iter)
        self.modelLines.remove(iter)

        if newsel==None:
            newsel=self.modelLines.get_iter_root()
        
        if newsel!=None:
            self.listeLines.get_selection().select_iter(newsel)

        self.listeLines.show()
   	#________________________________________________

   	
    #________________________________________________
    def update_parent(self,*args):
    #________________________________________________
        self.parent.show()
    #________________________________________________

    #________________________________________________
    def update(self):
    #________________________________________________
        self.EditedContext=self.parent.EditedContext
        
        self.modelLines.clear()

        for l in self.parent.switchLines:
                iter=self.modelLines.insert_before(None,None)
                self.modelLines.set_value(iter,0,l)
                self.modelLines.set_value(iter,1,Tools.Interface.gtk.TRUE)
        self.listeLines.set_model(self.modelLines)
        self.listeLines.show()
        self.show()
    #________________________________________________


    #________________________________________________
    def valide(self,*args):
    #________________________________________________
        def includeline(model, path, iter, liste):
            liste.append(model.get_value(iter,0))

        del(self.parent.switchLines)
        self.parent.switchLines=[]
        self.modelLines.foreach(includeline,self.parent.switchLines)
        self.close()
    #________________________________________________



#________________________________________________

#----------------------------------------------------------------------------------------
#------------------------------------- FENETRE SELECT CTXT ------------------------------
#----------------------------------------------------------------------------------------

#________________________________________________
class WSelectContextCtrl(WindowCtrl):
#________________________________________________

    #________________________________________________
    def __init__(self,parent):
    #________________________________________________
        WindowCtrl.__init__(self,"Fen_selectContext",parent)
        self.handlers = {"on_btannuler_selectctxt_clicked" : self.close,
                         "on_btvalider_selectctxt_clicked" : self.valide
                         }
        self.glade.signal_autoconnect(self.handlers)

        self.comboCtxt=Tools.GladeComboContainer(self.glade,"ctxt_selectctxt")
        
        #Init
        self.modelparent=None
    #________________________________________________

    #________________________________________________
    def update_parent(self,*args):
    #________________________________________________
        #specific function for parent update
        self.parent.listeCtxt.set_model(self.modelparent)
        self.parent.show()
    #________________________________________________


    #________________________________________________
    def update(self,model=None):
    #________________________________________________
        def removectxt(model,path,iter,list):
            val=model.get_value(iter,0)
            if val in list:
                list.remove(val)

        if model==None:
            Tools.debug("-- Error : model can't be null !")
            return
        self.modelparent=model

        #prpare la liste des contextes
        l=[]
        for c in GLOBAL.ListeContextes:
            l.append(c.name)
        #enleve les contextes dj inclus
        self.modelparent.foreach(removectxt,l)
        #rempli la combo
        self.comboCtxt.fill(l)

        self.show()
    #________________________________________________


    #________________________________________________
    def valide(self,*args):
        """ clic on 'valider' bouton """
    #________________________________________________
        if self.modelparent==None:
            raise "INTERNAL ERROR ! Null model !"
            return
        
        iter=self.modelparent.insert_before(None,None)
        self.modelparent.set_value(iter,0,self.comboCtxt.get_text())
        self.close()
    #________________________________________________


#____________________________________________________________________
    


#----------------------------------------------------------------------------------------
#-------------------------------- FENETRE DE SELECTION DE FICHIERS ----------------------
#----------------------------------------------------------------------------------------

#________________________________________________
class WSelectFileCtrl(WindowCtrl):
#________________________________________________

    #________________________________________________
    def __init__(self,parent):
    #________________________________________________
        WindowCtrl.__init__(self,"Fen_selectFile",parent)
        self.handlers = {"on_btok_cf_clicked" : self.valide,
                         "on_btcancel_cf_clicked" : self.close
                         }

        self.glade.signal_autoconnect(self.handlers)
        #Init
        self.field=None
    #________________________________________________


    #________________________________________________
    def update_parent(self,*args):
    #________________________________________________
        self.parent.show()
    #________________________________________________


    #________________________________________________
    def update(self,mode,field=None):
    #________________________________________________
        self.mode=mode
        self.field=field
        self.show()
    #________________________________________________


    #________________________________________________
    def valide(self,*args):
        """ Appui sur bouton Valider dans fentre sf"""
    #________________________________________________
        #Rcupration du nom de fichier slecionn
        filename=self.widget.get_filename()
        Tools.debug("File selected : "+filename)

        #remove path and gsm extension if present
        fn=filename.split("/")
        fn=fn[len(fn)-1].strip()
        fn=fn.rstrip(".gsm")
        filename=fn
        
        #Sauve ou charge le fichier selon le mode de la fentre
        if self.mode=="SAVE_ACTOS":
            Filesystem.save_actos_file(filename)
        elif self.mode=="LOAD_ACTOS":
            result,l1,l2=Filesystem.load_actos_file(filename)
            if result==True:
                GLOBAL.ListeUsers=l1
                GLOBAL.ListeContextes=l2
        elif self.mode=="SELECT_FILE":
            self.field.set_text(filename)
        else:
            Tools.debug("-- Error : Mode unknown for window selectFile !")
        
        #Masque la fentre et change le mode
        self.mode="NORMAL"
        self.close()
    #________________________________________________

#_______________________________________________________________





#----------------------------------------------------------------------------------------
#------------------------------------- FENETRE JITTER BUFFER ----------------------------
#----------------------------------------------------------------------------------------


#________________________________________________________________________
class WJitterBufferCtrl(WindowCtrl):
#________________________________________________________________________

    #________________________________________________
    def __init__(self,parent):
    #________________________________________________
        WindowCtrl.__init__(self,"Fen_jitter",parent)

        #definition des handlers d'vnements
        self.handlers = {"on_btvalider_jitter_clicked" : self.valide,
                         "on_btannuler_jitter_clicked" : self.close,
                         "on_check_jitter_toggled" : self.toggleJitter
                         }
        

        self.glade.signal_autoconnect(self.handlers)
        self.checkJitter=self.glade.get_widget("check_jitter")

        self.dropCount=self.glade.get_widget("spin_dropcount")
        self.maxBuffer=self.glade.get_widget("spin_maxjitterbuffer")
        self.maxExcess=self.glade.get_widget("spin_maxjitterexcess")
        self.groupSpins=Tools.GladeGroupContainer([self.dropCount,self.maxBuffer,self.maxExcess])

        #Init
        self.EditedProtocol=""
        self.mode="NORMAL"
    #________________________________________________


    #________________________________________________
    def toggleJitter(self,*args):
    #________________________________________________
         self.groupSpins.set_sensitive(self.checkJitter.get_active())
    #________________________________________________
        
    #________________________________________________
    def update_parent(self,*args):
    #________________________________________________
        #specific function for parent update
        self.parent.listeProtocols.set_sensitive(True)
        self.parent.listeProtocols.grab_focus()
        self.parent.select_protocol(self.parent.listeProtocols)
    #________________________________________________


    #________________________________________________
    def update(self,parentmodel,iter):
    #________________________________________________
        self.EditedProtocol=parentmodel.get_value(iter,0).strip()
        if self.EditedProtocol!="iax2":
            raise "INTERNAL ERROR ! This window can't be shown for others protocols than iax !"
            return
            
        active=(GLOBAL.PROTOCOLS["iax2"]["jitterbuffer"]=="yes")
        self.checkJitter.set_active(active)

        self.dropCount.set_value(float(GLOBAL.PROTOCOLS["iax2"]["dropcount"]))
        self.maxBuffer.set_value(float(GLOBAL.PROTOCOLS["iax2"]["maxjitterbuffer"]))
        self.maxExcess.set_value(float(GLOBAL.PROTOCOLS["iax2"]["maxexcessjitterbuffer"]))

        self.toggleJitter()
        self.show()
    #________________________________________________


    #________________________________________________
    def valide(self,*args):
    #________________________________________________
        active=self.checkJitter.get_active()
        GLOBAL.PROTOCOLS["iax2"]["jitterbuffer"]={False:"no",True:"yes"}[active]
        GLOBAL.PROTOCOLS["iax2"]["dropcount"]=str(self.dropCount.get_value()).split(".")[0]
        GLOBAL.PROTOCOLS["iax2"]["maxjitterbuffer"]=str(self.maxBuffer.get_value()).split(".")[0]
        GLOBAL.PROTOCOLS["iax2"]["maxexcessjitterbuffer"]=str(self.maxExcess.get_value()).split(".")[0]
        GLOBAL.TEMP_PROTOCOLS=Tools.copy.deepcopy(GLOBAL.PROTOCOLS)
        self.close()
    #_______________________________________________


#________________________________________________________________________



#----------------------------------------------------------------------------------------
#------------------------------------- FENETRE CODECS -----------------------------------
#----------------------------------------------------------------------------------------


#________________________________________________________________________
class WCodecsCtrl(WindowCtrl):
#________________________________________________________________________

    #________________________________________________
    def __init__(self,parent):
    #________________________________________________
        WindowCtrl.__init__(self,"Fen_codecs",parent)

        #definition des handlers d'vnements
        self.handlers = {"on_btvalider_codecs_clicked" : self.valide,
                         "on_btannuler_codecs_clicked" : self.close,
                         "on_entry_codecs_bandwidth_changed" : self.changeBand,
                         "on_list_codecs_cursor_changed" : self.selectCodec,
                         "on_bttoggle_codecs_clicked" : self.toggleCodec
                         }
        

        self.glade.signal_autoconnect(self.handlers)

        self.listeCodecs=self.glade.get_widget("list_codecs")
        self.modelCodecs = Tools.Interface.gtk.ListStore(Tools.Interface.gobject.TYPE_STRING,Tools.Interface.gobject.TYPE_STRING)

        self.comboBand=Tools.GladeComboContainer(self.glade,"codecs_bandwidth")
        self.labelInfo=self.glade.get_widget("label_codecs_info")

        #Init
        self.EditedProtocol=""
        self.EditedCodec=None
        self.mode="NORMAL"
        Tools.Build_liste(self.listeCodecs,["Codec","State"])
        self.modelCodecs=self.listeCodecs.get_model()

        #fill combo bandwidth
        self.comboBand.fill(GLOBAL.CODECS.keys())

    #________________________________________________

    #________________________________________________
    def update_parent(self,*args):
    #________________________________________________
        #specific function for parent update
        self.parent.listeProtocols.set_sensitive(True)
        self.parent.listeProtocols.grab_focus()
        self.parent.select_protocol(self.parent.listeProtocols)
    #________________________________________________


    #________________________________________________
    def selectCodec(self,*args):
        """ clic on codec list """
    #________________________________________________
       #Rcupration de la slection
        selected,iter=self.listeCodecs.get_selection().get_selected()
        if iter==None:
            Tools.debug("-- Error : "+_("ERROR_NO_SELECTION"))
            return
    
        self.EditedCodec=iter
    #________________________________________________


    #________________________________________________
    def createCodecsList(self,*args):
    #________________________________________________
        def func(model, path, iter, protocol):
            allowed=model.get_value(iter,1).strip()
            if allowed=="allow":
                l.append(model.get_value(iter,0).strip())
        
        if self.modelCodecs==None:
            return

        l=[]
        self.modelCodecs.foreach(func,l)
        GLOBAL.TEMP_PROTOCOLS[self.EditedProtocol]["codecs"]=Tools.copy.deepcopy(l)
        GLOBAL.TEMP_PROTOCOLS[self.EditedProtocol]["bandwidth"]=self.comboBand.get_text()
   #________________________________________________
    

    #________________________________________________
    def toggleCodec(self,*args):
    #________________________________________________
        if self.modelCodecs==None or self.EditedCodec==None:
            return
        allowed=self.modelCodecs.get_value(self.EditedCodec,1).strip()
        if allowed=="allow":
            allowed="disallow"
        else:
            allowed="allow"
    
        self.modelCodecs.set_value(self.EditedCodec,1,allowed)
        self.createCodecsList()
    #________________________________________________


    #________________________________________________
    def changeBand(self,widget=None,codecs=None):
    #________________________________________________
        #reinit list of codecs
        if self.comboBand.get_text()=="" or self.EditedProtocol=="":
            return

        if codecs==None:
            GLOBAL.TEMP_PROTOCOLS[self.EditedProtocol]["codecs"]

        self.modelCodecs = Tools.Interface.gtk.ListStore(Tools.Interface.gobject.TYPE_STRING,Tools.Interface.gobject.TYPE_STRING)
        #check if codec is allowed
        for i in GLOBAL.CODECS[self.comboBand.get_text()]:
            str="allow"
            if codecs!=None and i not in codecs:
                str="disallow"
            iter=self.modelCodecs.insert(0,row=[i+"        ",str])

        self.listeCodecs.set_model(self.modelCodecs)
        self.listeCodecs.show()
    #________________________________________________


    #________________________________________________
    def update(self,parentmodel,iter):
    #________________________________________________

        #update label info
        self.EditedProtocol=parentmodel.get_value(iter,0).strip()
        self.labelInfo.set_text("Codecs for "+self.EditedProtocol+" protocol")

        #init comboBand and codec list
        if "bandwidth" in GLOBAL.TEMP_PROTOCOLS[self.EditedProtocol]:
            self.comboBand.set_text(GLOBAL.TEMP_PROTOCOLS[self.EditedProtocol]["bandwidth"])
        else:
            self.comboBand.set_text("high")

        self.changeBand(codecs=GLOBAL.TEMP_PROTOCOLS[self.EditedProtocol]["codecs"])
        self.createCodecsList()
        
        #Show list
        self.listeCodecs.set_model(self.modelCodecs)
        self.listeCodecs.set_sensitive(True)
        self.listeCodecs.show()
                
        self.show()
    #________________________________________________


    #________________________________________________
    def valide(self,*args):
    #________________________________________________
        GLOBAL.PROTOCOLS[self.EditedProtocol]["codecs"]=Tools.copy.deepcopy(GLOBAL.TEMP_PROTOCOLS[self.EditedProtocol]["codecs"])
        GLOBAL.PROTOCOLS[self.EditedProtocol]["bandwidth"]=self.comboBand.get_text()
        GLOBAL.TEMP_PROTOCOLS=Tools.copy.deepcopy(GLOBAL.PROTOCOLS)
        self.close()
    #________________________________________________

#________________________________________________________________________

                    
#----------------------------------------------------------------------------------------
#------------------------------------- FENETRE QUEUES  ----------------------------------
#----------------------------------------------------------------------------------------


#________________________________________________
class WQueuesCtrl(WindowCtrl):
#________________________________________________

    #________________________________________________
    def __init__(self,parent):
    #________________________________________________
        WindowCtrl.__init__(self,"Fen_queues",parent)

        #definition des handlers d'vnements
        self.handlers = {"on_btvalider_editqu_clicked" : self.valide,
                         "on_btannuler_editqu_clicked" : self.close,
                         }
        

        self.glade.signal_autoconnect(self.handlers)
        self.ASSOCIATED_MAIN_BT=GLOBAL.FEN_MAIN.btQueues

        #Init
        self.listeAgents=self.glade.get_widget("tree_agents_editq")
        self.modelAgents = Tools.Interface.gtk.TreeStore(Tools.Interface.gobject.TYPE_STRING)
        Tools.Build_liste(self.listeAgents,["Agents"])

        self.listeQueues=self.glade.get_widget("tree_queues_editq")
        self.modelQueues= Tools.Interface.gtk.TreeStore(Tools.Interface.gobject.TYPE_STRING)
        Tools.Build_liste(self.listeQueues,["Queues"])
                        

    #________________________________________________

    def update(self,*args):

        self.modelAgents.clear()
        for u in GLOBAL.ListeUsers:
            if u.protocol=="agent":
                iiter=self.modelAgents.insert_before(None,None)
                self.modelAgents.set_value(iiter,0,u.name)
                
        self.listeAgents.show()
        self.listeQueues.show()
        self.show()
        


#----------------------------------------------------------------------------------------
#------------------------------------- FENETRE EXPORT  ----------------------------------
#----------------------------------------------------------------------------------------


#________________________________________________
class WExportCtrl(WindowCtrl):
#________________________________________________

    #________________________________________________
    def __init__(self,parent):
    #________________________________________________
        WindowCtrl.__init__(self,"Fen_export",parent)

        #definition des handlers d'vnements
        self.handlers = {"on_btvalider_export_clicked" : self.valide,
                         "on_btannuler_export_clicked" : self.close,
                         "on_radio_export_local_toggled" : self.togglelocal,
                         "on_radio_export_remote_toggled" :self.toggleremote,
                         "on_radio_export_formatstd_toggled" : self.toggleformatstd,
                         "on_radio_export_formatdestar_toggled" : self.toggledestar
                         }
        

        self.glade.signal_autoconnect(self.handlers)
        self.ASSOCIATED_MAIN_BT=GLOBAL.FEN_MAIN.btExport

        #Init
        self.radio_local=self.glade.get_widget("radio_export_local")
        self.radio_remote=self.glade.get_widget("radio_export_remote")

        self.entry_host=self.glade.get_widget("entry_export_host")
        self.entry_port=self.glade.get_widget("entry_export_port")
        self.entry_passwd=self.glade.get_widget("entry_export_passwd")
        self.entry_remotedir=self.glade.get_widget("entry_export_remotedir")
        self.entry_localdir=self.glade.get_widget("entry_export_localdir")
        
        self.format="STANDARD"
        self.grouplocal=self.glade.get_widget("hbox_export_local")
        self.groupremote=Tools.GladeGroupContainer([self.glade.get_widget("hbox_export_host"),
                                                    self.glade.get_widget("hbox_export_port"),
                                                    self.glade.get_widget("hbox_export_passwd"),
                                                    self.glade.get_widget("hbox_export_remotedir")])
    #________________________________________________


    #________________________________________________
    def togglelocal(self,*args):
    #________________________________________________
        self.groupremote.set_sensitive(False)
        self.grouplocal.set_sensitive(True)
    #________________________________________________

    #________________________________________________
    def toggleremote(self,*args):
    #________________________________________________
        self.groupremote.set_sensitive(True)
        self.grouplocal.set_sensitive(False)
    #________________________________________________

    #________________________________________________
    def toggleformatstd(self,*args):
    #________________________________________________
        self.format="STANDARD"
    #________________________________________________

    #________________________________________________
    def toggledestar(self,*args):
    #________________________________________________
        self.format="DESTAR"
    #________________________________________________


    #________________________________________________
    def update(self,*args):
    #________________________________________________
        self.entry_host.set_text(GLOBAL.GLOBAL_PARAMS["lastremotehost"])
        self.entry_port.set_text(GLOBAL.GLOBAL_PARAMS["lastremoteport"])
        self.entry_remotedir.set_text("/etc/asterisk/")
        self.entry_localdir.set_text("/etc/asterisk/")
        self.radio_local.set_active(True)
        self.togglelocal()
        self.show()
    #________________________________________________

    #________________________________________________
    def valide(self,*args):
        """ exportation de la configuration existante """
    #________________________________________________
        if self.radio_local.get_active():
            saveconfigdir=GLOBAL.GLOBAL_PARAMS["configdir"]
            dir=self.entry_localdir.get_text().strip()
            if dir[len(dir)-1]!="/":
                dir+="/"
            GLOBAL.GLOBAL_PARAMS["configdir"]=dir
            result=Filesystem.save_asterisk_config()
            GLOBAL.GLOBAL_PARAMS["configdir"]=saveconfigdir
            if result:
                self.close()
        else:
            host=self.entry_host.get_text().strip()
            GLOBAL.GLOBAL_PARAMS["lastremotehost"]=host
            port=int(self.entry_port.get_text().strip())
            GLOBAL.GLOBAL_PARAMS["lastremoteport"]=str(port)
            passwd=self.entry_passwd.get_text().strip()
            remotedir=self.entry_remotedir.get_text().strip()
            if remotedir[len(remotedir)-1]!="/":
                remotedir+="/"
            client=Com.ACTOS_Client(host,port,passwd,remotedir)
            result=client.run(port=8007)
            del client
            if result!=True:
                Tools.debug("-- Error : "+result)
            else:
                Tools.Interface.DialogBox(texte="Transfer complete !")
                self.close()
    #________________________________________________


#----------------------------------------------------------------------------------------
#------------------------------------- FENETRE IMPORT  ----------------------------------
#----------------------------------------------------------------------------------------


#________________________________________________
class WImportCtrl(WindowCtrl):
#________________________________________________

    #________________________________________________
    def __init__(self,parent):
    #________________________________________________
        WindowCtrl.__init__(self,"Fen_import",parent)

        #definition des handlers d'vnements
        self.handlers = {"on_btvalider_import_clicked" : self.valide,
                         "on_btannuler_import_clicked" : self.close,
                         "on_radio_import_local_toggled" : self.togglelocal,
                         "on_radio_import_remote_toggled" :self.toggleremote,
                         "on_radio_import_formatstd_toggled" : self.toggleformatstd,
                         "on_radio_import_formatdestar_toggled" : self.toggledestar
                         }
        

        self.glade.signal_autoconnect(self.handlers)
        self.ASSOCIATED_MAIN_BT=GLOBAL.FEN_MAIN.btImport

        #Init
        self.radio_local=self.glade.get_widget("radio_import_local")
        self.radio_remote=self.glade.get_widget("radio_import_remote")

        self.entry_host=self.glade.get_widget("entry_import_host")
        self.entry_port=self.glade.get_widget("entry_import_port")
        self.entry_passwd=self.glade.get_widget("entry_import_passwd")
        self.entry_remotedir=self.glade.get_widget("entry_import_remotedir")
        self.entry_localdir=self.glade.get_widget("entry_import_localdir")
        
        self.format="STANDARD"
        self.grouplocal=self.glade.get_widget("hbox_import_local")
        self.groupremote=Tools.GladeGroupContainer([self.glade.get_widget("hbox_import_host"),
                                                    self.glade.get_widget("hbox_import_port"),
                                                    self.glade.get_widget("hbox_import_passwd"),
                                                    self.glade.get_widget("hbox_import_remotedir")])
    #________________________________________________


    #________________________________________________
    def togglelocal(self,*args):
    #________________________________________________
        self.groupremote.set_sensitive(False)
        self.grouplocal.set_sensitive(True)
    #________________________________________________

    #________________________________________________
    def toggleremote(self,*args):
    #________________________________________________
        self.groupremote.set_sensitive(True)
        self.grouplocal.set_sensitive(False)
    #________________________________________________

    #________________________________________________
    def toggleformatstd(self,*args):
    #________________________________________________
        self.format="STANDARD"
    #________________________________________________

    #________________________________________________
    def toggledestar(self,*args):
    #________________________________________________
        self.format="DESTAR"
    #________________________________________________

    #________________________________________________
    def update(self,*args):
    #________________________________________________
        self.entry_host.set_text(GLOBAL.GLOBAL_PARAMS["lastremotehost"])
        self.entry_port.set_text(GLOBAL.GLOBAL_PARAMS["lastremoteport"])
        self.entry_remotedir.set_text("/etc/asterisk/")
        self.entry_localdir.set_text("/etc/asterisk/")
        self.radio_local.set_active(True)
        self.togglelocal()
        self.show()
    #________________________________________________

    #________________________________________________
    def valide(self,*args):
        """ exportation de la configuration existante """
    #________________________________________________
        if self.radio_local.get_active():
            saveconfigdir=GLOBAL.GLOBAL_PARAMS["configdir"]
            dir=self.entry_localdir.get_text().strip()
            if dir[len(dir)-1]!="/":
                dir+="/"
            GLOBAL.GLOBAL_PARAMS["configdir"]=dir
            Filesystem.load_asterisk_config()
            GLOBAL.GLOBAL_PARAMS["configdir"]=saveconfigdir
            self.close()
        else:
            host=self.entry_host.get_text().strip()
            GLOBAL.GLOBAL_PARAMS["lastremotehost"]=host
            port=int(self.entry_port.get_text().strip())
            GLOBAL.GLOBAL_PARAMS["lastremoteport"]=str(port)
            passwd=self.entry_passwd.get_text().strip()
            remotedir=self.entry_remotedir.get_text().strip()
            if remotedir[len(remotedir)-1]!="/":
                remotedir+="/"
            client=Com.ACTOS_Client(host,port,passwd,remotedir)
            result=client.run(port=8007,mode="GET")
            del client
            if result!=True:
                Tools.debug("-- Error : "+result)
            else:
                Tools.Interface.DialogBox(texte="Importation complete !")
                self.close()
    #________________________________________________


#----------------------------------------------------------------------------------------
#------------------------------------- FENETRE OPTIONS ----------------------------------
#----------------------------------------------------------------------------------------


#________________________________________________
class WOptionsCtrl(WindowCtrl):
#________________________________________________

    #________________________________________________
    def __init__(self,parent):
    #________________________________________________
        WindowCtrl.__init__(self,"Fen_options",parent)

        #definition des handlers d'vnements
        self.handlers = {"on_btvalider_opt_clicked" : self.valide,
                         "on_btannuler_opt_clicked" : self.close,
                         "on_btcodecs_opt_clicked" : self.editCodecs,
                         "on_btjitter_opt_clicked" : self.editJitterBuffer,
                         "on_listeprotocols_opt_press_event" : self.select_protocol,
                         "on_btsupprprot_opt_clicked" : self.delete_protocol,
                         "on_btnouveauprot_opt_clicked" : self.add_protocol,
                         "on_entry_fichierprot_opt_changed" : (self.update_protocol,"fichier"),
                         "on_entry_portprot_opt_changed" : (self.update_protocol,"port"),
                         "on_entry_bindprot_opt_changed" : (self.update_protocol,"bindaddr"),
                         "on_entry_opt_tos_changed" : (self.update_protocol,"tos"),
                         "on_check_natsupport_opt_toggled" : self.toggle_natsupport
                         }
        

        self.glade.signal_autoconnect(self.handlers)
        self.ASSOCIATED_MAIN_BT=GLOBAL.FEN_MAIN.btOptions

        self.children["codecs"]=WCodecsCtrl(self)
        self.children["jitter"]=WJitterBufferCtrl(self)
        self.btCodecs=self.glade.get_widget("btCodecs_opt")
        self.btJitter=self.glade.get_widget("btJitter_opt")

        self.listeProtocols=self.glade.get_widget("ListeProtocols_opt")
        self.comboLanguage=Tools.GladeComboContainer(self.glade,"language_opt")
        self.comborecordformat=Tools.GladeComboContainer(self.glade,"agrecordformat_opt")
        #Init
        self.EditedProtocol=""
        self.mode="NORMAL"
        Tools.Build_liste(self.listeProtocols,["Nom du protocole"])
        self.comborecordformat.fill(["wav","gsm","wav49"])
    #________________________________________________


    #________________________________________________
    def editJitterBuffer(self,*args):
        """clic on 'codecs' button"""
    #________________________________________________
        #Rcupration de la slection
        selected,iter=self.listeProtocols.get_selection().get_selected()
        if iter==None:
            Tools.debug("-- Error : "+_("ERROR_NO_SELECTION"))
            return

        #Affiche la fentre d'dition en mode EDIT
        self.mode = "EDIT"
        model=self.listeProtocols.get_model()
        self.children["jitter"].update(model,iter)
    #________________________________________________


    #________________________________________________
    def editCodecs(self,*args):
        """clic on 'codecs' button"""
    #________________________________________________

        #Rcupration de la slection
        selected,iter=self.listeProtocols.get_selection().get_selected()
        if iter==None:
            Tools.debug("-- Error : "+_("ERROR_NO_SELECTION"))
            return 

        #Rend la liste grise
        self.listeProtocols.set_sensitive(False)

        #Affiche la fentre d'dition en mode EDIT
        self.mode = "EDIT"
        model=self.listeProtocols.get_model()
        self.children["codecs"].update(model,iter)
    #________________________________________________


    #________________________________________________
    def toggle_natsupport(self,*args):
        """clic on 'nat support'"""
    #________________________________________________
        isnatactive=self.glade.get_widget("check_natsupport_opt").get_active()

        for i in ["externip","localnet","localmask"]:
            self.glade.get_widget("entry_"+i+"_opt").set_sensitive(isnatactive)
    #________________________________________________


    #________________________________________________
    def initListeProtocols(self):
    #________________________________________________
        #Init protocols list
        model = Tools.Interface.gtk.ListStore(Tools.Interface.gobject.TYPE_STRING)
        #Fill the list
        dicp=Tools.GetProtocolsSupported()
        for i in dicp.keys():
            if i not in GLOBAL.PROTOCOLS.keys():
                #if protocol doesn't exists, create it
                Tools.InitDefaultsProtocol(i)
            iter=model.insert(0,row=[i])
                
        #Show list
        self.listeProtocols.set_model(model)
        self.listeProtocols.set_sensitive(True)
        self.listeProtocols.show()
    #________________________________________________


    #________________________________________________
    def update(self,*args):
    #________________________________________________

        self.ASSOCIATED_MAIN_BT.set_sensitive(False)

        #Init fields
        for rep in ["mod","varlib","agi","spool","run","log","etc"]:
            self.glade.get_widget('entry_'+rep+'dir').set_text(GLOBAL.ASTERISK_DIRECTORIES["ast"+rep+"dir"])

        self.glade.get_widget('entry_spooldir').set_text(GLOBAL.GLOBAL_PARAMS["configdir"])
        
        logtofile=(GLOBAL.GLOBAL_PARAMS["logtype"].find("file")>-1)
        logtoscreen=(GLOBAL.GLOBAL_PARAMS["logtype"].find("screen")>-1)
        saveoldconf=(GLOBAL.GLOBAL_PARAMS["saveoldconf"]=="yes")
        natsupport=(GLOBAL.GLOBAL_PARAMS["natsupport"]=="yes")
        cleanbeforeimport=(GLOBAL.GLOBAL_PARAMS["cleanbeforeimport"]=="yes")
        autoimport=(GLOBAL.GLOBAL_PARAMS["autoimport"]=="yes")
        autoexport=(GLOBAL.GLOBAL_PARAMS["autoexport"]=="yes")
        autosavemua=(GLOBAL.GLOBAL_PARAMS["autosavemostusedactions"]=="yes")
        language=GLOBAL.GLOBAL_PARAMS["language"]
        self.glade.get_widget('check_logtofile_opt').set_active(logtofile)
        self.glade.get_widget('check_logtoscreen_opt').set_active(logtoscreen)
        self.glade.get_widget('check_saveoldconf_opt').set_active(saveoldconf)
        self.glade.get_widget('check_clean_opt').set_active(cleanbeforeimport)
        self.glade.get_widget('check_natsupport_opt').set_active(natsupport)
        self.glade.get_widget('check_autoimport_opt').set_active(autoimport)
        self.glade.get_widget('check_autoexport_opt').set_active(autoexport)
        self.glade.get_widget('check_autosavemostusedactions_opt').set_active(autosavemua)
        self.comboLanguage.fill(Tools.GetLanguagesList(language))
        self.comboLanguage.set_text(Tools.GetCurrentLanguageName())

        #Init Agents options
        for p in ["autologoff","wrapuptime","recordformat"]:
            self.glade.get_widget('entry_ag'+p+'_opt').set_text(GLOBAL.AGENT_PARAMS[p])
        for p in ["ackcall","recordagentcalls","createlink"]:
            self.glade.get_widget('check_ag'+p+'_opt').set_active((GLOBAL.AGENT_PARAMS[p]!="no"))


        #Init Net setup
        for str in ["externip","localnet","localmask"]:
            self.glade.get_widget("entry_"+str+"_opt").set_text(GLOBAL.GLOBAL_PARAMS[str])

        #Desactive fields at startup
        for str in ["entry_fichierprot",
                    "entry_bindprot",
                    "entry_portprot"]:
            a=self.glade.get_widget(str)
            a.set_text("")
            a.set_sensitive(False)

        self.btJitter.set_sensitive(False)
        self.btCodecs.set_sensitive(False)
  
        #Fill TOS Combo
        comboTos=self.glade.get_widget("combo_opt_tos")
        comboTos.set_popdown_strings(GLOBAL.TOS_TYPES)
        comboTos.set_sensitive(False)


        #Save current datas
        GLOBAL.TEMP_PROTOCOLS=Tools.copy.deepcopy(GLOBAL.PROTOCOLS)

        self.initListeProtocols()
        
        self.toggle_natsupport()

        self.show()
    #________________________________________________


    #________________________________________________
    def update_protocol(self,widget,index_changed):
        """Modification du champ fichier_prot"""
    #________________________________________________
        #Exit if not protocol is selected
        if self.EditedProtocol=="":
            return

        # Update the attributes of selected protocol
        try:
            GLOBAL.TEMP_PROTOCOLS[self.EditedProtocol][index_changed]=widget.get_text()
        except:
            Tools.debug("-- Error : "+_("ERROR_MODIFY_PROTOCOL",[self.EditedProtocol,index_changed]))
    #________________________________________________


    #________________________________________________
    def add_protocol(self,*args):
        """Clic on 'new protocol'"""
    #________________________________________________
        Tools.debug("-- TODO --")
    #________________________________________________


    #________________________________________________
    def delete_protocol(self,*args):
        """Clic on 'delete protocol'"""
    #________________________________________________
        if self.EditedProtocol=="":
            Tools.debug("-- Error : "+_("ERROR_NO_SELECTION"))
            return

        #Demande de confirmation
        result=Tools.Interface.DialogBox(_("MSG_CONFIRM_DELETE_PROTOCOL",[self.EditedProtocol]),type_buttons=Tools.Interface.gtk.BUTTONS_YES_NO,type_dialog=Tools.Interface.gtk.MESSAGE_QUESTION)
        if result==Tools.Interface.gtk.RESPONSE_NO:
            return

        try:
            del(GLOBAL.TEMP_PROTOCOLS[self.EditedProtocol])
            splitouze=GLOBAL.GLOBAL_PARAMS["protocols"].split("##")
            chaine=""
            for s in splitouze:
                if s.split("/")[0]!=self.EditedProtocol:
                    chaine+=s+"##"
            GLOBAL.GLOBAL_PARAMS["protocols"]=chaine.rstrip("##")
            Tools.debug(GLOBAL.GLOBAL_PARAMS["protocols"])
        except:
            Tools.debug("-- Error : can't delete protocol")

        #Mise  jour de la fentre
        self.initListeProtocols()
    #________________________________________________


    #________________________________________________
    def select_protocol(self,liste,*args):
        """Clic on a row of protocols list"""
    #________________________________________________
        #Get selected row
        selected,iter=liste.get_selection().get_selected()
        if iter==None:
            return

        #Init fields with values
        model=liste.get_model()
        self.EditedProtocol=model.get_value(iter,0)
        self.glade.get_widget('entry_fichierprot').set_text(GLOBAL.TEMP_PROTOCOLS[self.EditedProtocol]["fichier"])
        self.glade.get_widget('entry_portprot').set_text(GLOBAL.TEMP_PROTOCOLS[self.EditedProtocol]["port"])
        self.glade.get_widget('entry_bindprot').set_text(GLOBAL.TEMP_PROTOCOLS[self.EditedProtocol]["bindaddr"])
        self.glade.get_widget("entry_opt_tos").set_text(GLOBAL.TEMP_PROTOCOLS[self.EditedProtocol]["tos"])
        for str in ["entry_fichierprot",
                    "entry_bindprot",
                    "entry_portprot",
                    "combo_opt_tos"]:
            self.glade.get_widget(str).set_sensitive(True)

        self.btCodecs.set_sensitive(True)
        #Active/Desactive Jitter Buffer button (Jitter buffer is possible only for iax protocol)
        self.btJitter.set_sensitive(self.EditedProtocol=="iax2")

    #________________________________________________

    
    #________________________________________________
    def valide(self,*args):
        """clic on 'valider' bouton"""
    #________________________________________________

        Tools.debug("## Saving parameters...")
        Tools.debug("# File actos.conf...")
        
        #Save configuration directory
        for rep in ["mod","varlib","agi","spool","run","log","etc"]:
            dir=self.glade.get_widget('entry_'+rep+'dir').get_text().strip()
            if dir[len(dir)-1]!="/":
                dir+="/"
            GLOBAL.ASTERISK_DIRECTORIES["ast"+rep+"dir"]=dir
            if rep=="etc":
                GLOBAL.GLOBAL_PARAMS["configdir"]=dir

        #Save language
        GLOBAL.GLOBAL_PARAMS["language"]=Tools.GetLanguageAbrevFromName(self.comboLanguage.get_text())

        #Save net setup
        for i in ["externip","localnet","localmask"]:
            GLOBAL.GLOBAL_PARAMS[i]=self.glade.get_widget("entry_"+i+"_opt").get_text().strip()

        #Save and activate log mode
        log=""
        if self.glade.get_widget("check_logtofile_opt").get_active()==True:
            log+="(file)"
        if self.glade.get_widget("check_logtoscreen_opt").get_active()==True:
            log+="(screen)"
        Tools.debug_init(log)

        #Save other options
        noyes={False:"no",True:"yes"}
        GLOBAL.GLOBAL_PARAMS["cleanbeforeimport"]=noyes[self.glade.get_widget("check_clean_opt").get_active()]
        for field in ["saveoldconf",
                      "natsupport",
                      "autoimport",
                      "autoexport",
                      "autosavemostusedactions"]:
            GLOBAL.GLOBAL_PARAMS[field]=noyes[self.glade.get_widget("check_"+field+"_opt").get_active()]

        Tools.debug(GLOBAL.GLOBAL_PARAMS)

        #Save Agents options
        for p in ["autologoff","wrapuptime","recordformat"]:
            GLOBAL.AGENT_PARAMS[p]=self.glade.get_widget('entry_ag'+p+'_opt').get_text()
        for p in ["ackcall","recordagentcalls","createlink"]:
            GLOBAL.AGENT_PARAMS[p]=noyes[self.glade.get_widget('check_ag'+p+'_opt').get_active()]
                                 

        #Write in file actos.conf
        Filesystem.save_actos_config()

        #Validate modifications on protocols
        Tools.debug("# Protocols...")
        GLOBAL.PROTOCOLS=Tools.copy.deepcopy(GLOBAL.TEMP_PROTOCOLS)
        Tools.debug(GLOBAL.PROTOCOLS)
        
        self.close()
        Tools.debug("## Done")
    #________________________________________________


#_______________________________________________________________


#----------------------------------------------------------------------------------------
#------------------------------------- FENETRE CONFROOMS ----------------------------------
#----------------------------------------------------------------------------------------


#________________________________________________
class WConfRoomsCtrl(WindowCtrl):
#________________________________________________

    #________________________________________________
    def __init__(self,parent):
    #________________________________________________
        WindowCtrl.__init__(self,"Fen_confrooms",parent)

        #definition des handlers d'vnements
        self.handlers = {"on_btvalider_room_clicked" : self.close,
                         "on_btannuler_room_clicked" : self.annuler,
                         "on_btnew_room_clicked" : self.newroom,
                         "on_btremove_room_clicked" : self.removeroom,
                         "on_btedit_room_clicked" : self.editroom
                         }
        

        self.glade.signal_autoconnect(self.handlers)
        self.ASSOCIATED_MAIN_BT=GLOBAL.FEN_MAIN.btConfRooms

        self.children["editRoom"]=WEditRoomCtrl(self)


        self.listeRooms=self.glade.get_widget("liste_confrooms")
        self.SAVE_CONFROOMS={}
       
        #Init
        Tools.Build_liste(self.listeRooms,["Room name","Password (PIN)"])
        self.mode="SELECT"
    #________________________________________________

    def annuler(self,*args):
        GLOBAL.CONFROOMS=Tools.copy.deepcopy(self.SAVE_CONFROOMS)
        self.close()

    #________________________________________________
    def newroom(self,*args):
    #________________________________________________
        self.children["editRoom"].update(None)
    #________________________________________________
	
    #________________________________________________
    def editroom(self,*args):
    #________________________________________________
	#Get iterator on selected item
        selected,iter=self.listeRooms.get_selection().get_selected()
        if iter==None:
            Tools.debug("-- Error : "+_("ERROR_NO_SELECTION"))
            return
        self.children["editRoom"].update(str(self.modelRooms.get_value(iter,0)),str(self.modelRooms.get_value(iter,1)))
    #________________________________________________
	
    #________________________________________________
    def removeroom(self,*args):
    #________________________________________________
	    #Get iterator on selected item
        selected,iter=self.listeRooms.get_selection().get_selected()
        if iter==None:
            Tools.debug("-- Error : "+_("ERROR_NO_SELECTION"))
            return

        room=str(self.modelRooms.get_value(iter,0))

        #Select row
        newsel=self.modelRooms.iter_next(iter)
        
        try:
            del(GLOBAL.CONFROOMS[room])
            self.modelRooms.remove(iter)
        except:
            pass

        if newsel==None:
            newsel=self.modelRooms.get_iter_root()
        
        if newsel!=None:
            self.listeRooms.get_selection().select_iter(newsel)

        self.listeRooms.set_model(self.modelRooms)
        self.listeRooms.show()
   	#________________________________________________

    #________________________________________________
    def update_list(self,*args):
    #________________________________________________
       #fill list
       self.modelRooms=Tools.Interface.gtk.ListStore(Tools.Interface.gobject.TYPE_STRING,Tools.Interface.gobject.TYPE_STRING)
       for v in GLOBAL.CONFROOMS.keys():
           irow=[v,GLOBAL.CONFROOMS[v]]
           iter=self.modelRooms.insert(0,row=irow)
       self.listeRooms.set_model(self.modelRooms)
    #________________________________________________


    def update(self,*args):
        self.SAVE_CONFROOMS=Tools.copy.deepcopy(GLOBAL.CONFROOMS)
        self.update_list()
        self.listeRooms.set_model(self.modelRooms)
        self.show()

#_______________________________________________________________




#----------------------------------------------------------------------------------------
#------------------------------------- FENETRE EDITROOM ---------------------------------
#----------------------------------------------------------------------------------------


#________________________________________________
class WEditRoomCtrl(WindowCtrl):
#________________________________________________

    #________________________________________________
    def __init__(self,parent):
    #________________________________________________
        WindowCtrl.__init__(self,"Fen_editRoom",parent)
        self.handlers = {"on_btannuler_editroom_clicked" : self.close,
                         "on_btvalider_editroom_clicked" : self.valide
                        }
        self.glade.signal_autoconnect(self.handlers)

        #Init
        self.entryName=self.glade.get_widget("entry_name_editroom")
        self.entryPass=self.glade.get_widget("entry_pass_editroom")
        
        self.mode="CREATE"
        
    #___________________________________________________________


    #________________________________________________
    def update_parent(self,*args):
    #________________________________________________
        self.parent.update_list()
        self.parent.show()
    #________________________________________________

    #________________________________________________
    def update(self,rname=None,passw=None):
    #________________________________________________
        #update fields
        self.old_name=rname
        if rname==None:
            self.mode="CREATE"
            self.entryName.set_text("")
            self.entryPass.set_text("")
        else:
            self.mode="EDIT"
            self.entryName.set_text(rname)
            self.entryPass.set_text(passw)

        self.entryName.grab_focus()
        self.show()
    #________________________________________________

    #________________________________________________
    def valide(self,*args):
    #________________________________________________
        #get values
        var=Tools.string.upper(self.entryName.get_text().strip())
        value=self.entryPass.get_text().strip()
        
        if self.mode=="CREATE":
            if var in GLOBAL.CONFROOMS.keys():
                Tools.debug("-- Error :"+_("ERROR_VAR_ALREADY_EXISTS"))
                return
        elif self.mode=="EDIT":
            #delete old key
            if self.old_name in GLOBAL.CONFROOMS.keys():
                del(GLOBAL.CONFROOMS[self.old_name])

        #create new key
        GLOBAL.CONFROOMS[var]=value
        
        self.close()
    #________________________________________________

#________________________________________________



#----------------------------------------------------------------------------------------
#-------------------------------------- FENETRE PRINCIPALE ------------------------------
#----------------------------------------------------------------------------------------


#________________________________________________
class WMainCtrl(WindowCtrl):
#________________________________________________

    #________________________________________________
    def __init__(self,parent):
    #________________________________________________
        WindowCtrl.__init__(self,"Fen_main",parent)
        self.handlers = {"gtk_main_quit" : self.verifyandclose,
                         "on_fen_main_delete_event" : self.verifyandclose,
                         "on_btusers_main_clicked" : (self.create_child,"users"),
                         "on_btcontext_main_clicked" : (self.create_child,"contexts"),
                         "on_btoptions_main_clicked" : (self.create_child,"options"),
                         "on_btqueues_main_clicked" :  (self.create_child,"queues"),
                         "on_btvar_main_clicked" : (self.create_child,"variables"),
                         "on_btconnect_main_clicked" : self.twistedtransfer,
                         "on_btimport_main_clicked" : (self.create_child,"import"),
                         "on_btexport_main_clicked" : (self.create_child,"export"),
                         "on_btconfrooms_main_clicked" : (self.create_child,"confrooms")
                         
                         }
        self.glade.signal_autoconnect(self.handlers)


        GLOBAL.FEN_MAIN=self
        self.btUsers=self.glade.get_widget("btUsers_main")
        self.btContextes=self.glade.get_widget("btContextes_main")
        self.btVariables=self.glade.get_widget("btVariables_main")
        self.btOptions=self.glade.get_widget("btOptions_main")
        self.btQueues=self.glade.get_widget("btQueues_main")
        self.btExport=self.glade.get_widget("btExport_main")
        self.btImport=self.glade.get_widget("btImport_main")
        self.btConfRooms=self.glade.get_widget("btConfRooms_main")
                
        #Init
        self.show()
    #________________________________________________

    def verifyandclose(self,*args):
        if GLOBAL.GLOBAL_PARAMS["autoexport"]!="yes":
            message=_("MSG_CONFIRM_QUIT")
            result=Tools.Interface.DialogBox(message,type_buttons=Tools.Interface.gtk.BUTTONS_YES_NO,type_dialog=Tools.Interface.gtk.MESSAGE_QUESTION)
            if result==Tools.Interface.gtk.RESPONSE_YES:
                self.close()
        else:
            self.close()


    #________________________________________________
    def twistedtransfer(self,*args):
    #________________________________________________
        client=Actos_twisted.ACTOS_Client()
        client.run()
    #________________________________________________

    #________________________________________________
    def editVar(self,*args):
    #________________________________________________
        Tools.debug("-- TODO --")
    #________________________________________________

    #________________________________________________
    def create_child(self,widget=0,name="NULL"):
    #________________________________________________
        if name=="variables":
            if not name in self.children:
                self.children[name]=WVariablesCtrl(self)
            self.children[name].update(mode="LIST")
            return
        
        if not name in self.children:
            if name=="users":
                self.children[name]=WUsersCtrl(self)
            elif name=="contexts":
                self.children[name]=WContextsCtrl(self)
            elif name=="options":
                self.children[name]=WOptionsCtrl(self)
            elif name=="queues":
                self.children[name]=WQueuesCtrl(self)
            elif name=="export":
                self.children[name]=WExportCtrl(self)
            elif name=="import":
                self.children[name]=WImportCtrl(self)
            elif name=="confrooms":
            	self.children[name]=WConfRoomsCtrl(self)
            
            else:
                Tools.debug("-- Error : [create_child] can't create "+name+" window !")
                return

        self.children[name].update()
    #________________________________________________
        


    #________________________________________________
    def close(self,*args):
    #________________________________________________
        WindowCtrl.close(self,kill=True)
        self.parent.close()
    #________________________________________________


#_______________________________________________________________



#_______________________________________________________________
# IF SOMEONE LAUNCH THIS SCRIPT.... 
#_______________________________________________________________
if __name__ == "__main__":
    Tools.Interface.AboutMessage()
    print "Please launch actos.py\n"
#_______________________________________________________________
